import "../../FabricInit";
import * as React from "react";
import { IBaseProps } from "office-ui-fabric-react/lib/Utilities";

export interface IAccordion {
    /**
     * Expands the item with the given item key
     * @param itemKey the item key to be expanded
     */
    expand(itemKey: string): void;
    /**
     * Collapses the item with the given item key
     * @param itemKey the item key to be collapsed
     */
    collapse(itemKey: string): void;
}

export interface IAccordionProps extends IBaseProps {
    /**
     * The list of accordion items to display.
     */
    items: IAccordionItem[];
    /**
     * Whether or not all items start out expanded.
     * @default false
     */
    initiallyExpandAll?: boolean;
    /**
     * Fucntion to be called when an Accordion item is being expanded/collapsed.
     */
    onItemChanged?: (itemKey: string, expanding: bool) => void;
    /**
     * If provided, additional class name to provide on the root Accordion element.
     */
    className?: string;
}

export interface IAccordionItem {
    /**
     * The key for the accordion item. Should be unique.
     */
    key: string;
    /**
     * The text that is displayed for the accordion item, will be value of key if blank
     */
    title?: string
    /**
     * The content that will be rendered within the accordion item.
     */
    onRenderContent: () => JSX.Element;
}