import * as React from "react";
import { IReadonlyObservableValue } from '../../Core/Observable';
export interface ICheckboxProps {

    /**
     * Additional classes to be applied to component
     */
    className?: string;

    /**
     * Id of another element which labels this one for screen reader users.
     * Defaults to this checkbox's label element.
     */
    ariaLabelledBy?: string;

    /**
     * Aria Label for the Checkbox for screen reader users.
     */
    ariaLabel?: string;

    /**
     * The checked state of the checkbox.
     * @default false 
     */
    checked?: IReadonlyObservableValue<boolean> | boolean;

    /**
     * Optional class name to add to the checkbox element.
     */
    className?: string;

    /**
     * If true, the checkbox cannot be interacted with.
     */
    disabled?: boolean;

    /**
     * Set to true if you don't want the checkbox's focus managed by a FocusZone.
     */
    excludeFocusZone?: boolean;

    /**
     * Set to true to omit setting the tab index to 0.
     */
    excludeTabStop?: boolean;

    /**
     * An id to pass to the label element.
     */
    labelId?: string;

    /**
     * The aria role to put on the checkbox element.
     * @default checkbox
     */
    role?: string;

    /**
     * An optional label to render next to the checkbox.
     */
    label?: string;

    /**
     * An optional callback to handle checked state change events.
     */
    onChange?: (event: React.MouseEvent<HTMLElement> | React.KeyboardEvent<HTMLElement>, checked: boolean) => void;
}