import "../../FabricInit";
import * as React from "react";
import { IBaseProps } from "office-ui-fabric-react/lib/Utilities";
import { IComboBox, IComboBoxOption } from "office-ui-fabric-react/lib/ComboBox";
import { IComboBoxOption as IFabricComboBoxOption } from "office-ui-fabric-react/lib/ComboBox";

export interface IComboBoxOption extends IFabricComboBoxOption
{

}

export interface IComboBoxProps extends IBaseProps {

    /**
     * The string that describes the combo box. It can either be shown alongside the combo box or as a tooltip.
     */
    label?: string;


    /**
     * Whether the ComboBox is free form, meaning that the user input is not bound to provided options. Defaults to false.
     */
    allowFreeform?: boolean;

    /**
     * Optional callback to access the IComboBox interface. Use this instead of ref for accessing
     * the public methods and properties of the component.
     */
    componentRef?: (component: IComboBox | null) => void;

    /**
     * Whether the ComboBox auto completes. As the user is inputing text, it will be suggested potential matches from the list of options. If
     * the combo box is expanded, this will also scroll to the suggested option, and give it a selected style.
     *
     * @default true
     */
    autoComplete?: boolean;

    /**
     * If the control is disabled or not
     */
    disabled?: boolean;

    /**
     * Collection of options for this ComboBox
     */
    options: IComboBoxOption[];

    /**
     * Callback issues when either:
     * 1) the selected option changes
     * 2) a manually edited value is submitted. In this case there may not be a matched option if allowFreeform is also true
     *    (and hence only value would be true, the other parameter would be null in this case)
     */
    onChanged?: (option?: IComboBoxOption, index?: number, value?: string, submitPendingValueEvent?: any) => void;

    /**
     * Callback issued when the user changes the pending value in ComboBox
     */
    onPendingValueChanged?: (option?: IComboBoxOption, index?: number, value?: string) => void;

    /**
     * Function that gets invoked when the ComboBox menu is launched
     */
    onMenuOpen?: () => void;

    /**
     * Function that gets invoked when the ComboBox menu is dismissed
     */
    onMenuDismissed?: () => void;

    /**
     * Callback issued when the options should be resolved, if they have been updated or
     * if they need to be passed in the first time
     */
    onResolveOptions?: (options: IComboBoxOption[]) => IComboBoxOption[] | PromiseLike<IComboBoxOption[]>;

    /**
     * Callback issued when the ComboBox requests the list to scroll to a specific element
     */
    onScrollToItem?: (itemIndex: number) => void;

    /**
     * Value to show in the input, does not have to map to a combobox option
     */
    text?: string;

    /**
     * When options are scrollable the selected option is positioned at the top of the callout when it is opened (unless it has reached the end of the scrollbar).
     * @default false;
     */
    scrollSelectedToTop?: boolean;

    /**
     * Add additional content below the callout list.
     */
    onRenderLowerContent?: IRenderFunction<IComboBoxProps>;

    /**
     * Optional mode indicates if multi-choice selections is allowed.  Default to false
     */
    multiSelect?: boolean;

    /**
     * Descriptive label for the ISelectableDroppableText Error Message
     */
    errorMessage?: string;

    /**
     * Whether or not the ISelectableDroppableText is required.
     */
    required?: boolean;

    /**
     * Sets the 'aria-hidden' attribute on the ComboBox's button element instructing screen readers how to handle the element. This element is hidden by default because all functionality is handled by the input element and the arrow button is only meant to be decorative.
     * @default true
     */
    isButtonAriaHidden?: boolean;

    /**
    * Aria Label for the ISelectableDroppableText for screen reader users.
    */
    ariaLabel?: string;

    /**
    * Id of the ISelectableDroppableText
    */
    id?: string;

    /**
     * The key(s) that will be initially used to set a selected item.
     */
    defaultSelectedKey?: string | number | string[] | number[];

    /**
     * The key(s) of the selected item. If you provide this, you must maintain selection
     * state by observing onChange events and passing a new value in when changed.
     */
    selectedKey?: string | number | string[] | number[];

    /**
     * Optional custom renderer for the ISelectableDroppableText container
     */
    onRenderContainer?: IRenderFunction<ISelectableDroppableTextProps<T>>;

    /**
     * Optional custom renderer for the ISelectableDroppableText list
     */
    onRenderList?: IRenderFunction<ISelectableDroppableTextProps<T>>;

    /**
     * Optional custom renderer for the ISelectableDroppableText options
     */
    onRenderItem?: IRenderFunction<ISelectableOption>;

    /**
     * Optional custom renderer for the ISelectableDroppableText option content
     */
    onRenderOption?: IRenderFunction<ISelectableOption>;
}