import * as tslib_1 from "tslib";
import "../../FabricInit";
import * as React from "react";
import { DefaultButton, PrimaryButton } from "office-ui-fabric-react/lib/Button";
import { Icon } from "office-ui-fabric-react/lib/Icon";
import { TooltipHost } from '../../Tooltip';
import { BaseComponent, css, getId, getNativeProps, buttonProperties } from "office-ui-fabric-react/lib/Utilities";
import { VssContextualMenu } from '../../VssContextualMenu';
import { VssIcon } from "../../VssIcon";
import "./ContextualMenuButton.css";
/**
 * Component that contains a button which, when activated, will show a contextual menu with the items provided
*/
var ContextualMenuButton = /** @class */ (function (_super) {
    tslib_1.__extends(ContextualMenuButton, _super);
    function ContextualMenuButton(props) {
        var _this = _super.call(this, props) || this;
        _this._onClick = function (event) {
            var _a = _this.props, onClick = _a.onClick, useTargetPoint = _a.useTargetPoint;
            if (!onClick || !onClick(event)) {
                event.stopPropagation();
            }
            _this.setState({
                items: _this.props.getItems ? _this.props.getItems() : [],
            });
            if (event.clientX && useTargetPoint) {
                _this.setState({
                    isOpen: true,
                    providers: _this.props.getItemProviders ? _this.props.getItemProviders() : [],
                    useTargetPoint: true,
                    target: { x: event.clientX, y: event.clientY },
                });
            }
            else {
                // if this is a keydown event, we don't want to use target point we want to use target element
                _this.setState({
                    isOpen: true,
                    providers: _this.props.getItemProviders ? _this.props.getItemProviders() : [],
                    useTargetPoint: false,
                });
            }
        };
        _this._onDismiss = function () {
            if (_this._isMounted) {
                if (_this.props.onDismiss) {
                    _this.props.onDismiss();
                }
                _this.setState({ target: null, isOpen: false, useTargetPoint: false, items: undefined, providers: undefined }, function () { return _this.focus(); });
            }
        };
        if (_this.props.isOpen) {
            _this.state = { isOpen: true, providers: _this.props.getItemProviders ? _this.props.getItemProviders() : [], items: _this.props.getItems ? _this.props.getItems() : [] };
        }
        else {
            _this.state = { items: undefined, providers: undefined, isOpen: false };
        }
        _this._contextMenuExpandedId = getId("context-menu-expanded");
        _this._warnDeprecations({
            'targetPoint': 'target',
            'useTargetPoint': 'target',
        });
        return _this;
    }
    ContextualMenuButton.prototype.focus = function () {
        if (this._button && this._button.focus) {
            this._button.focus();
        }
    };
    ContextualMenuButton.prototype.componentWillReceiveProps = function (nextProps) {
        if (nextProps.isOpen && !this.state.items) {
            this.setState({
                items: this.props.getItems ? this.props.getItems() : [],
                providers: this.props.getItemProviders ? this.props.getItemProviders() : []
            });
        }
    };
    ContextualMenuButton.prototype.componentDidMount = function () {
        _super.prototype.componentDidMount.call(this);
        this._isMounted = true;
    };
    ContextualMenuButton.prototype.componentWillUnmount = function () {
        this._isMounted = false;
        _super.prototype.componentWillUnmount.call(this);
    };
    ContextualMenuButton.prototype.render = function () {
        var _this = this;
        var _a = this.props, className = _a.className, ctaButton = _a.ctaButton, directionalHint = _a.directionalHint, iconProps = _a.iconProps, title = _a.title, allocateSpaceWhileHidden = _a.allocateSpaceWhileHidden;
        var isOpen = this.state.isOpen || this.props.isOpen || false;
        var showTooltip = this.props.showTooltip && !isOpen;
        var target = this.state.target || this.props.target || this.props.targetPoint;
        var isTargetAPoint = this.state.useTargetPoint || this.props.useTargetPoint;
        if (isTargetAPoint === undefined) {
            // See target is a point
            isTargetAPoint = target && target.x !== undefined && target.y !== undefined;
        }
        var excludePropNames = ["title"];
        var buttonProps = tslib_1.__assign({}, getNativeProps(this.props, buttonProperties, excludePropNames), { componentRef: function (button) { return _this._button = button; }, onClick: this._onClick, ariaLabel: title, className: css("vss-ContextualMenuButton", "contextual-menu-button", // back compat
            className, {
                "open": isOpen,
                "space-allocated": allocateSpaceWhileHidden,
                "space-unallocated": !allocateSpaceWhileHidden
            }), "aria-haspopup": true, "aria-expanded": isOpen, "aria-controls": isOpen ? this._contextMenuExpandedId : null });
        var buttonChildren;
        if (iconProps) {
            buttonChildren = React.createElement(Icon, tslib_1.__assign({}, VssIcon.getIconProps(iconProps)));
        }
        else {
            buttonChildren = null;
        }
        var button;
        if (ctaButton) {
            button = React.createElement(PrimaryButton, tslib_1.__assign({}, buttonProps), buttonChildren);
        }
        else {
            button = React.createElement(DefaultButton, tslib_1.__assign({}, buttonProps), buttonChildren);
        }
        if (title && showTooltip) {
            button = React.createElement(TooltipHost, { content: title, tooltipProps: { directionalHint: 5 /* bottomCenter */ } }, button);
        }
        return (React.createElement("div", { className: ContextualMenuButton.ButtonContainerClassName, ref: ContextualMenuButton.buttonRef },
            button,
            isOpen &&
                this.state.items &&
                React.createElement(VssContextualMenu, { providers: this.state.providers, items: this.state.items, onDismiss: this._onDismiss, shouldFocusOnMount: true, gapSpace: isTargetAPoint ? 5 : 0, directionalHint: directionalHint, target: target || this.refs[ContextualMenuButton.buttonRef], ariaLabel: title })));
    };
    ContextualMenuButton.ButtonContainerClassName = 'vss-ContextualMenu--container';
    ContextualMenuButton.buttonRef = "button";
    return ContextualMenuButton;
}(BaseComponent));
export { ContextualMenuButton };
