import * as tslib_1 from "tslib";
import "../../FabricInit";
import { BaseComponent } from 'office-ui-fabric-react/lib/Utilities';
import { FILTER_CHANGE_EVENT } from '../../Utilities/Filter';
var FilterBarItem = /** @class */ (function (_super) {
    tslib_1.__extends(FilterBarItem, _super);
    function FilterBarItem() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._onFilterChanged = function (changedState) {
            if (changedState.hasOwnProperty(_this.props.filterItemKey)) {
                _this.onFilterChanged(changedState[_this.props.filterItemKey]);
            }
        };
        return _this;
    }
    FilterBarItem.prototype.componentWillMount = function () {
        var itemState = this.props.filter.getFilterItemState(this.props.filterItemKey);
        this.setState({ value: itemState && itemState.value, operator: itemState && itemState.operator });
        var throttleWait = this.getThrottleWait();
        if (throttleWait) {
            this._throttledSetFilterValue = this._async.debounce(this._setFilterValue, throttleWait, { leading: false, trailing: true });
        }
        else {
            this._throttledSetFilterValue = this._setFilterValue.bind(this);
        }
    };
    /**
     * Setting state when component state is different from filter item state
     * Needed for handling scenarios when filter object's setState method is called with suppressChangeEvent=true
     * In such cases, FILTER_CHANGE_EVENT is not triggered and thus component state is not updated
     * And then on re-rendering component renders with old state.
     * @param nextProps
     */
    FilterBarItem.prototype.componentWillReceiveProps = function (nextProps) {
        var itemState = nextProps.filter.getFilterItemState(nextProps.filterItemKey);
        // If we are throttling the changes from the filter item's state, there is a chance that the props are out of sync with the state.
        // Therefore, we only want to react to new changes in props if we are not throttling or if the setKey for the component has changed
        // (indicating we want to completely reset state, reacting to the props)
        if ((this.props.setKey !== nextProps.setKey) || (this.getThrottleWait() === 0 && !nextProps.filter.filterItemStatesAreEqual(nextProps.filterItemKey, itemState, this.state))) {
            this.setState({ value: itemState && itemState.value, operator: itemState && itemState.operator });
        }
    };
    FilterBarItem.prototype.componentDidMount = function () {
        this._events.on(this.props.filter, FILTER_CHANGE_EVENT, this._onFilterChanged);
    };
    FilterBarItem.prototype.componentWillUnmount = function () {
        this._events.off(this.props.filter, FILTER_CHANGE_EVENT, this._onFilterChanged);
    };
    FilterBarItem.prototype.onFilterChanged = function (filterState) {
        this.setState({
            value: filterState && filterState.value,
            operator: filterState && filterState.operator
        });
    };
    FilterBarItem.prototype.getThrottleWait = function () {
        return 0;
    };
    FilterBarItem.prototype.setFilterValue = function (filterState) {
        if (this.getThrottleWait()) {
            this.setState({
                value: filterState && filterState.value,
                operator: filterState && filterState.operator
            });
        }
        this._throttledSetFilterValue(filterState);
    };
    FilterBarItem.prototype._setFilterValue = function (filterState) {
        this.props.filter.setFilterItemState(this.props.filterItemKey, filterState);
    };
    return FilterBarItem;
}(BaseComponent));
export { FilterBarItem };
