import * as tslib_1 from "tslib";
import "../../FabricInit";
import * as React from "react";
import { BaseComponent, css } from "office-ui-fabric-react/lib/Utilities";
import { OverflowSet } from 'office-ui-fabric-react/lib/OverflowSet';
import { CommandBarButton, PrimaryButton } from 'office-ui-fabric-react/lib/Button';
import { TooltipHost } from '../../Tooltip';
import { FILTER_CHANGE_EVENT } from '../../Utilities/Filter';
var HeaderCommandBar = /** @class */ (function (_super) {
    tslib_1.__extends(HeaderCommandBar, _super);
    function HeaderCommandBar() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._onRenderItem = function (item) {
            var buttonElement = _this._renderButton(item);
            if (item.tooltipProps) {
                return (React.createElement(TooltipHost, tslib_1.__assign({}, item.tooltipProps), buttonElement));
            }
            return buttonElement;
        };
        _this._onRenderOverflowButton = function (data) {
            return (React.createElement(HeaderCommandBarOverflowMenuButton, { data: data }));
        };
        return _this;
    }
    HeaderCommandBar.prototype.render = function () {
        var _a = this.props, contextualMenuItems = _a.contextualMenuItems, viewMenuItems = _a.viewMenuItems;
        var primaryItems = contextualMenuItems ? contextualMenuItems.slice(0, 3) : [];
        var overflowItems = contextualMenuItems ? contextualMenuItems.slice(3) : [];
        return (React.createElement("div", { className: css("vss-Header-CommandBar flex-row", this.props.className) },
            viewMenuItems && viewMenuItems.length > 0 &&
                React.createElement(OverflowSet, { className: "vss-Header-ViewActions", items: viewMenuItems, overflowItems: [], onRenderItem: this._onRenderItem, onRenderOverflowButton: undefined }),
            contextualMenuItems && contextualMenuItems.length > 0 &&
                React.createElement(OverflowSet, { className: "vss-Header-Commands", items: primaryItems, overflowItems: overflowItems, onRenderItem: this._onRenderItem, onRenderOverflowButton: this._onRenderOverflowButton })));
    };
    HeaderCommandBar.prototype._renderButton = function (item) {
        var buttonProps = tslib_1.__assign({}, item);
        if (item.isPrimary) {
            return (React.createElement(PrimaryButton, tslib_1.__assign({}, buttonProps, { text: item.name, menuProps: item.subMenuProps, itemType: item.itemType && item.itemType.toString() })));
        }
        return (React.createElement(CommandBarButton, tslib_1.__assign({}, buttonProps, { text: item.name, menuProps: item.subMenuProps, itemType: item.itemType && item.itemType.toString() })));
    };
    return HeaderCommandBar;
}(BaseComponent));
export { HeaderCommandBar };
var HeaderCommandBarOverflowMenuButton = /** @class */ (function (_super) {
    tslib_1.__extends(HeaderCommandBarOverflowMenuButton, _super);
    function HeaderCommandBarOverflowMenuButton(props) {
        var _this = _super.call(this, props) || this;
        _this._onMenuOpen = function () {
            _this.setState({ isMenuOpen: true });
        };
        _this._onMenuDismiss = function () {
            _this.setState({ isMenuOpen: false });
        };
        _this.state = { isMenuOpen: false };
        return _this;
    }
    HeaderCommandBarOverflowMenuButton.prototype.render = function () {
        return React.createElement(CommandBarButton, { className: this.state.isMenuOpen && "menu-is-open", menuProps: {
                items: this.props.data,
                onMenuOpened: this._onMenuOpen,
                onMenuDismissed: this._onMenuDismiss
            }, menuIconProps: { iconName: "more" } });
    };
    return HeaderCommandBarOverflowMenuButton;
}(BaseComponent));
var HeaderCommandBarWithFilter = /** @class */ (function (_super) {
    tslib_1.__extends(HeaderCommandBarWithFilter, _super);
    function HeaderCommandBarWithFilter(props) {
        var _this = _super.call(this, props) || this;
        _this.onFilterClicked = function () {
            _this.props.filterToggled.value = !_this.props.filterToggled.value;
        };
        _this.onFilterChanged = function () {
            var hasChanges = _this.props.filter.hasChangesToReset();
            if (hasChanges !== _this.state.filterHasChanges) {
                _this.setState({
                    filterHasChanges: hasChanges
                });
            }
        };
        _this.state = { filterHasChanges: _this.props.filter.hasChangesToReset() };
        return _this;
    }
    HeaderCommandBarWithFilter.prototype.componentDidMount = function () {
        this.props.filter.subscribe(this.onFilterChanged, FILTER_CHANGE_EVENT);
    };
    HeaderCommandBarWithFilter.prototype.componentWillUnmount = function () {
        this.props.filter.unsubscribe(this.onFilterChanged, FILTER_CHANGE_EVENT);
    };
    HeaderCommandBarWithFilter.prototype.render = function () {
        var filterItem = {
            key: "filter",
            iconProps: {
                iconName: this.state.filterHasChanges ? "FilterSolid" : "Filter"
            },
            onClick: this.onFilterClicked
        };
        var viewMenuItems = this.props.viewMenuItems ? this.props.viewMenuItems.slice() : [];
        viewMenuItems.push(filterItem);
        return React.createElement(HeaderCommandBar, tslib_1.__assign({}, this.props, { viewMenuItems: viewMenuItems }));
    };
    return HeaderCommandBarWithFilter;
}(React.Component));
export { HeaderCommandBarWithFilter };
