import * as tslib_1 from "tslib";
import "../../FabricInit";
import * as React from 'react';
import * as VSSUIResources from '../../VSS.Resources.VSSUI';
import { BaseComponent, css } from 'office-ui-fabric-react/lib/Utilities';
import { childrenOfType } from '../../Utilities/ChildrenUtils';
import { HubViewOptionKeys } from '../../Utilities/HubViewState';
import { ObservableArray, ObservableLike } from '../../Core/Observable';
import { VIEW_OPTIONS_CHANGE_EVENT } from '../../Utilities/ViewOptions';
import { Fabric } from 'office-ui-fabric-react/lib/Fabric';
import { PivotBar } from '../../PivotBar';
import { HubHeader } from '../../HubHeader';
import { Splitter } from '../../Splitter';
import "./Hub.css";
var Hub = /** @class */ (function (_super) {
    tslib_1.__extends(Hub, _super);
    function Hub(props) {
        var _this = _super.call(this, props) || this;
        _this._onViewActionObservableChanged = function () {
            _this._viewActionsObservable.value = _this._underlyingViewActionsObservable.value.concat([_this._getFullScreenViewAction()]);
        };
        _this._onSelectedPivotChanged = function (newPivotKey) {
            _this.setState({
                selectedPivot: newPivotKey
            });
        };
        _this._onPivotClicked = function (ev, pivotKey) {
            if (pivotKey !== _this.props.hubViewState.selectedPivot.value) {
                if (!_this.props.hubViewState.canSwitchPivots(pivotKey)) {
                    var oldValue = _this.props.hubViewState.selectedPivot.value;
                    _this.pivotBar.selectPivot(oldValue);
                }
                else {
                    _this.props.hubViewState.selectedPivot.value = pivotKey;
                }
            }
        };
        // Do a one-time evaluation to see if the page was rendered in no-chrome mode
        // in which case we won't give the full screen options.
        _this._chromeless = document.body.classList.contains("chromeless");
        _this.state = {
            viewActions: _this._getViewActions(props),
            selectedPivot: _this.props.hubViewState.selectedPivot.value,
            inFullScreenMode: _this.props.hubViewState.viewOptions.getViewOption(HubViewOptionKeys.fullScreen)
        };
        return _this;
    }
    Hub.prototype.focus = function (focusItem) {
        switch (focusItem) {
            case 0 /* commands */:
                if (this.pivotBar) {
                    this.pivotBar.focus(focusItem);
                }
                break;
        }
    };
    /**
     * This gets executed when a container component re-renders PivotBar.
     * @param nextProps New props.
     */
    Hub.prototype.componentWillReceiveProps = function (nextProps) {
        if (this.props.viewActions !== nextProps.viewActions) {
            this.setState({
                viewActions: this._getViewActions(nextProps)
            });
        }
    };
    Hub.prototype.componentDidMount = function () {
        this.props.hubViewState.selectedPivot.subscribe(this._onSelectedPivotChanged);
        this._events.on(this.props.hubViewState.viewOptions, VIEW_OPTIONS_CHANGE_EVENT, this._onViewOptionChanged);
    };
    Hub.prototype.componentWillUnmount = function () {
        this.props.hubViewState.selectedPivot.unsubscribe(this._onSelectedPivotChanged);
        this._events.off(this.props.hubViewState.viewOptions, VIEW_OPTIONS_CHANGE_EVENT, this._onViewOptionChanged);
        if (this._underlyingViewActionsObservable) {
            this._underlyingViewActionsObservable.unsubscribe(this._onViewActionObservableChanged);
        }
    };
    Hub.prototype.render = function () {
        var _this = this;
        var children = React.Children.toArray(this.props.children);
        this.validateChildren(children);
        var _a = this.props, className = _a.className, commands = _a.commands, hideHeader = _a.hideHeader, hubViewState = _a.hubViewState, onRenderFilterBar = _a.onRenderFilterBar, pivotProviders = _a.pivotProviders, onBeforePivotChange = _a.onBeforePivotChange, onPivotBarRenderComplete = _a.onPivotBarRenderComplete, showFilterBarInline = _a.showFilterBarInline, showPivots = _a.showPivots, useShadowStyle = _a.useShadowStyle, pivotRenderingModeOptions = _a.pivotRenderingModeOptions, onRenderSecondaryContent = _a.onRenderSecondaryContent, splitterProps = _a.splitterProps, scrollableRegion = _a.scrollableRegion, pivotHeaderAriaLabel = _a.pivotHeaderAriaLabel;
        var _b = this.state, selectedPivot = _b.selectedPivot, viewActions = _b.viewActions, inFullScreenMode = _b.inFullScreenMode;
        var pivotProps = {
            onPivotClicked: this._onPivotClicked,
            onBeforePivotChange: onBeforePivotChange,
            commands: commands,
            selectedPivot: selectedPivot,
            viewOptions: hubViewState.viewOptions,
            filter: hubViewState.filter,
            pivotProviders: pivotProviders,
            showPivots: inFullScreenMode ? false : showPivots,
            onRenderComplete: onPivotBarRenderComplete,
            viewActions: viewActions,
            onRenderFilterBar: onRenderFilterBar,
            showFilterBarInline: showFilterBarInline,
            showFilterBarViewOptionKey: HubViewOptionKeys.showFilterBar,
            filterBarClassName: 'vss-Hub--filterBar',
            pivotRenderingModeOptions: pivotRenderingModeOptions,
            pivotBarContentClassName: this.props.pivotBarContentClassName,
            isPivotBarContentScrollable: scrollableRegion === 1 /* PivotBar */,
            isCommandBarSticky: scrollableRegion === 0 /* Hub */,
            pivotBarSize: "tall" /* Tall */,
            headerAriaLabel: pivotHeaderAriaLabel,
        };
        var pivotComponent = (React.createElement(PivotBar, tslib_1.__assign({}, pivotProps, { className: css('vss-HubPivotBar', { 'vss-HubPivotBar--scrollable': scrollableRegion === 0 /* Hub */, 'shadowStyle': useShadowStyle, 'hideHeader': hideHeader }, this.props.pivotBarClassName), componentRef: (function (pivotBar) { _this.pivotBar = pivotBar; }) }), children));
        var hubContent = pivotComponent;
        if (onRenderSecondaryContent) {
            hubContent = (React.createElement(Splitter, tslib_1.__assign({}, splitterProps, { onRenderFarElement: function () { return pivotComponent; }, onRenderNearElement: onRenderSecondaryContent })));
        }
        return React.createElement(Fabric, { className: css('vss-Hub', className) }, hubContent);
    };
    Hub.prototype.validateChildren = function (children) {
        var hubHeader = childrenOfType(children, HubHeader)[0];
        if (!hubHeader || hubHeader.index !== 0) {
            throw new Error('Expected first child of Hub to be HubHeader.');
        }
        if (this.state.inFullScreenMode) {
            children[hubHeader.index] = React.cloneElement(children[hubHeader.index], {
                collapseBreadcrumb: true
            });
        }
    };
    Hub.prototype._getFullScreenViewAction = function () {
        return {
            key: HubViewOptionKeys.fullScreen,
            important: true,
            viewActionRenderArea: 2 /* farRight */,
            actionType: 1 /* OnOff */,
            actionProps: {
                showText: false,
                onAriaLabel: VSSUIResources.ExitFullScreenAriaLabel,
                offAriaLabel: VSSUIResources.EnterFullScreenAriaLabel,
                onIconProps: { iconName: 'BackToWindow' },
                offIconProps: { iconName: 'FullScreen' }
            }
        };
    };
    Hub.prototype._getViewActions = function (props) {
        if (ObservableLike.isObservable(props.viewActions)) {
            if (!props.hideFullScreenToggle && !this._chromeless) {
                if (this._underlyingViewActionsObservable) {
                    this._underlyingViewActionsObservable.unsubscribe(this._onViewActionObservableChanged);
                }
                // Create a new observable that adds the full screen action
                var viewActionObservable = props.viewActions;
                var newViewActions = new ObservableArray(viewActionObservable.value.concat([this._getFullScreenViewAction()]));
                this._underlyingViewActionsObservable = viewActionObservable;
                this._viewActionsObservable = newViewActions;
                viewActionObservable.subscribe(this._onViewActionObservableChanged);
                return newViewActions;
            }
            return props.viewActions;
        }
        else {
            var viewActions = props.viewActions || [];
            if (!props.hideFullScreenToggle && !this._chromeless) {
                viewActions.push(this._getFullScreenViewAction());
            }
            return viewActions;
        }
    };
    Hub.prototype._onViewOptionChanged = function (changedOptions) {
        // Handle full-screen mode changes
        if (HubViewOptionKeys.fullScreen in changedOptions) {
            var inFullScreenMode = changedOptions[HubViewOptionKeys.fullScreen] === true;
            this.setState({ inFullScreenMode: inFullScreenMode });
        }
    };
    Hub.defaultProps = {
        scrollableRegion: 1 /* PivotBar */
    };
    return Hub;
}(BaseComponent));
export { Hub };
