import "../../FabricInit";
import * as React from "react";
import { IBreadCrumbData, IBreadcrumbItem } from "office-ui-fabric-react/lib/Breadcrumb";
import { IBaseProps } from "office-ui-fabric-react/lib/Utilities";
import { IItemIndicatorProps } from '../../ItemIndicator';
import { IItemPickerProvider, IPickListProps, IPickListItem, IPickListGroup, IPickListAction, IPickListIndicator, IPickListSelection } from '../../PickList';
import { IVssIconProps } from '../../VssIcon';
import { IObservableValue } from '../../Core/Observable';
/**
 * This file has been deprecated in favor of VssBreadcrumb, remove after sprint 139
 */

export interface IHubBreadcrumbBaseProps extends IBaseProps {
    /**
     * Optional index where overflow items will be collapsed. Defaults to 0.
     */
    overflowIndex?: number;

    /**
     * Method to call when reducing the length of the breadcrumb.
     * Return undefined to never reduce breadcrumb length
     */
    onReduceData?: (data: IBreadCrumbData) => IBreadCrumbData | undefined;

    /**
     * The maximum number of breadcrumbs to display before coalescing.
     * If not specified, all breadcrumbs will be rendered.
     */
    maxDisplayedItems?: number;
}

export interface IHubBreadcrumbProps extends IHubBreadcrumbBaseProps {
    /**
     * items to display
     */
    items: IHubBreadcrumbItemInternal[];

    /**
     * Max width for the bradcrumb item
     */
    maxItemWidth?: string;

    /**
     * Class name to use for the pick list for the header item picker dropdown
     */
    pickListClassName?: string;

    /**
     * Minimum width for the header item picker dropdown, in pixels
     */
    pickListMinWidth?: number;

    /**
     * Aria label to be used for the entire breadcrumb
     */
    ariaLabel?: string;
}

export interface IHubPickerDropdownProps extends IPickListProps {

    /**
     * Breadcrumb item to use for showing the picker dropdown.
     */
    item: IHubBreadcrumbItemInternal;

    /**
     * Element that the callout will pin to.
     */
    getPinElement: () => HTMLElement;

    /**
     * MinWidth for the PickList used in the dropdown
     */
    minWidth: number;

    /**
     * Class name to use for the pick list for the header item picker dropdown
     */
    pickListClassName?: string;

    /**
     * Called when the Picker Dropdown is opened
     */
    onPickListOpened?: () => void;

    /**
     * Called when the Picker Dropdown is closed
     */
    onPickListClosed?: () => void;
}

export interface IHubBreadcrumbItem extends IBreadcrumbItem {

    /**
     * Props for an icon that is displayed on the left side of the breadcrumb
     */
    leftIconProps?: IVssIconProps;

    /**
     * Aria label to be used for an individual breadcrumb item
     */
    ariaLabel?: string;

    /**
     * Specifies behavior for the PickList that can be shown next to each breadcrumb item.
     */
    itemPicker?: IItemPickerProvider;

    /**
     * Specifies whether the breadcrumb is a header title item.
     */
    isHeaderPickerItem?: boolean;
}

export interface IHubBreadcrumbItemInternal extends IHubBreadcrumbItem {
    /**
     * Callback that is called when the selection of the PickList changes
     * 
     * @returns True to prevent the default handler for the event that triggered the change.
     */
    onSelectionChanged?: (selection: IPickListSelection) => void | boolean;

    /**
     * If true, do not show the item picker and make breadcrumb non-clickable.
     */
    nonInteractive?: boolean;
}

/**
 * Deprecated in favor of IItemPickerProvider<T>
 */
export interface IHeaderItemPicker extends IItemPickerProvider<any> {
}

export interface IHubBreadcrumb {
    /**
     * Hub breadcrumb pick list if supplied
     */
    pickList?: IHubBreadcrumbPickList;
}

export interface IHubBreadcrumbPickList {
    /**
     * Open the pick list dropdown
     */
    openDropdown(): void;

    /**
     * Close the pick list dropdown
     */
    closeDropdown(): void;

    /**
     * Toggles the pick list dropdown - opens it if it is currently closed,
     * and closes it if it is currently empty
     */
    toggleDropdown(): void;
}