import "../../Core/core.scss";
import "../../Common.scss";
import * as React from "react";
import { IObserverProps } from "../../Components/Observer/Observer.Props";
export interface IObserverState {
    [propName: string]: any;
}
/**
 * Handles subscription to properties that are IObservableValues, so that components don't have to handle on their own.
 *
 * Usage:
 *
 * <Observer myObservableValue={observableValue} className='foo'>
 *     <MyComponent myObservableValue='' />
 * </Observer>
 *
 * Your component will get re-rendered with the new value of myObservableValue whenever that value changes.
 * Additionally, any additional props set on the Observer will also get passed down.
 */
export declare class Observer extends React.Component<IObserverProps, IObserverState> {
    private subscriptions;
    constructor(props: Readonly<IObserverProps>);
    render(): JSX.Element;
    componentDidMount(): void;
    componentWillUnmount(): void;
    componentWillReceiveProps(nextProps: IObserverProps): void;
    private _subscribe;
    private _unsubscribe;
    private _onValueChanged;
}
