export interface IPanel {

}

export const enum PanelSize {
    Small = 0,
    Medium = 1,
    Large = 2,
}
/**
 * 
 */
export interface IPanelProps {

    /**
     * Label used to describe the contents of the panel for aria enabled devices.
     */
    ariaLabel?: string;

    /**
     * selector (cssClass for now) of the first focusable element. If no selector is supplied a hidden
     * element will be created and focus given to the panel through this element.
     */
    firstFocusableSelector?: string | (() => string);

    /**
     * DOM element id.
     */
    id?: string;

    /**
     * The panel title you want to display.
     */
    title?: string;

     /**
     * If the panel should show a separator between the header, body, and footer.
     * @default: false
     */
    showSeparator?: boolean;

    /**
     * Optional, description of panel.
     */
    description?: string;

    /**
     * The delegate to handle dismissing the panel. This will be called when the user clicks the close button on the panel.
     */
    onDismiss: () => void;

    /**
     * Function to render the footer of the panel. This is where you should but buttons that navigate or save progress in the panel.
     */
    onRenderPanelFooter?: () => JSX.Element;

    /**
     * Function to render a badge instead of using the title and description props. If this is passed in both title and description are ignored.
     */
    onRenderPanelBadgeHeader?: () => JSX.Element;

    /**
     * Optional callback to access the IPanel interface. Use this instead of ref for accessing
     * the public methods and properties of the component.
     */
    componentRef?: (component: IPanel | null) => void;

    /**
     * Size of the panel.
     * @default PanelSize.Medium
     */
    size?: PanelSize;
}