import "../../FabricInit";
import React = require("react");
import { ISuggestionModel } from "office-ui-fabric-react/lib/Pickers";

export interface ISuggestionItemProps<T> {
    /**
     * The component ref of the item
     */
    componentRef?: () => void;
    
    /**
     * The Suggestions model of the suggestion item.
     */
    suggestionModel: ISuggestionModel<T>;

    /**
     * The callback used to render the suggestion item.
     */
    renderSuggestion: (item: T, suggestionItemProps?: ISuggestionItemProps<T>) => JSX.Element;

    /**
     * The callback when an item is clicked
     */
    onClick: () => void;

    /**
     * The callback when the remove button for an item is clicked
     */
    onRemoveItem: (ev: React.MouseEvent<HTMLButtonElement>) => void;

    /**
     * Class name to apply to the item.
     */
    className?: string;

    /**
     * The key for the item
     */
    id?: string;

    /**
     * Boolean indicating if the show remove button should be shown.
     * @default true
     */
    showRemoveButton?: boolean;

    /**
     * If the item should be shown as selected even if its not in the suggestion model.
     * @default false
     */
    isSelectedOverride?: boolean;

    /**
     * The ARIA label for the button to remove the suggestion from the list.
     */
    removeButtonAriaLabel?: string;
}