import "../../FabricInit";
import { IBaseProps } from "office-ui-fabric-react/lib/Utilities";
import { IIdentity } from "../PeoplePicker/SharedPeoplePicker.Props";

/**
 * Definitions for ContactCardContactLine used by ContactCard.
 */
export interface ContactCardContactLineProps extends IBaseProps {
    /**
     * Label for line.
     */
    label?: string;

    /**
     * Content for line.
     */
    content?: string;

    /**
     * (Optional) Link for line.
     */
    link?: string;

    /**
     * (Optional) to pad top.
     */
    padTop?: boolean;
}

/**
 * Definitions for DefaultCardContactLine (used by DefaultCard).
 */
export interface DefaultCardContactLineProps extends IBaseProps {
    /**
     * Name for icon.
     */
    iconName?: string;

    /**
     * Content for line.
     */
    content?: string;

    /**
     * (Optional) Link for line.
     */
    link?: string;
}

/**
 * Definitions for ContactCard.
 */
export interface ContactCardProps extends IBaseProps {
    /**
     * Identity object for persona and contact information.
     */
    identity?: IIdentity;
}

export interface OrganizationCardProps extends ContactCardProps {
    /**
     * List of managers.
     */
    managerList?: IIdentity[];

    /**
     * List of direct reports.
     */
    directReportList?: IIdentity[];

    /**
     * Method to handle identity click.
     */
    onClickEntity?: (identifier: string | IIdentity) => void;
}

/**
 * Definitions for DefaultCard.
 */
export interface DefaultCardProps extends ContactCardProps {
    /**
     * Direct manager identity object.
     */
    manager?: IIdentity;

    /**
     * Previous header exists boolean.
     */
    isPreviousHeader?: boolean;

    /**
     * Method to show contact card.
     */
    showContactCard: () => void;

    /**
     * Method to show organization card.
     */
    showOrganizationCard?: () => void;

    /**
     * Method to handle identity click.
     */
    onClickEntity?: (identifier: string | IIdentity) => void;

    /**
     * Function to set the focus to first active element of focuszone on update of the card.
     */
    setFocus?: () => void;
}