import "../../FabricInit";
import { IBaseProps, IPoint } from "office-ui-fabric-react/lib/Utilities";
import { IIdentity, IPersonaConnections } from "../PeoplePicker/SharedPeoplePicker.Props";

export const enum CardType {
    Default,
    Contact,
    Organization
}

export interface IDataState {
    header?: IIdentity;
    identity: IIdentity;
    managerList?: IIdentity[];
    directReportList?: IIdentity[];
    previousDataState?: IDataState;
    cardType?: CardType;
    displayName?: string;
    imageUrl?: string;
    email?: string;
    source?: string;
}

/*
 * Definitions for PersonaCard.
 */
export interface PersonaCardProps extends IBaseProps {
    /*
    * (Optional) Initial identity object, if available.
    */
    identity?: IIdentity;

    /*
	* Consumer id passed from the caller of the persona card and passed into the identity call.
	*/
    consumerId?: string;

    /*
    * (Optional) Unique attribute (entityID or signInAddress or uniqueName) to get identity object (if no identity available).
    */
    uniqueAttribute?: string;

    /*
    * (Optional) Initial header props to use if content is hosted inside an external component
    */
    initialHeader?: HeaderElementProps;

    /*
    * (Optional) Display name, if passed, in will be used to display when identity service fails to fetch the identity
    */
    displayName?: string;

    /*
    * (Optional) Image url, if passed, in will be used to display when identity service fails to fetch the identity
    */
    imageUrl?: string;

    /*
    * (Optional) Reference HTML component.
    * @deprecated use target instead
    */
    referenceHTMLComponent?: HTMLElement;

    /*
    * (Optional) The target that the PersonCard should try to position itself based on.
    * Can be an HTMLElement or a point object with the X and Y coordinates set
    */
    target?: HTMLElement | IPoint;

    /*
    * (Optional) Callback when card is dismissed.
    */
    onDismissCallback?: () => void;

    /**
     * Request connection information about a given Entity.
     */
    onRequestConnectionInformation: (entity: IIdentity, getDirectReports?: boolean) => IPersonaConnections | PromiseLike<IPersonaConnections>;
    
    /**
     * Request Entity information about a given Entity.
     */
    getEntityFromUniqueAttribute: (entityId: string) => IIdentity | PromiseLike<IIdentity>;
}

export interface PersonaCardContentProps extends IBaseProps, PersonaCardProps {
    /*
    * Data props from parent component to get the identity, cardtype etc information
    */
    dataProps: IDataState;
    /*
    * (Optional) Onclick of any persona entity
    */
    onClickEntity?: (identity: IIdentity) => void;

    /*
    * (Optional) Onclick of any persona header
    */
    onHeaderClick?: () => void;

    /*
    * (Optional) Callback to open Organization Card
    */
    onShowOrganizationCard?: () => void;

    /*
    * (Optional) Callback to open Contact Card
    */
    onShowContactCard: () => void;

    /*
    * (Optional) Prevent requests if working
    */
    working?: boolean;
}

/**
 * Definitions for HeaderElement used by PersonaCard.
 */
export interface HeaderElementProps extends IBaseProps {
    /**
     * Current data state (can be null).
     */
    identity?: IIdentity;

    /**
     * Function on click (show default card or pop breadcrumb).
     */
    onClickFunction?: () => void;

    /**
     * Function to set the focus to first active element of focuszone on update of the card.
     */
    setFocus?: () => void;
}