import * as tslib_1 from "tslib";
import "../../FabricInit";
/**
 * The PersonaCard is intended to show contact and organization information for an identity.
 * You may pass the entity directly or you may pass a unique attribute (e.g. uniqueName, entityID, signInAddress) as a prop.
 *
 */
import * as React from "react";
import { BaseComponent } from "office-ui-fabric-react/lib/Utilities";
import * as Utils from "./PersonaCardIdentityUtils";
import { PersonaCardContent } from "./PersonaCardContent";
var PersonaCard = /** @class */ (function (_super) {
    tslib_1.__extends(PersonaCard, _super);
    function PersonaCard(props) {
        var _this = _super.call(this, props) || this;
        _this._onDismissCallback = function () {
            _this.props.onDismissCallback();
        };
        // Handle going back
        _this._headerOnClickHandler = function () {
            if (_this.state.dataState.previousDataState) {
                _this.setState({
                    dataState: _this.state.dataState.previousDataState,
                    working: false
                });
            }
            else if (_this.props.initialHeader && _this.props.initialHeader.onClickFunction) {
                _this.props.initialHeader.onClickFunction();
            }
        };
        _this._onShowContactCard = function () {
            var newDataState = tslib_1.__assign({}, _this.state.dataState, { previousDataState: _this.state.dataState, cardType: 1 /* Contact */, header: _this.state.dataState.identity });
            _this.setState({
                dataState: newDataState
            });
        };
        _this._onShowOrganizationCard = function () {
            // Do not handle click event if working
            if (_this.state.working || _this._dismissed) {
                return;
            }
            var currentDataState = _this.state.dataState;
            if (currentDataState.managerList && currentDataState.managerList.length <= 1) {
                // Only make the call if we don't have the data already
                var currentDataState_1 = _this.state.dataState;
                var newDataState = tslib_1.__assign({}, currentDataState_1, { cardType: 2 /* Organization */, header: currentDataState_1.identity, previousDataState: currentDataState_1 });
                _this.setState({
                    dataState: newDataState,
                    working: true
                });
                _this._resolveIdentity(newDataState, _this.props.onRequestConnectionInformation(currentDataState_1.identity, true));
            }
            else {
                // Already have the data so we can present
                var currentDataState_2 = _this.state.dataState;
                var newDataState = tslib_1.__assign({}, currentDataState_2, { cardType: 2 /* Organization */, header: currentDataState_2.identity, previousDataState: currentDataState_2 });
                _this.setState({
                    dataState: newDataState,
                    working: false
                });
            }
        };
        // Handle entity click
        _this._onClickEntity = function (identity) {
            // Do not handle click event if working
            if (_this.state.working) {
                return;
            }
            var currentDataState = _this.state.dataState;
            var newDataState = {
                identity: identity,
                cardType: 0 /* Default */,
                header: currentDataState.identity,
                directReportList: [],
                managerList: [],
                previousDataState: currentDataState,
                displayName: identity.displayName,
                imageUrl: identity.image,
                email: identity.mail
            };
            _this.setState({
                dataState: newDataState,
                working: true
            });
            // API call to get identity
            _this._getIdentityByUniqueAttribute(identity);
        };
        _this._updateConnections = function (dataState, connections) {
            // Don't call if the component is unmounted.
            if (_this._dismissed) {
                return;
            }
            // ensure we haven't attempted to update which card we were in before the callback happens.
            if (_this.state.dataState && dataState.identity === _this.state.dataState.identity) {
                _this.setState({
                    dataState: tslib_1.__assign({}, _this.state.dataState, { managerList: connections.managers && connections.managers.reverse(), directReportList: connections.directReports }),
                    working: false
                });
            }
        };
        _this._updateEntity = function (identity) {
            var hasCompleteIdentity = Utils.isCompleteIdentity(identity, true);
            var requireConnections = Utils.isAadUser(identity) || Utils.isAdUser(identity);
            var imageUrl = _this.props.imageUrl ? _this.props.imageUrl : identity.image;
            // Update data state and working state (visible state updated later)
            var newDataState = tslib_1.__assign({}, _this.state.dataState, { identity: identity, displayName: identity.displayName, imageUrl: imageUrl });
            // Check for authenticated users. For authenticated users, go on to make the connections call. For non-authenticated users, stop the calls and load the card with images.
            if (!hasCompleteIdentity) {
                _this.setState({
                    working: true
                });
            }
            else if (requireConnections) {
                _this.setState({ dataState: newDataState, working: false });
                _this._resolveIdentity(newDataState, _this.props.onRequestConnectionInformation(newDataState.identity));
            }
            else {
                _this.setState({ dataState: newDataState, working: false });
            }
        };
        // Setup state data and history
        var initialDataState = {
            identity: props.identity,
            managerList: null,
            directReportList: null,
            previousDataState: null,
            cardType: 0 /* Default */,
            header: props.initialHeader ? props.initialHeader.identity : null,
            displayName: props.displayName,
            imageUrl: props.imageUrl,
            email: _this._getEmail()
        };
        var moreWorkNeeded = true;
        if (!props.identity && !props.uniqueAttribute && props.displayName) {
            // All we have is displayName and may be imageUrl. Simply go ahead and setup the state for displaying the card without any identity call.
            initialDataState.identity = {
                entityId: "",
                entityType: "user",
                originDirectory: "vsd",
                originId: "",
                displayName: props.displayName,
                image: props.imageUrl,
                signInAddress: initialDataState.email
            };
            moreWorkNeeded = false;
        }
        _this.state = {
            dataState: initialDataState,
            working: moreWorkNeeded
        };
        return _this;
    }
    PersonaCard.prototype.componentDidMount = function () {
        if (this.state.working) {
            this._setupInitialData(this.props.uniqueAttribute);
        }
    };
    PersonaCard.prototype.componentWillUnmount = function () {
        this._dismissed = true;
    };
    //Render
    PersonaCard.prototype.render = function () {
        return this.props.uniqueAttribute || this.props.identity || this.props.displayName ? (React.createElement(PersonaCardContent, tslib_1.__assign({}, this.props, { dataProps: this.state.dataState, onClickEntity: this._onClickEntity, onDismissCallback: this._onDismissCallback, onShowContactCard: this._onShowContactCard, onShowOrganizationCard: this._onShowOrganizationCard, working: this.state.working, onHeaderClick: this._headerOnClickHandler }))) : null;
    };
    // Helper method to get initial data (which includes only direct manager)
    PersonaCard.prototype._setupInitialData = function (uniqueAttribute) {
        var dataState = this.state.dataState;
        if (!dataState.identity) {
            // Get identity first, then get connections in callback
            this._getIdentityByUniqueAttribute(uniqueAttribute);
        }
        else if (!Utils.isCompleteIdentity(dataState.identity)) {
            // Seems to be cached, refetch identity by unique attribute (entityId)
            this._getIdentityByUniqueAttribute(dataState.identity);
        }
        else {
            this._resolveIdentity(dataState, this.props.onRequestConnectionInformation(dataState.identity));
        }
    };
    // Helper method to get identity information given an entity ID
    PersonaCard.prototype._resolveIdentity = function (dataState, identity) {
        var _this = this;
        var identityAsEntity = identity;
        var identityAsPromiseLike = identity;
        if (identityAsPromiseLike && identityAsPromiseLike.then) {
            identityAsPromiseLike.then(function (connections) {
                _this._updateConnections(dataState, connections);
            });
        }
        else if (identityAsEntity) {
            this._updateConnections(dataState, identityAsEntity);
        }
    };
    // Helper method to get identity information given an entity ID
    PersonaCard.prototype._resolveIEntity = function (identity) {
        var _this = this;
        var identityAsEntity = identity;
        var identityAsPromiseLike = identity;
        if (identityAsPromiseLike && identityAsPromiseLike.then) {
            identityAsPromiseLike.then(function (connections) {
                _this._updateEntity(connections);
            });
        }
        else if (identityAsEntity) {
            this._updateEntity(identityAsEntity);
        }
    };
    // Helper method to get identity information given an entity ID
    PersonaCard.prototype._getIdentityByUniqueAttribute = function (identifier) {
        var uniqueAttribute;
        if (typeof identifier == "string") {
            uniqueAttribute = identifier;
        }
        else if (Utils.isAadUser(identifier) || Utils.isAdUser(identifier)) {
            uniqueAttribute = identifier.entityId;
        }
        else {
            uniqueAttribute = identifier.signInAddress; // VSD users (MSA accounts)
        }
        this._resolveIEntity(this.props.getEntityFromUniqueAttribute(uniqueAttribute));
    };
    PersonaCard.prototype._getEmail = function () {
        // This is for displaying a fall back card when identities search doesn't fetch any identity
        if (Utils.isAadUser(this.props.identity) || Utils.isAdUser(this.props.identity)) {
            return this.props.identity.mail;
        }
        else if (this.props.identity && this.props.identity.signInAddress) {
            return this.props.identity.signInAddress;
        }
        else if (this.props.identity && this.props.identity.mail) {
            return this.props.identity.mail;
        }
        else if (this.props.uniqueAttribute) {
            // Check if uniqueAttribute is an email
            var parts = this.props.uniqueAttribute.split("@");
            if (parts.length === 2 && parts[0].length >= 1 && parts[1].length >= 3) {
                var domainParts = parts[1].split(".");
                if (domainParts.length > 1) {
                    return this.props.uniqueAttribute;
                }
            }
        }
        return null;
    };
    return PersonaCard;
}(BaseComponent));
export { PersonaCard };
