import * as tslib_1 from "tslib";
import "../../FabricInit";
import * as React from "react";
import * as Resources from '../../VSS.Resources.VSSUI';
import { BaseComponent } from "office-ui-fabric-react/lib/Utilities";
import { Callout } from "office-ui-fabric-react/lib/Callout";
import { FocusZone, FocusZoneDirection } from "office-ui-fabric-react/lib/FocusZone";
import { Spinner, SpinnerType } from "office-ui-fabric-react/lib/Spinner";
import { css, DelayedRender } from "office-ui-fabric-react/lib/Utilities";
import { ContactCard } from "./ContactCard";
import { DefaultAbridgedCard } from "./DefaultAbridgedCard";
import { DefaultSimpleCard } from "./DefaultSimpleCard";
import { DefaultCard } from "./DefaultCard";
import { OrganizationCard } from "./OrganizationCard";
import * as Utils from "./PersonaCardIdentityUtils";
import { PersonaCardHeaderElement as HeaderElement } from "./PersonaCardHeaderElement";
/**
 * The content of the callout for the persona card.
 */
var PersonaCardContent = /** @class */ (function (_super) {
    tslib_1.__extends(PersonaCardContent, _super);
    function PersonaCardContent() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._onFocusZoneRef = function (focusZone) {
            _this._focusZone = focusZone;
            _this._focusOnUpdate();
        };
        _this._focusOnUpdate = function () {
            var directReport = _this.props.dataProps.directReportList;
            // Setting the focus only if card is loaded completely for which case directReportList is not null
            if (_this.props.dataProps.cardType === 0 /* Default */ && directReport) {
                _this._focusZone.focus();
            }
            else if (_this.props.dataProps.cardType !== 0 /* Default */) {
                _this._focusZone.focus();
            }
        };
        // State change helper methods
        _this._onDismissCallout = function () {
            if (_this.props.onDismissCallback) {
                _this.props.onDismissCallback();
            }
        };
        return _this;
    }
    PersonaCardContent.prototype.componentDidUpdate = function () {
        var isVsdUser = Utils.isVsdUser(this.props.dataProps.identity);
        /*
        * Avoid set focus getting called multiple times, since the header element will set it after it mounts/updates
        * Also for default abridged card, set it only after focus zone ref is available
        */
        if (!this.props.dataProps.header && !isVsdUser) {
            this._focusOnUpdate();
        }
    };
    // Render
    PersonaCardContent.prototype.render = function () {
        // Get target element
        var targetElement = this.props.referenceHTMLComponent ? this.props.referenceHTMLComponent : this.props.target;
        // Use component state information to render component
        return (React.createElement("div", null,
            React.createElement(Callout, { onDismiss: this._onDismissCallout, target: targetElement, ariaLabel: Resources.PeoplePickerProfileCardAriaLabel, role: "alert", isBeakVisible: false, gapSpace: 10, className: "persona-card-callout", directionalHint: 11 /* rightTopEdge */ },
                React.createElement("div", null, this._renderCard()))));
    };
    PersonaCardContent.prototype._renderCard = function () {
        var cssClassName = "persona-card-styles";
        if (this.props.working && !Utils.isCompleteIdentity(this.props.dataProps.identity)) {
            // Still working to get initial data
            return (React.createElement("div", { className: css(cssClassName, "loading") },
                React.createElement(DelayedRender, { delay: 300 },
                    React.createElement(Spinner, { label: Resources.Loading, type: SpinnerType.large, className: "persona-card-loading-spinner" }))));
        }
        var isAdOrAad = Utils.isAdUser(this.props.dataProps.identity) || Utils.isAadUser(this.props.dataProps.identity);
        var headerIdentity = this.props.dataProps.header || null;
        return (React.createElement(FocusZone, { className: cssClassName, handleTabKey: 1 /* all */, isCircularNavigation: true, componentRef: this._onFocusZoneRef, direction: FocusZoneDirection.vertical },
            isAdOrAad &&
                headerIdentity && (React.createElement(HeaderElement, { identity: headerIdentity, onClickFunction: this.props.onHeaderClick, setFocus: this._focusOnUpdate })),
            this._renderInnerCard()));
    };
    PersonaCardContent.prototype._renderInnerCard = function () {
        // Get component state data
        var _a = this.props.dataProps, identity = _a.identity, managerList = _a.managerList, directReportList = _a.directReportList;
        if (!identity) {
            return null;
        }
        var isVsdUser = Utils.isVsdUser(identity);
        var isWmdUser = Utils.isWmdUser(identity);
        var isAadUser = Utils.isAadUser(identity);
        var isAdUser = Utils.isAdUser(identity);
        if (isVsdUser) {
            return React.createElement(DefaultAbridgedCard, { identity: identity });
        }
        else if (isWmdUser) {
            return React.createElement(DefaultSimpleCard, { identity: identity });
        }
        else if (isAadUser || isAdUser) {
            switch (this.props.dataProps.cardType) {
                case 1 /* Contact */:
                    return React.createElement(ContactCard, { identity: identity });
                case 2 /* Organization */:
                    return (React.createElement(OrganizationCard, { identity: identity, managerList: managerList, directReportList: directReportList, onClickEntity: this.props.onClickEntity }));
                case 0 /* Default */:
                default:
                    var manager = managerList && managerList.length > 0 ? managerList[managerList.length - 1] : null;
                    return (React.createElement(DefaultCard, { identity: identity, manager: manager, isPreviousHeader: !!this.props.dataProps.previousDataState || !!this.props.initialHeader, showContactCard: this.props.onShowContactCard, showOrganizationCard: this.props.onShowOrganizationCard, onClickEntity: this.props.onClickEntity, setFocus: this._focusOnUpdate }));
            }
        }
        return null;
    };
    return PersonaCardContent;
}(BaseComponent));
export { PersonaCardContent };
