/// <reference types="react" />
import "../../FabricInit";
import { BaseComponent } from 'office-ui-fabric-react/lib/Utilities';
import { IPickList, IPickListItem, IPickListProps } from '../../Components/PickList/PickList.Props';
import "./PickList.css";
export interface IPickListState {
    /**
     * The currently selected keys in the PickList
     */
    selectedKeys?: string[];
    initiallySelectedKeys?: string[];
    /**
     * The current set of items. If a promise is supplied through the items prop, then this
     * is initially empty until the promise has been resolved.
     */
    items?: any[];
    /**
     * True if a promise was supplied for the items prop and it has not yet been fulfilled.
     */
    itemsLoading?: boolean;
    /**
     * The wrapped data models for all items provided to the PickList
     */
    groupDictionary?: {
        [key: string]: IPickListItemInternal[];
    };
    searchResults?: any[];
    /**
     * The wrapped data models for all items provided to the PickList, given the current search text
     */
    filteredItems?: IPickListItemInternal[];
    isSearchInProgress?: boolean;
    searchError?: Error;
    selectedOperatorKey?: string;
    initiallySelectedOperatorKey?: string;
}
export declare const enum MessageLevel {
    none = 0,
    info = 1,
    error = 2
}
export interface IPickListItemInternal extends IPickListItem {
    isSelected: boolean;
    isSeparator?: boolean;
    isGroupHeader?: boolean;
    messageLevel?: MessageLevel;
    isLoadingIndicator?: boolean;
    onRender?: (element?: JSX.Element) => JSX.Element;
    data?: any;
}
export declare class PickList extends BaseComponent<IPickListProps, IPickListState> implements IPickList {
    private _id;
    private _focusZone;
    private _searchFocusZone;
    private _pivotFocusZone;
    private _textField;
    private _pickListElement;
    private _operatorGroup;
    private _firstRow;
    private _isMounted;
    private _maintainsOwnState;
    private _initialItemsLoaded;
    private _hideLoadingContent;
    private _loadingContentHidden;
    private _focusOnInitialLoad;
    private _hideSearchingContent;
    private _searchingContentHidden;
    private _hideSearchingContentTimeout;
    private _groupLoadRequested;
    private _searchTextChangeDelegate;
    private _searchText;
    private _itemsAnnouncer;
    private _selectedPivot;
    private _actionCount;
    private _ariaDescribedById;
    private _list;
    private _itemDictionary;
    constructor(props: IPickListProps);
    clearSelection(): void;
    reset(): void;
    selectAll(): void;
    focus(forceRefocus?: boolean): boolean;
    /**
     * Scroll to list item by index
     */
    scrollToIndex(index: number): void;
    /**
     * Scroll to selected item in the list
     */
    scrollToSelected(): void;
    forceUpdate(): void;
    componentWillMount(): void;
    componentDidMount(): void;
    componentWillUnmount(): void;
    componentWillReceiveProps(newProps: IPickListProps): void;
    render(): JSX.Element;
    private _getPivotBar;
    private _onPivotBarFocused;
    /**
     * Handle the onKeyPress event on the pivot button.
     */
    private _onPivotKeyPress;
    /**
     * Handles the onClick event on the pivot button.
     */
    private _onPivotClick;
    private _updateSelectedPivot;
    private _getInitialPivot;
    private _getCurrentPivot;
    private _rebuild;
    private _buildFromItems;
    private _onSearchContainerFocused;
    private _onKeyDown;
    private _onSearchTextChanged;
    private _onClearInputKeyPress;
    private _onClearInputClick;
    private _updateSearchText;
    private _announceSearchResults;
    private _wrapItems;
    private _groupItems;
    private _filterByText;
    private _defaultFilterItemByText;
    private _buildGroupedItems;
    private _requestGroupLoad;
    private _getOnSearchPromise;
    private _onSelectionChanged;
    private _onOperatorChanged;
    private _onRenderCell;
    private _onRenderCellSingleSelection;
    private _onRenderCellMultipleSelection;
    private _onSingleSelectionChanged;
    private _onMultipleSelectionChanged;
    private _onActiveElementChanged;
    private _getIndicators;
    private _getKeysFromItems;
    private _hasNewItems;
    private _haveGroupsChanged;
    private _isThenable;
    private _arrayEquals;
}
