import * as tslib_1 from "tslib";
import "../../FabricInit";
import * as React from 'react';
import * as ReactDOM from 'react-dom';
import { CommandButton } from 'office-ui-fabric-react/lib/Button';
import { ChoiceGroup } from 'office-ui-fabric-react/lib/ChoiceGroup';
import { Checkbox } from '../../Checkbox';
import { FocusZone, FocusZoneDirection } from 'office-ui-fabric-react/lib/FocusZone';
import { List } from 'office-ui-fabric-react/lib/List';
import { SelectionMode } from 'office-ui-fabric-react/lib/Selection';
import { Spinner, SpinnerSize } from 'office-ui-fabric-react/lib/Spinner';
import { TextField } from 'office-ui-fabric-react/lib/TextField';
import { TooltipOverflowMode, TooltipHost } from '../../Tooltip';
import { BaseComponent, css, format, getRTLSafeKeyCode, doesElementContainFocus, getFirstFocusable, getId, warn } from 'office-ui-fabric-react/lib/Utilities';
import { ItemIndicatorSet } from '../../ItemIndicator';
import { VssIcon } from '../../VssIcon';
import * as VSSUIResources from '../../VSS.Resources.VSSUI';
import * as Utils_Accessibility from '../../Core/Util/Accessibility';
import "./PickList.css";
var SELECT_ALL_ITEM_KEY = 'select-all-item';
var DEFAULT_MIN_ITEMS_NO_SEARCH = 8;
var HIDE_LOADING_ITEMS_TIME_MS = 250;
var SEARCH_TEXT_DEBOUNCE_TIME_MS = 250;
var SEARCH_TEXT_ANNOUNCE_TIME_MS = 1000;
var ANNOUNCE_DELAY = 200;
var MAX_PIVOT_ITEM_WIDTH = 340;
var DEFAULT_ROW_HEIGHT = 32;
var PickList = /** @class */ (function (_super) {
    tslib_1.__extends(PickList, _super);
    function PickList(props) {
        var _this = _super.call(this, props) || this;
        _this._groupLoadRequested = {};
        _this._searchText = '';
        _this._selectedPivot = '';
        _this._onPivotBarFocused = function () {
            if (_this._pivotFocusZone) {
                _this._pivotFocusZone.focus();
            }
        };
        _this._onSearchContainerFocused = function () {
            if (_this._searchFocusZone) {
                _this._searchFocusZone.focus(true);
            }
        };
        _this._onKeyDown = function (ev) {
            if (ev.which === 9 /* tab */) {
                var onFocusLost = _this.props.onFocusLost;
                if (onFocusLost) {
                    onFocusLost();
                }
            }
        };
        _this._onSearchTextChanged = function (searchText) {
            _this._searchText = searchText;
            _this._searchTextChangeDelegate(searchText);
        };
        _this._onClearInputKeyPress = function (ev) {
            if (ev.which === 13 /* enter */ || ev.which === 32 /* space */) {
                ev.preventDefault();
                _this._onClearInputClick();
            }
        };
        _this._onClearInputClick = function () {
            _this._searchText = '';
            _this._updateSearchText();
            if (_this._textField) {
                _this._textField.focus();
            }
        };
        _this._updateSearchText = function (resultsToContinue) {
            var _a = _this.props, groups = _a.groups, onSearch = _a.onSearch;
            var isSearchInProgress = _this.state.isSearchInProgress;
            var searchText = _this._searchText;
            if (searchText) {
                if (onSearch) {
                    window.setTimeout(function () {
                        if (_this.state.isSearchInProgress) {
                            Utils_Accessibility.announce(VSSUIResources.Searching, true, ANNOUNCE_DELAY);
                        }
                    }, SEARCH_TEXT_ANNOUNCE_TIME_MS);
                    isSearchInProgress = true;
                    /**
                     * Set _hideSearchingContent to true to indicate to us to not render search loading indicator.
                     * This state is set for a short amount of time on initial render to avoid flickering
                     * of the progress indicator when the search can be completed quickly.
                     */
                    if (_this._hideSearchingContentTimeout) {
                        window.clearTimeout(_this._hideSearchingContentTimeout);
                        _this._hideSearchingContentTimeout = 0;
                    }
                    _this._hideSearchingContent = true;
                    _this._hideSearchingContentTimeout = window.setTimeout(function () {
                        _this._hideSearchingContent = false;
                        if (_this._searchingContentHidden) {
                            _this.forceUpdate();
                        }
                    }, HIDE_LOADING_ITEMS_TIME_MS);
                    _this._getOnSearchPromise(searchText, resultsToContinue).then(function (searchResults) {
                        // Ignore search results from previous searches. Only update results if the search string is still current
                        if (searchText === _this._searchText) {
                            if (!searchResults) {
                                _this.setState({ isSearchInProgress: false });
                            }
                            else {
                                isSearchInProgress = false;
                                var _a = _this.state, groupDictionary_1 = _a.groupDictionary, itemsLoading_1 = _a.itemsLoading, selectedKeys_1 = _a.selectedKeys;
                                var searchPickListItems = _this._wrapItems(searchResults, selectedKeys_1, 'search-results');
                                var groupDictionaryWithSearchResults = _this._groupItems(searchPickListItems, groupDictionary_1);
                                var filteredGroupDictionary_1 = _this._filterByText(searchText, groupDictionaryWithSearchResults);
                                var groupedItems_1 = _this._buildGroupedItems(searchText, isSearchInProgress, searchResults, filteredGroupDictionary_1, groups, selectedKeys_1, itemsLoading_1);
                                _this._announceSearchResults(filteredGroupDictionary_1, isSearchInProgress, itemsLoading_1);
                                _this.setState({ isSearchInProgress: isSearchInProgress, filteredItems: groupedItems_1, searchResults: searchResults, searchError: undefined }, function () { return _this._rebuild(_this.props); });
                            }
                        }
                    }, function (error) {
                        _this.setState({ isSearchInProgress: false, searchError: error }, function () { return _this._rebuild(_this.props); });
                    });
                }
            }
            var _b = _this.state, groupDictionary = _b.groupDictionary, itemsLoading = _b.itemsLoading, selectedKeys = _b.selectedKeys;
            var filteredGroupDictionary = _this._filterByText(searchText, groupDictionary);
            var groupedItems = _this._buildGroupedItems(searchText, isSearchInProgress, [], filteredGroupDictionary, groups, selectedKeys, itemsLoading);
            // If scroll were kept at a high index, it will make "Loading" visible initially
            // which may result in a cascade of loading pages until they reach such index.
            _this.scrollToIndex(0);
            _this._announceSearchResults(filteredGroupDictionary, isSearchInProgress, itemsLoading);
            _this.setState({
                filteredItems: groupedItems,
                isSearchInProgress: isSearchInProgress,
                searchResults: resultsToContinue ? _this.state.searchResults : [],
            }, function () { return _this._rebuild(_this.props); });
        };
        _this._defaultFilterItemByText = function (searchText, item) {
            return item.name.toLowerCase().indexOf(searchText.toLowerCase()) !== -1;
        };
        _this._onSelectionChanged = function (newSelectedKeys, selectedOperatorKey) {
            var preventDefault = false;
            if (_this._isMounted) {
                var onSelectionChanged = _this.props.onSelectionChanged;
                var newSelectedOperatorKey = selectedOperatorKey || _this.state.selectedOperatorKey;
                if (onSelectionChanged) {
                    var selectedItems = newSelectedKeys.filter(function (key) { return (key !== SELECT_ALL_ITEM_KEY && !!_this._itemDictionary[key]); }).map(function (key) { return _this._itemDictionary[key].data; });
                    preventDefault = onSelectionChanged({ selectedItems: selectedItems, selectedOperatorKey: newSelectedOperatorKey }) === true;
                }
                if (_this._maintainsOwnState) {
                    var filteredItems = _this.state.filteredItems.map(function (item) {
                        item.isSelected = newSelectedKeys.some(function (key) { return key === item.key; });
                        return item;
                    });
                    _this.setState({
                        filteredItems: _this.state.filteredItems.slice(),
                        selectedKeys: newSelectedKeys,
                        selectedOperatorKey: newSelectedOperatorKey
                    });
                }
            }
            return preventDefault;
        };
        _this._onOperatorChanged = function (newSelectedOperatorKey) {
            _this._onSelectionChanged(_this.state.selectedKeys, newSelectedOperatorKey);
        };
        _this._onRenderCell = function (item, index, headingIndexes) {
            var selectionMode = _this.props.selectionMode;
            if (item.isSeparator) {
                if (item.isGroupHeader) {
                    if (item.onRender) {
                        return item.onRender();
                    }
                    else if (item.name) {
                        return React.createElement("div", { role: "heading", className: css("vss-PickList--groupHeader", { "first-group": index === 0 }) }, item.name);
                    }
                    else {
                        return React.createElement("div", { className: "vss-PickList--groupSeparator" });
                    }
                }
                else if (item.isLoadingIndicator) {
                    var div = React.createElement("div", { className: "vss-PickList--loading" },
                        React.createElement(Spinner, { size: SpinnerSize.medium, label: item.name }));
                    return item.onRender
                        ? item.onRender(div)
                        : div;
                }
                else if (item.messageLevel) {
                    return React.createElement("div", { className: css("vss-PickList--message", item.messageLevel === 2 /* error */ && "is-error") }, item.name);
                }
                else {
                    return React.createElement("div", null, item.name);
                }
            }
            var firstInGroup = index === 0 || _this.state.filteredItems[index - 1].isSeparator;
            // Determine location in group
            var containsNoGroups = headingIndexes.length === 0;
            var groupsBeforeElement = containsNoGroups ? 0 : headingIndexes.filter(function (x) { return x < index; }).length;
            var actualIndex = index - groupsBeforeElement;
            var actualSize = _this.state.filteredItems.length - headingIndexes.length;
            if (selectionMode === SelectionMode.multiple) {
                return _this._onRenderCellMultipleSelection(item, firstInGroup, actualIndex, actualSize);
            }
            return _this._onRenderCellSingleSelection(item, firstInGroup, actualIndex, actualSize);
        };
        _this._onRenderCellSingleSelection = function (item, firstInGroup, index, setSize) {
            var indicators = _this._getIndicators(item, _this.props.indicators);
            var disabled = typeof item.disabled === "function" ? item.disabled() : item.disabled;
            return React.createElement(PickListRow, { ariaLabel: item.ariaLabel || item.name, ariaDescribedBy: _this.props.itemAriaDescription && _this._ariaDescribedById, ref: function (row) { if (index === 0) {
                    _this._firstRow = row;
                } }, dataSelectionKey: item.key, isSelected: item.isSelected, setSize: setSize, itemIndex: index, href: item.href, disabled: disabled, firstInGroup: firstInGroup, onKeyboardEnter: function () {
                    return _this._onSingleSelectionChanged(item.key, disabled);
                } },
                React.createElement(PickListSelectableElement, { item: item, indicators: indicators, onSelectionChanged: function () {
                        return _this._onSingleSelectionChanged(item.key, disabled);
                    }, onRenderItemText: _this.props.onRenderItemText }));
        };
        _this._onRenderCellMultipleSelection = function (item, firstInGroup, index, setSize) {
            var indicators = _this._getIndicators(item, _this.props.indicators);
            var disabled = typeof item.disabled === "function" ? item.disabled() : item.disabled;
            return React.createElement(PickListRow, { ariaLabel: item.ariaLabel || item.name, ariaDescribedBy: _this.props.itemAriaDescription && _this._ariaDescribedById, ref: function (row) { if (index === 0) {
                    _this._firstRow = row;
                } }, dataSelectionKey: item.key, spaceBarTriggersClick: true, isSelected: item.isSelected, setSize: setSize, itemIndex: index, disabled: disabled, firstInGroup: firstInGroup, onKeyboardEnter: function () {
                    return _this._onMultipleSelectionChanged(item.key, !item.isSelected, disabled);
                } },
                React.createElement(PickListCheckboxElement, { item: item, indicators: indicators, onSelectionChanged: function (isSelected) {
                        return _this._onMultipleSelectionChanged(item.key, isSelected, disabled);
                    } }));
        };
        _this._onActiveElementChanged = function (element) {
            if (_this.props.selectionMode === SelectionMode.single && _this.props.selectOnFocus) {
                var key = element.getAttribute('data-selection-key');
                if (key) {
                    _this._onSelectionChanged([key]);
                }
            }
        };
        _this._id = _this.props.id || getId('PickList');
        _this._maintainsOwnState = false;
        _this._itemDictionary = {};
        if (props.initiallySelectedItems && props.selectedItems) {
            console.warn("Please only provide ONE of initiallySelectedItems OR selectedItems as props");
        }
        if (props.initiallySelectedItems) {
            _this._maintainsOwnState = true;
        }
        if (props.pivots && props.pivots.length) {
            _this._selectedPivot = _this._getInitialPivot(props);
        }
        _this.state = {
            isSearchInProgress: false,
            initiallySelectedKeys: [],
            selectedKeys: [],
            filteredItems: [],
            searchResults: [],
            items: [],
            selectedOperatorKey: props.selectedOperatorKey,
            initiallySelectedOperatorKey: props.selectedOperatorKey
        };
        _this._searchTextChangeDelegate = _this._async.debounce(function () { return _this._updateSearchText(); }, SEARCH_TEXT_DEBOUNCE_TIME_MS);
        /**
         * Set _hideLoadingContent to true to indicate to us to not render items/groups that are still in the loading state.
         * This state is set for a short amount of time on initial render to avoid flickering
         * of loading indicators when the items can be loaded quickly.
         */
        _this._hideLoadingContent = true;
        window.setTimeout(function () {
            _this._hideLoadingContent = false;
            if (_this._loadingContentHidden) {
                _this.forceUpdate();
            }
        }, HIDE_LOADING_ITEMS_TIME_MS);
        _this._ariaDescribedById = getId("picklist-described-by");
        return _this;
    }
    PickList.prototype.clearSelection = function () {
        this._onSelectionChanged([], this.state.initiallySelectedOperatorKey);
    };
    PickList.prototype.reset = function () {
        this._onSelectionChanged(this.state.initiallySelectedKeys, this.state.initiallySelectedOperatorKey);
    };
    PickList.prototype.selectAll = function () {
        if (this.props.selectionMode === SelectionMode.multiple) {
            // This is currently only supported for cases where all of the items are passed
            // directly to the pickList and not provided through the onSearch method. We still
            // need to handle the case where the pickList is showing items retrieved via search.
            this._onSelectionChanged(this._getKeysFromItems(this.state.items, this.state.items));
        }
    };
    PickList.prototype.focus = function (forceRefocus) {
        if (!forceRefocus && this._pickListElement && document.activeElement && doesElementContainFocus(this._pickListElement)) {
            // a child of the picklist already has focus, so do not try to refocus.
            return false;
        }
        if (!this._initialItemsLoaded) {
            this._focusOnInitialLoad = true;
        }
        if (this._searchFocusZone) {
            this._searchFocusZone.focus();
            return true;
        }
        else if (this._focusZone) {
            if (this._operatorGroup) {
                this._operatorGroup.focus();
            }
            else if (this._firstRow) {
                return this._focusZone.focusElement(ReactDOM.findDOMNode(this._firstRow));
            }
            else {
                return this._focusZone.focus();
            }
        }
        //No items so return false
        return false;
    };
    /**
     * Scroll to list item by index
     */
    PickList.prototype.scrollToIndex = function (index) {
        if (this._list) {
            this._list.scrollToIndex(index, function () { return DEFAULT_ROW_HEIGHT; });
        }
    };
    /**
     * Scroll to selected item in the list
     */
    PickList.prototype.scrollToSelected = function () {
        var filteredItems = this.state.filteredItems;
        if (filteredItems) {
            var firstSelectedIndex = filteredItems.findIndex(function (item) { return item.isSelected; });
            if (firstSelectedIndex > 0) {
                this.scrollToIndex(firstSelectedIndex);
            }
        }
    };
    PickList.prototype.forceUpdate = function () {
        if (this._isMounted) {
            this._rebuild(this.props);
        }
    };
    PickList.prototype.componentWillMount = function () {
        this._rebuild(this.props);
    };
    PickList.prototype.componentDidMount = function () {
        this._isMounted = true;
        var _a = this.props, pivots = _a.pivots, onSelectedPivotChanged = _a.onSelectedPivotChanged;
        if (onSelectedPivotChanged
            && pivots && pivots.length > 0
            && this._selectedPivot && this._selectedPivot !== pivots[0].id) {
            onSelectedPivotChanged(this._selectedPivot);
        }
    };
    PickList.prototype.componentWillUnmount = function () {
        if (this._itemsAnnouncer) {
            this._itemsAnnouncer.cancel();
        }
        this._isMounted = false;
    };
    PickList.prototype.componentWillReceiveProps = function (newProps) {
        var _this = this;
        var _a = this.props, selectedItems = _a.selectedItems, selectedOperatorKey = _a.selectedOperatorKey;
        var haveGroupsChanged = this._haveGroupsChanged(newProps);
        this._hasNewItems(newProps).then(function (hasNewItems) {
            if (!_this._isMounted) {
                return;
            }
            if (hasNewItems || haveGroupsChanged) {
                _this._rebuild(newProps);
            }
            else if (!_this._arrayEquals(selectedItems, newProps.selectedItems)) {
                var newSelectedKeys_1 = _this._getKeysFromItems(newProps.selectedItems, _this.state.items);
                var filteredItems = _this.state.filteredItems.map(function (item) {
                    item.isSelected = newSelectedKeys_1.some(function (key) { return key === item.key; });
                    return item;
                });
                var _loop_1 = function (itemKey) {
                    var item = _this._itemDictionary[itemKey];
                    item.isSelected = newSelectedKeys_1.some(function (key) { return key === itemKey; });
                };
                for (var itemKey in _this._itemDictionary) {
                    _loop_1(itemKey);
                }
                _this.setState({
                    filteredItems: filteredItems,
                    selectedKeys: newSelectedKeys_1,
                    selectedOperatorKey: newProps.selectedOperatorKey
                });
            }
            else if (selectedOperatorKey !== newProps.selectedOperatorKey) {
                _this.setState({
                    selectedOperatorKey: newProps.selectedOperatorKey
                });
            }
        });
    };
    PickList.prototype.render = function () {
        var _this = this;
        var _a = this.props, getActions = _a.getActions, isSearchable = _a.isSearchable, searchBoxAriaLabel = _a.searchBoxAriaLabel, onSearch = _a.onSearch, className = _a.className, operators = _a.operators, _b = _a.minItemsForSearchBox, minItemsForSearchBox = _b === void 0 ? DEFAULT_MIN_ITEMS_NO_SEARCH : _b, shouldVirtualize = _a.shouldVirtualize, onMouseLeave = _a.onMouseLeave, onMouseEnter = _a.onMouseEnter, itemAriaDescription = _a.itemAriaDescription;
        var _c = this.state, filteredItems = _c.filteredItems, itemsLoading = _c.itemsLoading;
        var items = filteredItems ? filteredItems.slice() : [];
        var hidingAllInitialItems = items.length === 0 && this._loadingContentHidden;
        var pivotBar = this._getPivotBar();
        var currentPivot = this._getCurrentPivot();
        var actions = !hidingAllInitialItems && getActions && getActions(items) || [];
        var searchTextPlaceholder = currentPivot && currentPivot.searchTextPlaceholder || this.props.searchTextPlaceholder;
        var noItemsText = currentPivot && currentPivot.noItemsText || this.props.noItemsText;
        if (currentPivot && currentPivot.getActions) {
            var pivotActions = currentPivot.getActions(items) || [];
            actions = pivotActions.concat(actions);
        }
        this._actionCount = actions.length;
        var actionList = actions.length > 0 &&
            actions.map(function (action, index) {
                return (React.createElement(PickListAction, { action: action, itemIndex: index, key: String(index), setSize: _this._actionCount }));
            });
        var clearSearchTextIconProps = this._searchText ? {
            iconName: 'Clear',
            className: 'clear-filter-input',
            tabIndex: 0,
            onClick: this._onClearInputClick,
            onKeyPress: this._onClearInputKeyPress,
            role: "button",
            ariaLabel: VSSUIResources.FilterTextItemClearLabel
        } : undefined;
        var operatorGroup = items && items.length > 1 && operators &&
            React.createElement(PickListOperatorGroup, { operators: operators, selectedOperatorKey: this.state.selectedOperatorKey, onChange: this._onOperatorChanged, componentRef: function (component) { return _this._operatorGroup = component; } });
        var itemsList = null;
        var onShouldVirtualize = typeof shouldVirtualize === "function" ? function () { return shouldVirtualize(items); } : undefined;
        if (items.length) {
            var role = !!pivotBar ? 'tabpanel' : null;
            var ariaExpanded = !!pivotBar ? true : null;
            var hasItemsClass = items[0].key !== "search-no-results" && "vss-PickList--hasItems";
            var headingIndexes_1 = items.filter(function (item) { return item.isGroupHeader; }).map(function (item) { return items.indexOf(item); });
            itemsList =
                React.createElement("div", { className: css('vss-PickList--items', hasItemsClass), role: 'listbox', "data-is-scrollable": true },
                    React.createElement(List, { role: role, "aria-expanded": ariaExpanded, items: items, onRenderCell: function (item, index) { return _this._onRenderCell(item, index, headingIndexes_1); }, onShouldVirtualize: onShouldVirtualize, componentRef: function (list) { return _this._list = list; } }));
        }
        else if (noItemsText && items.length === 0 && !itemsLoading) {
            itemsList = React.createElement("div", { role: "alert", "aria-live": "assertive", className: 'vss-PickList--noItemsText' }, noItemsText);
        }
        var style = {};
        if (this.props.minWidth) {
            style.minWidth = this.props.minWidth + "px";
        }
        if (this.props.maxWidth) {
            style.maxWidth = this.props.maxWidth + "px";
        }
        if (this.props.width) {
            if (items.length) {
                //If we have items, we don't want the length of the items list
                //to be wider than the containing element, so use a fixed width
                style.width = this.props.width + "px";
            }
            else {
                //If we don't have items, we're fine with the width of the no
                //items text growing beyond the containing element, so use a min-width
                style.minWidth = this.props.width + "px";
            }
        }
        if (this.props.maxHeight) {
            style.maxHeight = this.props.maxHeight + "px";
        }
        return React.createElement("div", { className: css('vss-PickList', className), style: style, onKeyDown: this._onKeyDown, ref: this._resolveRef('_pickListElement'), id: this._id, onMouseEnter: onMouseEnter, onMouseLeave: onMouseLeave },
            itemAriaDescription && React.createElement("div", { className: "vss-PickList--ariaDescription", id: this._ariaDescribedById }, itemAriaDescription),
            React.createElement(FocusZone, { className: "vss-PickList--focusContainer", ref: this._resolveRef('_focusZone'), isInnerZoneKeystroke: function (ev) { return (ev.which === getRTLSafeKeyCode(39 /* right */)); }, direction: FocusZoneDirection.vertical, onActiveElementChanged: this._onActiveElementChanged, isCircularNavigation: true },
                isSearchable && !hidingAllInitialItems && (this.state.items.length >= minItemsForSearchBox || onSearch) &&
                    React.createElement("div", { "data-is-focusable": true, "data-disable-click-on-enter": true, onFocus: this._onSearchContainerFocused },
                        React.createElement(FocusZone, { ref: this._resolveRef('_searchFocusZone'), direction: FocusZoneDirection.horizontal },
                            React.createElement(TextField, { className: 'vss-PickList--searchTextField', ref: this._resolveRef('_textField'), placeholder: searchTextPlaceholder, value: this._searchText, ariaLabel: searchBoxAriaLabel || searchTextPlaceholder, onChanged: this._onSearchTextChanged, onKeyDown: this._onKeyDown, iconProps: clearSearchTextIconProps }))),
                operatorGroup,
                pivotBar,
                itemsList,
                actionList &&
                    React.createElement("div", { className: "vss-PickList--actions", role: "listbox" }, actionList)));
    };
    PickList.prototype._getPivotBar = function () {
        var _this = this;
        var components = [];
        if (this.props.pivots && (this.props.limitSearchToPivot || !this._searchText)) {
            this.props.pivots.forEach(function (pivot) {
                var isSelected = _this._selectedPivot === pivot.id;
                components.push(React.createElement(CommandButton, { id: pivot.id, key: pivot.id, className: css("vss-Picklist--pivotBar-button", isSelected ? "selected" : ""), onClick: _this._onPivotClick.bind(_this, pivot.id), onKeyPress: _this._onPivotKeyPress.bind(_this, pivot.id), ariaLabel: pivot.name, role: 'tab', "aria-selected": isSelected, "aria-posinset": components.length, "aria-setsize": _this.props.pivots.length },
                    React.createElement("span", { className: "innerContainer" },
                        React.createElement("span", { className: "text", style: { maxWidth: pivot.maxWidth || MAX_PIVOT_ITEM_WIDTH } },
                            React.createElement(TooltipHost, { content: pivot.name, overflowMode: TooltipOverflowMode.Parent }, pivot.name)))));
            });
        }
        if (components.length) {
            return (React.createElement("div", { className: "vss-PickList--pivotBar", role: "tablist", "data-is-focusable": true, onFocus: this._onPivotBarFocused },
                React.createElement(FocusZone, { direction: FocusZoneDirection.horizontal, ref: this._resolveRef('_pivotFocusZone') }, components)));
        }
    };
    /**
     * Handle the onKeyPress event on the pivot button.
     */
    PickList.prototype._onPivotKeyPress = function (itemKey, ev) {
        ev.preventDefault();
        if (ev.which === 13 /* enter */) {
            this._updateSelectedPivot(itemKey);
        }
    };
    /**
     * Handles the onClick event on the pivot button.
     */
    PickList.prototype._onPivotClick = function (itemKey, ev) {
        ev.preventDefault();
        this._updateSelectedPivot(itemKey);
    };
    PickList.prototype._updateSelectedPivot = function (itemKey) {
        if (this._selectedPivot !== itemKey) {
            this._selectedPivot = itemKey;
            this.props.onSelectedPivotChanged && this.props.onSelectedPivotChanged(itemKey);
            if (this.props.limitSearchToPivot && this._searchText) {
                this._updateSearchText();
            }
            this.scrollToIndex(0);
            this.forceUpdate();
        }
    };
    PickList.prototype._getInitialPivot = function (_a) {
        var selectedItems = _a.selectedItems, initiallySelectedItems = _a.initiallySelectedItems, getListItem = _a.getListItem, groups = _a.groups, pivots = _a.pivots;
        var firstSelectedItem = selectedItems && selectedItems[0] ||
            initiallySelectedItems && initiallySelectedItems[0];
        if (firstSelectedItem && getListItem) {
            var groupKey_1 = getListItem(firstSelectedItem).groupKey;
            if (groupKey_1) {
                var group = groups.filter(function (g) { return g.key === groupKey_1; })[0];
                if (group && group.pivotId) {
                    return group.pivotId;
                }
            }
        }
        return pivots[0].id;
    };
    PickList.prototype._getCurrentPivot = function () {
        var _this = this;
        return this.props.pivots && this.props.pivots.filter(function (p) { return p.id === _this._selectedPivot; })[0];
    };
    PickList.prototype._rebuild = function (props) {
        var _this = this;
        if (this.props.items !== props.items && this._itemsAnnouncer) {
            this._itemsAnnouncer.cancel();
            this._itemsAnnouncer = null;
        }
        if (this._isThenable(props.items)) {
            this._buildFromItems(props, [], true);
            props.items.then(function (resolvedItems) {
                // Ensure that the items promise prop is still current
                if (_this.props.items === props.items) {
                    _this._buildFromItems(props, resolvedItems, false);
                }
            }, function (rejectReason) {
                // Ensure that the items promise prop is still current
                if (_this.props.items === props.items) {
                    _this._buildFromItems(props, [], false);
                    console.warn("Failed to get items for PickList: " + ((rejectReason && rejectReason.message) ? rejectReason.message : rejectReason));
                }
            });
            if (!this._itemsAnnouncer) {
                var announcer = Utils_Accessibility.ProgressAnnouncer.forPromise(props.items, {
                    announceStartMessage: VSSUIResources.AnnounceLoadingItems,
                    announceEndMessage: VSSUIResources.AnnounceFinishedLoadingItems,
                    announceErrorMessage: VSSUIResources.AnnounceErrorLoadingItems
                });
                this._itemsAnnouncer = announcer;
            }
        }
        else {
            this._buildFromItems(props, props.items || [], false);
        }
    };
    PickList.prototype._buildFromItems = function (props, items, loading) {
        var _this = this;
        var initialItemsPreviouslyLoaded = this._initialItemsLoaded;
        var initiallySelectedKeys = this.state.initiallySelectedKeys;
        var selectedKeys = this.state.selectedKeys;
        var selectedOperatorKey = this.state.selectedOperatorKey;
        if (!loading && !initialItemsPreviouslyLoaded && items.length) {
            if (props.initiallySelectedItems) {
                initiallySelectedKeys = this._getKeysFromItems(props.initiallySelectedItems, items);
                selectedKeys = initiallySelectedKeys;
            }
            this._initialItemsLoaded = true;
        }
        if (!this._maintainsOwnState) {
            selectedKeys = this._getKeysFromItems(props.selectedItems, items);
            selectedOperatorKey = props.selectedOperatorKey;
        }
        var _a = this.state, searchResults = _a.searchResults, isSearchInProgress = _a.isSearchInProgress, itemsLoading = _a.itemsLoading;
        var wrappedItems = this._wrapItems(items, selectedKeys);
        var searchWrappedItems = this._wrapItems(searchResults, selectedKeys, 'search-results');
        var groupDictionary = this._groupItems(wrappedItems);
        var groupDictionaryWithSearchResults = this._groupItems(searchWrappedItems, groupDictionary);
        var filteredGroupDictionary = this._filterByText(this._searchText, groupDictionaryWithSearchResults);
        var groupedItems = this._buildGroupedItems(this._searchText, isSearchInProgress, searchResults, filteredGroupDictionary, props.groups, selectedKeys, loading);
        // Only announce if items were loading and just finished
        if (itemsLoading && this._searchText) {
            this._announceSearchResults(filteredGroupDictionary, isSearchInProgress, loading);
        }
        this.setState({ items: items, groupDictionary: groupDictionary, initiallySelectedKeys: initiallySelectedKeys, filteredItems: groupedItems, selectedKeys: selectedKeys, itemsLoading: loading, selectedOperatorKey: selectedOperatorKey }, function () {
            if (!initialItemsPreviouslyLoaded && _this._initialItemsLoaded && _this._focusOnInitialLoad) {
                // We want to force a re-focus, just in case something else in the PickList has grabbed focus
                // before the initial items have been loaded. Otherwise, we can get in a state where the
                // first action has focus instead of the first item or search box.
                _this.focus(true);
            }
        });
    };
    PickList.prototype._announceSearchResults = function (filteredGroupDictionary, isSearchInProgress, itemsLoading) {
        if (!isSearchInProgress && !itemsLoading) {
            var resultCount = 0;
            for (var key in filteredGroupDictionary) {
                resultCount += filteredGroupDictionary[key].length;
            }
            Utils_Accessibility.announce(resultCount > 0 ? format(VSSUIResources.AnnounceFilterResultCount, resultCount) :
                VSSUIResources.NoFilterResults, true, ANNOUNCE_DELAY);
        }
    };
    PickList.prototype._wrapItems = function (items, selectedKeys, overrideGroupKey) {
        var _this = this;
        var _a = this.props, getListItem = _a.getListItem, onSearch = _a.onSearch;
        var pickListItems = items.map(function (item) {
            var pickListItem = getListItem ? getListItem(item) : {
                name: item,
                key: item
            };
            if (!pickListItem.groupKey) {
                pickListItem.groupKey = 'undefined-key';
            }
            var pickListItemInternal = tslib_1.__assign({}, pickListItem, { isSelected: selectedKeys.some(function (key) { return key === pickListItem.key; }), data: item });
            if (overrideGroupKey) {
                pickListItemInternal.groupKey = overrideGroupKey;
            }
            _this._itemDictionary[pickListItem.key] = pickListItemInternal;
            return pickListItemInternal;
        });
        return pickListItems;
    };
    PickList.prototype._groupItems = function (items, initialGroupDictionary) {
        var groupDictionary = {};
        if (initialGroupDictionary) {
            for (var key in initialGroupDictionary) {
                groupDictionary[key] = initialGroupDictionary[key].slice();
            }
        }
        if (items && items.length) {
            items.forEach(function (item) {
                var groupKey = item.groupKey;
                if (!groupDictionary[groupKey]) {
                    groupDictionary[groupKey] = [];
                }
                groupDictionary[groupKey].push(item);
            });
        }
        return groupDictionary;
    };
    PickList.prototype._filterByText = function (searchText, groupDictionary) {
        var filteredGroupDictionary = {};
        if (searchText) {
            var onFilterItemByText_1 = this.props.onFilterItemByText || this._defaultFilterItemByText;
            var _loop_2 = function (key) {
                if (this_1._selectedPivot && this_1.props.limitSearchToPivot && key !== 'search-results') {
                    var group = this_1.props.groups.filter(function (g) { return g.key === key; })[0];
                    if (!group || group.pivotId !== this_1._selectedPivot) {
                        return "continue";
                    }
                }
                var groupItems = groupDictionary[key];
                var filteredGroupItems = groupItems.filter(function (item) {
                    return item.isSeparator || onFilterItemByText_1(searchText, item, item.data);
                });
                filteredGroupDictionary[key] = filteredGroupItems;
            };
            var this_1 = this;
            for (var key in groupDictionary) {
                _loop_2(key);
            }
        }
        else {
            filteredGroupDictionary = groupDictionary;
        }
        return filteredGroupDictionary;
    };
    PickList.prototype._buildGroupedItems = function (searchText, isSearchInProgress, searchResults, groupDictionary, groups, selectedKeys, itemsLoading) {
        var _this = this;
        var groupedItems = [];
        var _a = this.props, itemsLoadingText = _a.itemsLoadingText, onSearch = _a.onSearch, onHasMoreSearchResults = _a.onHasMoreSearchResults, searchResultsGroupName = _a.searchResultsGroupName, searchResultsLoadingText = _a.searchResultsLoadingText, selectionMode = _a.selectionMode, showSelectAll = _a.showSelectAll;
        var currentPivot = this._getCurrentPivot();
        var searchNoResultsText = format((currentPivot && currentPivot.searchNoResultsText) ||
            this.props.searchNoResultsText ||
            VSSUIResources.NoFilterResults, searchText || "");
        var allGroups = [{ key: 'undefined-key' }].concat(groups || []);
        if (onSearch) {
            allGroups.push({
                key: 'search-results',
                name: searchResultsGroupName
            });
        }
        var needsSeparator = false;
        var groupLoadingIndicatorAdded = false;
        this._loadingContentHidden = false;
        this._searchingContentHidden = false;
        allGroups.forEach(function (group) {
            var groupKey = group.key;
            var itemsForGroup = groupDictionary[groupKey];
            var isGroupVisible = !_this._selectedPivot ||
                _this._selectedPivot === group.pivotId ||
                _this._searchText && !_this.props.limitSearchToPivot ||
                group.key === 'search-results';
            if (_this._selectedPivot && !group.pivotId && groupKey !== 'search-results' && itemsForGroup && itemsForGroup.length) {
                warn('Picklist pivots are being used and items were provided with no pivot id.  All items should be within a group with a valid pivot id');
            }
            if (itemsForGroup && itemsForGroup.length && isGroupVisible) {
                if ((needsSeparator && groupKey !== 'search-results') || group.name) {
                    // Don't add separator in front of search results, this is a special group with online results
                    // that don't need to be separated from local results.
                    groupedItems.push({
                        isSeparator: true,
                        isGroupHeader: true,
                        name: group.name,
                        key: 'group-name-' + group.key,
                        data: undefined,
                        isSelected: false,
                        onRender: group.onRenderGroupHeader ? function () { return group.onRenderGroupHeader(group); } : undefined
                    });
                }
                groupedItems = groupedItems.concat(itemsForGroup);
                needsSeparator = true;
                if (group.isLoading && !(_this._searchText && onSearch)) {
                    groupedItems.push({
                        isSeparator: true,
                        isLoadingIndicator: true,
                        name: group.loadingMessage || itemsLoadingText || VSSUIResources.Loading,
                        key: 'items-loading-' + group.key,
                        data: undefined,
                        isSelected: false,
                        onRender: function (element) {
                            _this._requestGroupLoad(group.key);
                            return element;
                        },
                    });
                }
                else if (groupKey === 'search-results' && onHasMoreSearchResults && onHasMoreSearchResults(searchResults)) {
                    groupedItems.push({
                        isSeparator: true,
                        isLoadingIndicator: true,
                        name: searchResultsLoadingText || VSSUIResources.Searching,
                        key: 'search-loading-' + group.key,
                        data: undefined,
                        isSelected: false,
                        onRender: function (element) {
                            if (!isSearchInProgress) {
                                _this._updateSearchText(searchResults);
                            }
                            return element;
                        },
                    });
                }
            }
            else if (groupKey === 'search-results') {
                if (searchText && searchText.length && onSearch) {
                    if (isSearchInProgress && _this._hideSearchingContent) {
                        _this._searchingContentHidden = true;
                    }
                    else if (isSearchInProgress || (!isSearchInProgress && groupedItems.length === 0)) {
                        if (needsSeparator || group.name) {
                            groupedItems.push({
                                isSeparator: true,
                                isGroupHeader: true,
                                name: group.name || "",
                                key: 'group-name-' + group.key,
                                isSelected: false
                            });
                        }
                        if (isSearchInProgress) {
                            groupedItems.push({
                                isSeparator: true,
                                isLoadingIndicator: true,
                                name: searchResultsLoadingText || VSSUIResources.Searching,
                                key: 'search-loading-' + group.key,
                                data: undefined,
                                isSelected: false
                            });
                        }
                        else if (_this.state.searchError) {
                            groupedItems.push({
                                isSeparator: true,
                                messageLevel: 2 /* error */,
                                name: _this.state.searchError.message,
                                key: 'search-error-' + group.key,
                                data: undefined,
                                isSelected: false
                            });
                        }
                        else {
                            groupedItems.push({
                                isSeparator: true,
                                messageLevel: 1 /* info */,
                                name: searchNoResultsText,
                                key: 'search-no-results-' + group.key,
                                data: undefined,
                                isSelected: false
                            });
                        }
                        needsSeparator = true;
                    }
                }
            }
            else if ((itemsLoading && group.key === 'undefined-key' && !_this._selectedPivot) ||
                (group.isLoading && isGroupVisible && !(_this._searchText && onSearch))) {
                if (_this._hideLoadingContent) {
                    _this._loadingContentHidden = true;
                }
                else if (!groupLoadingIndicatorAdded) {
                    // Only add loading spinner for the first loading group rather than n spinners.
                    groupLoadingIndicatorAdded = true;
                    if (needsSeparator || group.name) {
                        groupedItems.push({
                            isSeparator: true,
                            isGroupHeader: true,
                            name: group.name,
                            key: 'group-name-' + group.key,
                            data: undefined,
                            isSelected: false
                        });
                    }
                    groupedItems.push({
                        isSeparator: true,
                        isLoadingIndicator: true,
                        name: group.loadingMessage || itemsLoadingText || VSSUIResources.Loading,
                        key: 'items-loading-' + group.key,
                        data: undefined,
                        isSelected: false,
                        onRender: function (element) {
                            _this._requestGroupLoad(group.key);
                            return element;
                        },
                    });
                    needsSeparator = true;
                }
            }
        });
        // No custom onSearch but a filter was supplied which matched no values. Add
        // a message indicating this.
        if (searchText && searchText.length && !isSearchInProgress && groupedItems.length === 0) {
            groupedItems.push({
                isSeparator: true,
                messageLevel: 1 /* info */,
                name: searchNoResultsText,
                key: 'search-no-results',
                data: undefined,
                isSelected: false
            });
        }
        if (selectionMode === SelectionMode.multiple && showSelectAll && groupedItems.length > 1) {
            var isSelected = selectedKeys.indexOf(SELECT_ALL_ITEM_KEY) !== -1;
            var selectAllItem = {
                key: SELECT_ALL_ITEM_KEY,
                name: VSSUIResources.PickListSelectAllActionText,
                isSelected: !!isSelected
            };
            groupedItems.splice(0, 0, selectAllItem);
            this._itemDictionary[SELECT_ALL_ITEM_KEY] = selectAllItem;
        }
        return groupedItems;
    };
    PickList.prototype._requestGroupLoad = function (groupKey) {
        var _this = this;
        if (this.props.onGroupLoadRequest &&
            !this._groupLoadRequested[groupKey] &&
            !this.state.itemsLoading) {
            this._groupLoadRequested[groupKey] = true;
            this.props.onGroupLoadRequest(groupKey)
                .then(function () {
                // Reset the flag, so we can request it again (another page) if group is still loading
                delete _this._groupLoadRequested[groupKey];
            });
        }
    };
    PickList.prototype._getOnSearchPromise = function (searchText, resultsToContinue) {
        var onSearch = this.props.onSearch;
        var searchItems = onSearch ? onSearch(searchText, this.state.items, this._selectedPivot, resultsToContinue) : [];
        return Promise.resolve(searchItems);
    };
    PickList.prototype._onSingleSelectionChanged = function (key, disabled) {
        if (disabled) {
            return false;
        }
        return this._onSelectionChanged([key]);
    };
    PickList.prototype._onMultipleSelectionChanged = function (key, isSelected, disabled) {
        if (disabled) {
            return false;
        }
        var newSelectedKeys = [];
        if (key === SELECT_ALL_ITEM_KEY) {
            if (isSelected) {
                newSelectedKeys = this._getKeysFromItems(this.state.items, this.state.items);
            }
            else {
                newSelectedKeys = [];
            }
        }
        else {
            newSelectedKeys = this.state.selectedKeys.filter(function (key) { return key !== SELECT_ALL_ITEM_KEY; });
            if (isSelected) {
                newSelectedKeys.push(key);
            }
            else {
                newSelectedKeys.splice(newSelectedKeys.indexOf(key), 1);
            }
            if (newSelectedKeys.length === this.state.items.length) {
                newSelectedKeys = [SELECT_ALL_ITEM_KEY].concat(newSelectedKeys);
            }
        }
        return this._onSelectionChanged(newSelectedKeys);
    };
    PickList.prototype._getIndicators = function (pickListItem, indicators) {
        return indicators ? indicators.map(function (indicator) {
            if (pickListItem.data) {
                return indicator.getItemIndicator(pickListItem.data);
            }
        }) : [];
    };
    PickList.prototype._getKeysFromItems = function (items, allItems) {
        var _this = this;
        if (items) {
            var selectedKeys = [];
            if (this.props.getListItem) {
                selectedKeys = items.map(function (item) {
                    return _this.props.getListItem(item).key;
                });
            }
            else {
                selectedKeys = items;
            }
            if (allItems && selectedKeys.length === allItems.length) {
                selectedKeys = [SELECT_ALL_ITEM_KEY].concat(selectedKeys);
            }
            return selectedKeys;
        }
        return [];
    };
    PickList.prototype._hasNewItems = function (newProps) {
        var _this = this;
        if (this._isThenable(newProps.items)) {
            return newProps.items["then"](function (value) {
                return !_this._arrayEquals(value, _this.state.items);
            });
        }
        else {
            return new Promise(function (resolve) { resolve(); }).then(function () { return !_this._arrayEquals(_this.state.items, newProps.items); });
        }
    };
    PickList.prototype._haveGroupsChanged = function (newProps) {
        var newGroups = newProps.groups;
        var oldGroups = this.props.groups;
        if (!newGroups && !oldGroups) {
            return false;
        }
        else if (!newGroups || !oldGroups) {
            return true;
        }
        else {
            if (newGroups.length !== oldGroups.length) {
                return true;
            }
            else {
                return newGroups.some(function (newGroup, index) {
                    return oldGroups[index].key !== newGroup.key ||
                        oldGroups[index].name !== newGroup.name ||
                        oldGroups[index].isLoading !== newGroup.isLoading;
                });
            }
        }
    };
    PickList.prototype._isThenable = function (obj) {
        return obj && typeof obj["then"] === "function";
    };
    PickList.prototype._arrayEquals = function (array1, array2) {
        if (array1 && array2) {
            if (array1.length !== array2.length) {
                return false;
            }
            for (var index = 0; index < array1.length; index++) {
                if (array1[index] !== array2[index]) {
                    return false;
                }
            }
            return true;
        }
        else {
            return array1 === array2;
        }
    };
    return PickList;
}(BaseComponent));
export { PickList };
var PickListRow = /** @class */ (function (_super) {
    tslib_1.__extends(PickListRow, _super);
    function PickListRow() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._onKeyDown = function (ev) {
            if (ev.which === 13 /* enter */ ||
                (ev.which === 32 /* space */ && _this.props.spaceBarTriggersClick)) {
                var target = ev.target;
                if (target === _this._rowDiv) {
                    if (_this.props.href) {
                        if (ev.ctrlKey || ev.shiftKey) {
                            // Shift-enter or Ctrl-enter. Open in new window/tab
                            window.open(_this.props.href, "_blank", "noopener");
                        }
                        else {
                            // Regular click - invoke handler
                            if (!_this.props.onKeyboardEnter()) {
                                // Not handled, perform navigation
                                window.location.href = _this.props.href;
                            }
                        }
                    }
                    else {
                        // No hyperlink. Always invoke the change handler.
                        _this.props.onKeyboardEnter();
                    }
                }
            }
            if (ev.which === 37 /* left */) {
                var target = ev.target;
                if (target.id === getFirstFocusable(_this._rowDiv, target, true).id) {
                    _this._rowDiv.focus();
                }
            }
        };
        return _this;
    }
    PickListRow.prototype.render = function () {
        return React.createElement("div", { role: 'option', "aria-selected": this.props.isSelected, "aria-label": this.props.ariaLabel, "aria-describedby": this.props.ariaDescribedBy, "aria-setsize": this.props.setSize, "aria-posinset": this.props.itemIndex + 1, className: css('vss-PickList--row', this.props.disabled && 'disabled', this.props.firstInGroup && 'first-in-group'), ref: this._resolveRef('_rowDiv'), "data-is-focusable": true, onKeyDown: this._onKeyDown, "data-selection-key": this.props.dataSelectionKey },
            React.createElement(FocusZone, { direction: FocusZoneDirection.horizontal }, this.props.children));
    };
    return PickListRow;
}(BaseComponent));
var PickListSelectableElement = /** @class */ (function (_super) {
    tslib_1.__extends(PickListSelectableElement, _super);
    function PickListSelectableElement() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._onClick = function (ev) {
            var _a = _this.props, item = _a.item, onSelectionChanged = _a.onSelectionChanged;
            if (onSelectionChanged) {
                if (item.href && (ev.ctrlKey || ev.shiftKey || ev.button === 1 || ev.button === 2)) {
                    // No-op. This is an item that has a hyperlink AND this is a
                    // ctrl/shift-click event or middle/right-click event. Let the browser handle this
                }
                else {
                    if (onSelectionChanged()) {
                        ev.preventDefault();
                    }
                }
            }
        };
        return _this;
    }
    PickListSelectableElement.prototype.render = function () {
        var _a = this.props, item = _a.item, indicators = _a.indicators;
        var itemContent = React.createElement("span", { className: "vss-PickList--selectableElementButton-text" }, this.props.onRenderItemText ? this.props.onRenderItemText(item, item.data) : item.name);
        var children = [];
        if (item.iconProps) {
            var className = 'vss-PickList--selectableElementButtonIcon';
            if (item.iconProps.className) {
                className = css(className, item.iconProps.className);
            }
            children.push(React.createElement(VssIcon, tslib_1.__assign({ key: 'icon' }, item.iconProps, { className: className })));
        }
        children.push(React.createElement("div", { key: 'text-container', className: 'vss-PickList--selectableElementButton-textContainer' },
            React.createElement(TooltipHost, { content: item.tooltipText || item.name, overflowMode: item.tooltipText && item.tooltipText !== item.name ? undefined : TooltipOverflowMode.Parent }, itemContent)));
        var disabled = typeof item.disabled === "function" ? item.disabled() : item.disabled;
        return React.createElement("div", { className: css('vss-PickList--element', 'vss-PickList--selectableElement', { 'is-selected': item.isSelected, 'disabled': disabled }) },
            item.href ?
                React.createElement("a", { className: 'vss-PickList--selectableElementButton', tabIndex: -1, "data-is-focusable": false, href: item.href, onClick: this._onClick }, children) :
                React.createElement("div", { className: 'vss-PickList--selectableElementButton', role: 'button', onClick: this._onClick, tabIndex: -1, "data-is-focusable": false }, children),
            React.createElement(ItemIndicatorSet, { indicators: indicators, onClick: this._onClick }));
    };
    return PickListSelectableElement;
}(BaseComponent));
var PickListCheckboxElement = /** @class */ (function (_super) {
    tslib_1.__extends(PickListCheckboxElement, _super);
    function PickListCheckboxElement() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._onClick = function (ev) {
            if (!ev.defaultPrevented) {
                _this.props.onSelectionChanged(!_this.props.item.isSelected);
                ev.preventDefault();
            }
        };
        return _this;
    }
    PickListCheckboxElement.prototype.render = function () {
        var _a = this.props, item = _a.item, indicators = _a.indicators;
        var icon = item.iconProps ? React.createElement(VssIcon, tslib_1.__assign({ className: 'vss-PickList--multiSelectIcon' }, item.iconProps)) : null;
        var disabled = typeof item.disabled === "function" ? item.disabled() : item.disabled;
        var label = React.createElement("div", { className: 'vss-PickList--multiSelectTextContainer' },
            React.createElement(TooltipHost, { content: item.name, overflowMode: TooltipOverflowMode.Parent },
                React.createElement("span", { className: css('vss-PickList--multiSelectText', disabled && 'disabled') }, item.name)));
        var indicatorSet = React.createElement(ItemIndicatorSet, { indicators: indicators });
        return React.createElement("div", { className: css('vss-PickList--element', 'vss-PickList--multiSelectableElement'), onClick: this._onClick },
            React.createElement(Checkbox, { className: 'vss-PickList--checkbox', checked: item.isSelected, disabled: disabled, ariaLabel: item.name }),
            icon,
            label,
            indicatorSet);
    };
    return PickListCheckboxElement;
}(BaseComponent));
var PickListAction = /** @class */ (function (_super) {
    tslib_1.__extends(PickListAction, _super);
    function PickListAction() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._onClick = function (ev) {
            var action = _this.props.action;
            if (action.onClick) {
                action.onClick(ev);
            }
        };
        _this._onKeyDown = function (ev) {
            if (!_this.props.action.disabled && (ev.which === 13 /* enter */ || ev.which === 32 /* space */)) {
                var action = _this.props.action;
                var ctrlOrShift = ev.ctrlKey || ev.shiftKey;
                // Invoke the click handler unless we have an href and ctrl/shift was pressed
                if (action.onClick && (!action.href || !ctrlOrShift)) {
                    _this.props.action.onClick(ev);
                }
                else if (action.href) {
                    if (ctrlOrShift) {
                        window.open(action.href, "_blank", "noopener");
                    }
                    else {
                        window.location.href = action.href;
                    }
                }
            }
        };
        return _this;
    }
    PickListAction.prototype.render = function () {
        var _a = this.props, setSize = _a.setSize, itemIndex = _a.itemIndex;
        var _b = this.props.action, href = _b.href, iconProps = _b.iconProps, name = _b.name, disabled = _b.disabled, ariaLabel = _b.ariaLabel, tooltipText = _b.tooltipText;
        var actionContents = (React.createElement(CommandButton, { className: 'vss-PickList--selectableElementButton', onClick: this._onClick, disabled: disabled, href: href, tabIndex: -1, "data-is-focusable": false, styles: { textContainer: { textAlign: "left" } }, iconProps: iconProps ? VssIcon.getIconProps(iconProps, 'vss-PickList--selectableElementButtonIcon') : undefined }, name));
        if (tooltipText) {
            actionContents = (React.createElement(TooltipHost, { content: tooltipText, hostClassName: "vss-PickList--action-tooltip" }, actionContents));
        }
        return React.createElement("div", { "aria-label": ariaLabel !== undefined ? ariaLabel : name, "aria-posinset": itemIndex + 1, "aria-setsize": setSize, className: "vss-PickList--action", "data-is-focusable": true, role: "option", onKeyDown: this._onKeyDown }, actionContents);
    };
    return PickListAction;
}(BaseComponent));
var PickListOperatorGroup = /** @class */ (function (_super) {
    tslib_1.__extends(PickListOperatorGroup, _super);
    function PickListOperatorGroup() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._onChange = function (ev, option) {
            _this.props.onChange(option.key);
        };
        return _this;
    }
    PickListOperatorGroup.prototype.render = function () {
        var _this = this;
        var _a = this.props, operators = _a.operators, onChange = _a.onChange, selectedOperatorKey = _a.selectedOperatorKey;
        var options = operators.map(function (operator) { return { text: operator.name, key: operator.key }; });
        return React.createElement(ChoiceGroup, { selectedKey: selectedOperatorKey, options: options, onChange: this._onChange, componentRef: function (component) { return _this._choiceGroup = component; } });
    };
    PickListOperatorGroup.prototype.focus = function () {
        if (this._choiceGroup) {
            this._choiceGroup.focus();
        }
    };
    return PickListOperatorGroup;
}(BaseComponent));
