import * as tslib_1 from "tslib";
import "../../FabricInit";
import * as React from 'react';
import { BaseComponent, css, getId, createRef, findIndex } from 'office-ui-fabric-react/lib/Utilities';
import { FocusZone, FocusZoneDirection } from 'office-ui-fabric-react/lib/FocusZone';
import { CommandButton } from 'office-ui-fabric-react/lib/Button';
import { TooltipHost, TooltipOverflowMode } from '../../Tooltip';
import { Dropdown, DropdownMenuItemType } from 'office-ui-fabric-react/lib/Dropdown';
import { childrenOfType } from '../../Utilities/ChildrenUtils';
import { PivotBarItem } from '../../Components/PivotBar/PivotBarItem';
import { Accordion } from '../../Accordion';
import { DynamicLink } from '../../Dynamic';
import { HubTileRegion } from '../../HubHeader';
import { FilterBar } from '../../FilterBar';
import { FILTER_APPLIED_EVENT } from '../../Utilities/Filter';
import { VssIcon } from '../../VssIcon';
import { ObservableArray, ObservableLike } from '../../Core/Observable';
import { ContributedItemArray, ContributedItemWithIdArray } from '../../Utilities/ItemContribution';
import { VIEW_OPTIONS_CHANGE_EVENT } from '../../Utilities/ViewOptions';
import { arrayEquals } from '../../Utilities/Internal';
import { getFriendlyDisplayValue } from '../../Utilities/FriendlyNumber';
import { PivotCommandBar } from '../../Components/PivotBar/PivotCommandBar';
import * as VSSUI_Resources from '../../VSS.Resources.VSSUI';
import "./PivotBar.css";
var MAX_PIVOT_ITEM_WIDTH = 340;
/**
 * How long after load to wait (in milliseconds) to see if the selected pivot gets contributed
 * before giving up and selecting a different pivot instead.
 */
export var MAX_PIVOT_WAIT = 1000;
/**
 * A PivotBar combines a Pivot component (tabs and tabbed content) along with a set of
 * commands (rendered after the pivots) and view actions (rendered on the far right).
 */
var PivotBar = /** @class */ (function (_super) {
    tslib_1.__extends(PivotBar, _super);
    function PivotBar(props) {
        var _this = _super.call(this, props) || this;
        _this._subscribedToPivotObservables = false;
        /** True if we have successfully found the selected pivot after load */
        _this._pivotSelected = false;
        /** Remembers which pivot items have been rendered */
        _this._pivotItemsRendered = {};
        _this._pivotHeader = createRef();
        _this._onFilterBarDismissClicked = function () {
            var _a = _this.props, showFilterBarViewOptionKey = _a.showFilterBarViewOptionKey, viewOptions = _a.viewOptions;
            if (viewOptions && showFilterBarViewOptionKey) {
                _this.props.viewOptions.setViewOption(showFilterBarViewOptionKey, false);
            }
            if (_this._commandBar) {
                _this._commandBar.focus();
            }
            _this.setState({ showFilterBar: false });
        };
        _this._onDropDownSelected = function (option) {
            var itemKey = option.key;
            _this._updateSelectedItem(itemKey, null);
        };
        _this._renderPivotItem = function (item, index) {
            var itemKey = item.itemKey;
            var isSelected = _this.state.selectedPivot === itemKey;
            return (React.createElement(DynamicLink, { url: item.url, className: css('vss-PivotBar--itemLink', item.child && item.child.props.headerClassName), tabIndex: -1, role: 'presentation', "data-is-focusable": false, key: itemKey },
                React.createElement(CommandButton, { id: item.id, className: css("vss-PivotBar--button", isSelected ? "selected" : ""), onClick: _this._onPivotClick.bind(_this, itemKey), onKeyPress: _this._onKeyPress.bind(_this, itemKey), ariaLabel: item.ariaLabel, role: 'tab', "aria-selected": isSelected, "aria-posinset": index + 1, "aria-setsize": _this.state.items.length }, _this._renderPivotItemContent(item))));
        };
        _this._onShowFilterBarClicked = function () {
            _this.setState({
                showFilterBar: !_this.state.showFilterBar
            });
        };
        _this._onFilterApplied = function () {
            var hasAppliedFilter = _this.props.filter.hasChangesToReset();
            if (hasAppliedFilter !== _this.state.hasAppliedFilter) {
                _this.setState({
                    hasAppliedFilter: hasAppliedFilter
                });
            }
        };
        _this._onContributedPivotsChanged = function () {
            _this.setState({
                items: _this._getPivotBarItems(_this.props)
            });
            if (!_this._pivotSelected) {
                _this.setState(_this._buildInitialPivotState());
            }
            else if (_this.state.selectedPivot && !_this.getSelectedPivot()) {
                // our selected pivot has disappeared, choose a new one
                var selectedPivot = void 0;
                if (_this.props.initialSelectedPivot && _this._getPivotBarItemByItemKey(_this.props.initialSelectedPivot)) {
                    selectedPivot = _this.props.initialSelectedPivot;
                }
                else if (_this.state.items && _this.state.items.length > 0) {
                    selectedPivot = _this.state.items[0].itemKey;
                }
                if (selectedPivot) {
                    _this.selectPivot(selectedPivot);
                }
            }
        };
        _this._onViewOptionChanged = function (changedOptions) {
            var showFilterBarViewOptionKey = _this.props.showFilterBarViewOptionKey;
            var viewActions = _this.state.viewActions;
            var needsUpdate = false;
            if (showFilterBarViewOptionKey in changedOptions) {
                _this._focusFilterBar = true;
                needsUpdate = true;
            }
            if (!needsUpdate) {
                var allViewActions_1 = [];
                if (viewActions) {
                    viewActions.forEach(function (va) { return allViewActions_1.push.apply(allViewActions_1, va.value); });
                }
                var _loop_1 = function (changedOptionKey) {
                    if (findIndex(allViewActions_1, function (a) { return a.key === changedOptionKey; }) >= 0) {
                        needsUpdate = true;
                        return "break";
                    }
                };
                for (var changedOptionKey in changedOptions) {
                    var state_1 = _loop_1(changedOptionKey);
                    if (state_1 === "break")
                        break;
                }
            }
            if (needsUpdate) {
                _this.forceUpdate();
            }
        };
        _this._makeIPivotBarItemFromContribution = function (item, index) {
            return {
                contributedItem: item,
                id: _this._makeContributedPivotBarItemId(item, index),
                itemKey: item.itemKey || item.id,
                groupKey: item.groupKey,
                name: item.text,
                order: item.order,
                ariaLabel: item.ariaLabel,
                onRenderFilterBar: item.renderFilterBar,
                commandProviders: item.commandProviders,
                viewActionProviders: item.viewActionProviders,
                // iconProps: null,
                badgeCount: item.badgeCount,
                url: item['url'],
                hidden: item.hidden,
                onBeforePivotChange: item.onBeforePivotChange
            };
        };
        _this._getPivotBarItemId = function (item) {
            return item.contributedItem ? item.contributedItem.id : (item.itemKey || item.id);
        };
        _this._id = getId('PivotBar');
        _this._setupPivotObservables(_this.props);
        _this._actionsManager = new PivotActionsManager();
        _this._disposables.push(_this._actionsManager);
        _this.state = {
            items: _this._getPivotBarItems(_this.props)
        };
        // _buildInitialPivotState needs to have this.state.items set.
        _this.state = tslib_1.__assign({ hasAppliedFilter: _this.props.filter ? _this.props.filter.hasChangesToReset() : false, items: _this.state.items }, _this._buildInitialPivotState());
        return _this;
    }
    PivotBar.prototype.focus = function (focusItem) {
        if (this._commandBar) {
            switch (focusItem) {
                case 0 /* commands */:
                    this._commandBar.focus();
                    break;
            }
        }
    };
    PivotBar.prototype.selectPivot = function (pivotKey) {
        if (!this._getPivotBarItemByItemKey(pivotKey)) {
            return false;
        }
        this._updateSelectedItem(pivotKey, null);
        return true;
    };
    /**
     * Gets the currently selected pivot
     */
    PivotBar.prototype.getSelectedPivot = function () {
        return this._getPivotBarItemByItemKey(this.state.selectedPivot);
    };
    /**
     * This gets executed when a container component re-renders PivotBar.
     * @param nextProps New props.
     */
    PivotBar.prototype.componentWillReceiveProps = function (nextProps) {
        var _this = this;
        if (!arrayEquals(this.props.pivotProviders, nextProps.pivotProviders)) {
            this._setupPivotObservables(nextProps);
        }
        // Get the pivot items from the child elements
        var items = this._getPivotBarItems(nextProps);
        this.setState(function (prevState, props) {
            return tslib_1.__assign({ items: items }, _this._buildInitialPivotState(props, items, true));
        });
    };
    PivotBar.prototype.componentDidMount = function () {
        if (this.props.viewOptions && this.props.showFilterBarViewOptionKey) {
            this._events.on(this.props.viewOptions, VIEW_OPTIONS_CHANGE_EVENT, this._onViewOptionChanged);
        }
        if (this.props.filter) {
            this._events.on(this.props.filter, FILTER_APPLIED_EVENT, this._onFilterApplied);
        }
        this._updateHeaderTopIfNeeded();
        this._subscribeToPivotObservables();
    };
    PivotBar.prototype.componentDidUpdate = function () {
        this._updateHeaderTopIfNeeded();
        if (this.props.onRenderComplete) {
            this.props.onRenderComplete();
        }
    };
    PivotBar.prototype.componentWillUnmount = function () {
        if (this.props.viewOptions && this.props.showFilterBarViewOptionKey) {
            this._events.off(this.props.viewOptions, VIEW_OPTIONS_CHANGE_EVENT, this._onViewOptionChanged);
        }
        if (this.props.filter) {
            this._events.off(this.props.filter, FILTER_APPLIED_EVENT, this._onFilterApplied);
        }
        if (this._contributedPivotWait) {
            clearTimeout(this._contributedPivotWait);
            this._contributedPivotWait = null;
        }
        this._unsubscribeFromPivotObservables();
    };
    PivotBar.prototype.render = function () {
        var _this = this;
        var _a = this.props, className = _a.className, filter = _a.filter, showPivots = _a.showPivots, showFilterBarViewOptionKey = _a.showFilterBarViewOptionKey, filterBarClassName = _a.filterBarClassName, pivotRenderingModeOptions = _a.pivotRenderingModeOptions, isCommandBarSticky = _a.isCommandBarSticky, isPivotBarContentScrollable = _a.isPivotBarContentScrollable, _b = _a.pivotBarSize, pivotBarSize = _b === void 0 ? "compact" /* Compact */ : _b;
        var _c = this.state, showFilterBar = _c.showFilterBar, headerTop = _c.headerTop;
        var children = React.Children.toArray(this.props.children) || [];
        var rawChildren = children.filter(function (c) { return typeof c !== 'object' || (c.type !== PivotBarItem && c.type !== FilterBar && c.type !== HubTileRegion); });
        var hubTileRegion = null;
        var hubTileRegionIndex = this._getHubTileRegionIndex(children);
        if (hubTileRegionIndex > -1) {
            hubTileRegion = children[hubTileRegionIndex];
        }
        var pivotItems = null;
        var showHubTiles = hubTileRegionIndex > -1;
        if (showPivots === undefined) {
            showPivots = (this.state.items && this.state.items.length > 1);
        }
        else {
            showHubTiles = showHubTiles && showPivots;
        }
        if (showPivots) {
            pivotItems = this._renderPivotItems(pivotRenderingModeOptions);
        }
        this._singleLineMode = !showPivots && !showHubTiles;
        var barClassName = this._singleLineMode ? "vss-PivotBar--bar-one-line" : "vss-PivotBar--bar-two-line";
        if (this.props.viewOptions && showFilterBarViewOptionKey) {
            showFilterBar = this.props.viewOptions.getViewOption(showFilterBarViewOptionKey);
        }
        var filterBarElement;
        var filterBarSettings = this._getFilterBarSettings(children);
        var showFilterInline = filterBarSettings && filterBarSettings.showInline;
        var shouldApplyStick = this.props.isPivotBarContentScrollable === false && !!isCommandBarSticky;
        if (filterBarSettings && (showFilterBar || filterBarSettings.showInline)) {
            filterBarElement = filterBarSettings.filterBarElement;
            filterBarElement = React.cloneElement(filterBarElement, {
                className: css(filterBarElement.props.className, filterBarClassName, shouldApplyStick ? "vss-PivotBar--filter-sticky" : ""),
                filter: filterBarElement.props.filter || filter,
                onDismissClicked: this._onFilterBarDismissClicked,
                onMounted: function (filterBar) {
                    if (!showFilterInline && _this._focusFilterBar) {
                        filterBar.focus();
                    }
                }
            });
        }
        // render a separator before the commands if we have items or we're in single line mode
        var prependSeparator = !!(pivotItems || this._singleLineMode);
        var stickyClassName = shouldApplyStick ? "sticky" : "";
        var rendered = (pivotRenderingModeOptions && pivotRenderingModeOptions.mode === 2 /* VerticalTabs */) ?
            (React.createElement("div", { className: css("vss-PivotBar", className) },
                React.createElement("div", { className: 'vss-PivotBar--header vertical' },
                    rawChildren,
                    showHubTiles && hubTileRegion),
                React.createElement("div", { className: "vss-PivotBar--pivotsContainer vertical" },
                    pivotItems,
                    React.createElement("div", { className: "vss-PivotBar--barAndContentContainer" },
                        React.createElement("div", { className: css("vss-PivotBar--bar", pivotBarSize) },
                            this._renderCommandBar(showFilterInline ? filterBarElement : null, !!filterBarSettings, false),
                            showFilterInline ? null : filterBarElement),
                        this._renderContent())))) : (React.createElement("div", { className: css("vss-PivotBar", className) },
            React.createElement("div", { className: css(barClassName, stickyClassName, { "bottom-border": !showFilterBar }), style: { top: this._singleLineMode ? 0 : headerTop } },
                React.createElement("div", { className: 'vss-PivotBar--header', ref: this._pivotHeader },
                    rawChildren,
                    showHubTiles && hubTileRegion),
                React.createElement("div", { className: css("vss-PivotBar--bar", pivotBarSize) },
                    pivotItems,
                    this._renderCommandBar(showFilterInline ? filterBarElement : null, !!filterBarSettings, prependSeparator))),
            showFilterInline ? null : filterBarElement,
            this._renderContent()));
        return rendered;
    };
    PivotBar.prototype._updateHeaderTopIfNeeded = function () {
        if (this.props.isPivotBarContentScrollable === false && !!this.props.isCommandBarSticky && this.state.headerTop === undefined && !this._singleLineMode) {
            this.setState({ headerTop: "-" + this._pivotHeader.current.clientHeight + "px" });
        }
    };
    PivotBar.prototype._renderPivotItems = function (pivotRenderingModeOptions) {
        if (pivotRenderingModeOptions && pivotRenderingModeOptions.mode === 1 /* DropDown */) {
            return this._renderDropdown(pivotRenderingModeOptions.props);
        }
        else if (pivotRenderingModeOptions && pivotRenderingModeOptions.mode === 2 /* VerticalTabs */) {
            return this._renderVerticalTabs();
        }
        else {
            return this._renderHorizontalTabs();
        }
    };
    PivotBar.prototype._renderDropdown = function (props) {
        var options = [];
        if (this.state.items.length > 0) {
            var lastGroupKey = this.state.items[0].groupKey;
            for (var _i = 0, _a = this.state.items; _i < _a.length; _i++) {
                var item = _a[_i];
                var name_1 = item.name, itemKey = item.itemKey, groupKey = item.groupKey;
                if (lastGroupKey !== groupKey) {
                    options.push({ key: groupKey + "-separator", text: "-", itemType: DropdownMenuItemType.Divider });
                    lastGroupKey = groupKey;
                }
                options.push({ key: itemKey, text: name_1 });
            }
        }
        return React.createElement(Dropdown, { ariaLabel: VSSUI_Resources.PivotItemAriaLabel, className: "vss-PivotBar--dropdown", selectedKey: this.state.selectedPivot, options: options, onChanged: this._onDropDownSelected, dropdownWidth: props.dropdownWidth });
    };
    PivotBar.prototype._pivotCompare = function (a, b) {
        return (a === undefined ? 100 : a) - (b === undefined ? 100 : b);
    };
    PivotBar.prototype._renderHorizontalTabs = function () {
        var _this = this;
        var pivots = this.state.items ? this.state.items.sort(function (a, b) { return _this._pivotCompare(a.order, b.order); }) : null;
        var _a = this.props, _b = _a.pivotBarSize, pivotBarSize = _b === void 0 ? "compact" /* Compact */ : _b, headerAriaLabel = _a.headerAriaLabel;
        return (React.createElement("div", { role: "navigation", "aria-label": headerAriaLabel },
            React.createElement(FocusZone, { className: css("vss-PivotBar--pivots", pivotBarSize), role: "tablist", direction: FocusZoneDirection.horizontal }, pivots && pivots.map(this._renderPivotItem))));
    };
    PivotBar.prototype._renderVerticalTabs = function () {
        var _this = this;
        // Get a sorted list of pivot groups to pass to the nav
        var pivotGroups = [];
        var pivotGroupMap = {};
        var pivotNavAriaLabel;
        if (this.props.pivotRenderingModeOptions && this.props.pivotRenderingModeOptions.props) {
            var _a = this.props.pivotRenderingModeOptions.props, groups_2 = _a.groups, navAriaLabel = _a.navAriaLabel;
            if (groups_2) {
                for (var _i = 0, groups_1 = groups_2; _i < groups_1.length; _i++) {
                    var group = groups_1[_i];
                    pivotGroupMap[group.key] = true;
                }
                pivotGroups = groups_2.sort(function (groupA, groupB) { return _this._pivotCompare(groupA.rank, groupB.rank); });
            }
            pivotNavAriaLabel = navAriaLabel;
        }
        // Map pivot bar items to their groups.
        // If an item has no groupKey, or its group is not defined,
        // it will be rendered at the end without a group.
        var ungroupedItems = [];
        var groupedItems = this.state.items.reduce(function (reduced, item) {
            if (item.groupKey && pivotGroupMap[item.groupKey]) {
                (reduced[item.groupKey] = reduced[item.groupKey] || []).push(item);
            }
            else {
                ungroupedItems.push(item);
            }
            return reduced;
        }, {});
        var groups = [React.createElement(Accordion, { key: "PivotBarVerticalAccordion", className: "pivot-bar--accordion", initiallyExpandAll: true, items: pivotGroups.map(function (group) {
                    var item;
                    if (groupedItems[group.key]) {
                        groupedItems[group.key].sort(function (a, b) { return _this._pivotCompare(a.order, b.order); });
                        item = {
                            key: group.key,
                            title: group.name,
                            onRenderContent: function () { return React.createElement("div", { key: item.key, role: "tablist" }, groupedItems[group.key].map(function (pivotItem, index) { return _this._renderVerticalPivotItem(pivotItem, index, groupedItems[group.key].length); })); }
                        };
                    }
                    return item;
                }).filter(function (item) { return item !== undefined; }) })];
        groups.push(React.createElement("div", { className: "pivot-bar--ungrouped", key: "PivotBarVerticalUngrouped", role: "tablist" }, ungroupedItems.map(function (pivotItem, index) { return _this._renderVerticalPivotItem(pivotItem, index, ungroupedItems.length); })));
        return React.createElement(FocusZone, { direction: FocusZoneDirection.vertical, className: "vss-PivotBar--pivots vertical flex flex-column", "aria-label": pivotNavAriaLabel }, groups);
    };
    PivotBar.prototype._renderVerticalPivotItem = function (item, index, groupLength) {
        var itemKey = item.itemKey;
        var isSelected = this.state.selectedPivot === itemKey;
        return (React.createElement(DynamicLink, { url: item.url, className: css('vss-PivotBar--itemLink-v', item.child && item.child.props.headerClassName), tabIndex: -1, role: 'presentation', "data-is-focusable": false, key: itemKey },
            React.createElement(CommandButton, { id: item.id, className: css("vss-PivotBar--button-v", isSelected ? "selected" : ""), onClick: this._onPivotClick.bind(this, itemKey), onKeyPress: this._onKeyPress.bind(this, itemKey), ariaLabel: item.ariaLabel, role: 'tab', "aria-selected": isSelected, "aria-posinset": index + 1, "aria-setsize": groupLength },
                React.createElement(VssIcon, { className: "pivotbar-pivot--icon-v", iconName: item.iconProps && item.iconProps.iconName }),
                React.createElement(TooltipHost, { content: item.name, overflowMode: TooltipOverflowMode.Parent }, item.name))));
    };
    PivotBar.prototype._renderPivotItemContent = function (item) {
        var name = item.name, iconProps = item.iconProps, badgeCount = item.badgeCount, maxItemWidth = item.maxItemWidth;
        var badgeDisplayValue = undefined;
        var badgeTooltip = undefined;
        if (typeof badgeCount === "string") {
            badgeDisplayValue = badgeCount;
        }
        else if (typeof badgeCount === "number") {
            badgeDisplayValue = getFriendlyDisplayValue(badgeCount);
            if (badgeDisplayValue !== badgeCount.toString()) {
                badgeTooltip = badgeCount.toString();
            }
        }
        var badgeElement = undefined;
        if (badgeDisplayValue) {
            badgeElement = (React.createElement("span", { className: "badge" }, badgeDisplayValue));
            if (badgeTooltip) {
                badgeElement = (React.createElement(TooltipHost, { content: badgeTooltip }, badgeElement));
            }
        }
        return (React.createElement("span", { className: "innerContainer" },
            iconProps !== undefined && (React.createElement(VssIcon, tslib_1.__assign({ className: "icon" }, iconProps))),
            name !== undefined &&
                React.createElement("span", { className: "text", style: { maxWidth: maxItemWidth || MAX_PIVOT_ITEM_WIDTH } },
                    React.createElement(TooltipHost, { content: name, overflowMode: TooltipOverflowMode.Parent }, name)),
            badgeElement));
    };
    PivotBar.prototype._renderCommandBar = function (filterBar, showFilterAction, prependSeparator) {
        var _this = this;
        return (React.createElement(PivotCommandBar, { componentRef: (function (commandBar) { _this._commandBar = commandBar; }), className: "vss-PivotBar--commandBar", commands: this.state.commands, viewActions: this.state.viewActions, filterBar: filterBar, showFilterAction: showFilterAction, showFilterBarViewOptionKey: this.props.showFilterBarViewOptionKey, hasAppliedFilter: this.state.hasAppliedFilter, prependSeparator: prependSeparator, viewOptions: this.props.viewOptions, onShowFilterBarClicked: this._onShowFilterBarClicked, role: "region", ariaLabel: this.props.headerAriaLabel }));
    };
    PivotBar.prototype._renderContent = function () {
        var content = [];
        var selectedPivot = (this.state.selectedPivot || "").toUpperCase();
        var isContentScrollable = (this.props.isPivotBarContentScrollable !== undefined) ? this.props.isPivotBarContentScrollable : true;
        for (var _i = 0, _a = this.state.items; _i < _a.length; _i++) {
            var item = _a[_i];
            var itemKeyUpperCase = item.itemKey.toUpperCase();
            var isSelected = selectedPivot === itemKeyUpperCase;
            var shouldHide = !isSelected
                && item.deselectionBehavior === 1 /* Hide */
                && this._pivotItemsRendered[itemKeyUpperCase] === true;
            if (isSelected || shouldHide) {
                content.push(React.createElement("div", { key: itemKeyUpperCase, className: css("vss-PivotBar--content", { "vss-PivotBar--content-unscrollable": !isContentScrollable, "hide-content": shouldHide }, this.props.pivotBarContentClassName), role: "tabpanel", tabIndex: -1, "data-is-scrollable": isContentScrollable }, item && (item.child || item.contributedItem.render(css("vss-PivotBarItem", "content-host")))));
                this._pivotItemsRendered[itemKeyUpperCase] = true;
            }
        }
        return content;
    };
    PivotBar.prototype._makePivotBarItemId = function (child) {
        return this._id + "-Tab" + child.index;
    };
    PivotBar.prototype._makeContributedPivotBarItemId = function (item, index) {
        return this._id + "-ContributedTab" + index;
    };
    PivotBar.prototype._getPivotBarItemByItemKey = function (itemKey, items) {
        if (items === void 0) { items = this.state.items; }
        var itemKeyUpperCase = (itemKey || "").toUpperCase();
        var index = findIndex(items, function (i) { return i.itemKey.toUpperCase() === itemKeyUpperCase; });
        return index >= 0 ? items[index] : undefined;
    };
    /**
     * Extracts the pivot items from the child elements.
     * @param props PivotBar properties.
     */
    PivotBar.prototype._getPivotBarItems = function (props) {
        var groupedItems = {};
        var pivotChildren = childrenOfType(props.children, PivotBarItem);
        var pushItem = function (item) {
            var groupKey = item.groupKey || "";
            var items = groupedItems[groupKey];
            if (!items) {
                items = [];
                groupedItems[groupKey] = items;
            }
            items.push(item);
        };
        var order = 100;
        for (var _i = 0, pivotChildren_1 = pivotChildren; _i < pivotChildren_1.length; _i++) {
            var pivot = pivotChildren_1[_i];
            var props_1 = pivot.item.props;
            if (!props_1.hidden) {
                pushItem({
                    child: pivot.item,
                    id: this._makePivotBarItemId(pivot),
                    itemKey: props_1.itemKey,
                    name: props_1.name,
                    groupKey: props_1.groupKey,
                    order: props_1.order != undefined ? props_1.order : order++,
                    ariaLabel: props_1.ariaLabel,
                    iconProps: props_1.iconProps,
                    badgeCount: props_1.badgeCount,
                    maxItemWidth: props_1.maxItemWidth,
                    commands: props_1.commands,
                    viewActions: props_1.viewActions,
                    commandProviders: props_1.commandProviders,
                    viewActionProviders: props_1.viewActionProviders,
                    deselectionBehavior: props_1.deselectionBehavior,
                    url: props_1.url,
                    onBeforePivotChange: props_1.onBeforePivotChange,
                    onRenderFilterBar: props_1.onRenderFilterBar
                });
            }
        }
        // Add contributed pivots from the observed collections
        this._contributedPivots.forEach(function (pivot) {
            return pivot.value.forEach(function (item) {
                if (!item.hidden) {
                    pushItem(item);
                }
            });
        });
        return Object.keys(groupedItems).reduce(function (acc, key) {
            groupedItems[key].forEach(function (item) { return acc.push(item); });
            return acc;
        }, []);
    };
    PivotBar.prototype._setupPivotObservables = function (props) {
        var _this = this;
        var index = 0;
        var wasSubscribed = this._subscribedToPivotObservables;
        if (wasSubscribed) {
            this._unsubscribeFromPivotObservables();
        }
        this._contributedPivots = [];
        if (props.pivotProviders) {
            props.pivotProviders.forEach(function (pivotProvider) {
                var contributedItems = new ContributedItemWithIdArray(pivotProvider, function (item) { return _this._makeIPivotBarItemFromContribution(item, index++); }, _this._getPivotBarItemId);
                _this._contributedPivots.push(contributedItems);
            });
            if (wasSubscribed) {
                this._subscribeToPivotObservables();
            }
        }
    };
    PivotBar.prototype._unsubscribeFromPivotObservables = function () {
        var _this = this;
        this._contributedPivots.forEach(function (pivots) { return pivots.unsubscribe(_this._onContributedPivotsChanged); });
        this._subscribedToPivotObservables = false;
    };
    PivotBar.prototype._subscribeToPivotObservables = function () {
        var _this = this;
        this._contributedPivots.forEach(function (pivots) { return pivots.subscribe(_this._onContributedPivotsChanged); });
        this._subscribedToPivotObservables = true;
    };
    PivotBar.prototype._getFilterBarSettings = function (children) {
        var _a = this.props, onRenderFilterBar = _a.onRenderFilterBar, showFilterBarInline = _a.showFilterBarInline;
        var filterBarElement;
        // Check if the pivot itself has a custom filter bar
        var selectedPivot = this._getPivotBarItemByItemKey(this.state.selectedPivot);
        if (selectedPivot && selectedPivot.onRenderFilterBar) {
            filterBarElement = selectedPivot.onRenderFilterBar();
        }
        // Invoke a custom onRenderFilterBar prop which can return a custom filter bar (e.g. not of type FilterBar)
        if (!filterBarElement && onRenderFilterBar) {
            filterBarElement = onRenderFilterBar();
        }
        // Look for a child FilterBar if none was supplied by the onRenderFilterBar prop
        if (!filterBarElement) {
            var filterBarIndex = findIndex(children, function (child) { return typeof child === 'object' && child.type === FilterBar; });
            if (filterBarIndex >= 0) {
                var childFilterBarElement = children[filterBarIndex];
                // Make sure that the child FilterBar has one or more child filters
                var filterBarChildren = React.Children.toArray(childFilterBarElement.props.children);
                if (filterBarChildren && filterBarChildren.length >= 1) {
                    filterBarElement = childFilterBarElement;
                    // Show the filterbar inline if it has only one filter which is a text filter
                    if (filterBarChildren.length === 1 && !showFilterBarInline) {
                        var firstFilterBarChild = filterBarChildren[0];
                        if (typeof firstFilterBarChild === 'object' && firstFilterBarChild.props.isTextItem === true) {
                            showFilterBarInline = true;
                        }
                    }
                }
            }
        }
        if (filterBarElement) {
            return {
                filterBarElement: filterBarElement,
                showInline: showFilterBarInline
            };
        }
    };
    PivotBar.prototype._getHubTileRegionIndex = function (children) {
        return findIndex(children, function (child) { return typeof child === 'object' && child.type === HubTileRegion; });
    };
    /**
     * Determines whether the key exists in the pivot items or not.
     */
    PivotBar.prototype._isKeyValid = function (itemKey, items) {
        if (items === void 0) { items = this.state.items; }
        return itemKey !== undefined && this._getPivotBarItemByItemKey(itemKey, items) !== undefined;
    };
    PivotBar.prototype._buildInitialPivotState = function (props, items, resetActions) {
        var _this = this;
        if (props === void 0) { props = this.props; }
        if (resetActions === void 0) { resetActions = false; }
        var getFirstPivot = function () {
            var currentItems = items || _this._getPivotBarItems(props);
            if (currentItems.length > 0) {
                return currentItems[0].itemKey;
            }
            return null;
        };
        var selectedPivot = null;
        if (!props.initialSelectedPivot && !props.selectedPivot) {
            // if there is no selected pivot, default to the first pivot
            selectedPivot = getFirstPivot();
        }
        if (!selectedPivot && props.initialSelectedPivot && this._isKeyValid(props.initialSelectedPivot)) {
            selectedPivot = props.initialSelectedPivot;
        }
        if (!selectedPivot && props.selectedPivot && this._isKeyValid(props.selectedPivot)) {
            selectedPivot = props.selectedPivot;
        }
        if (!this._isKeyValid(selectedPivot, items)) {
            // display no pivot while we wait to see if the pivot will be contributed
            selectedPivot = null;
            if (!this._contributedPivotWait) {
                this._contributedPivotWait = setTimeout(function () {
                    if (!_this._pivotSelected) {
                        // if the selected pivot has not yet been contributed, select the first pivot
                        selectedPivot = getFirstPivot();
                        if (selectedPivot) {
                            _this._updateSelectedItem(selectedPivot, null);
                        }
                    }
                    _this._contributedPivotWait = null;
                }, MAX_PIVOT_WAIT);
            }
        }
        this._pivotSelected = selectedPivot !== null;
        return this._buildNewPivotState(props, selectedPivot, items, resetActions);
    };
    PivotBar.prototype._buildNewPivotState = function (props, selectedPivotItemKey, items, resetActions) {
        if (resetActions === void 0) { resetActions = false; }
        var selectedPivot = this._getPivotBarItemByItemKey(selectedPivotItemKey, items);
        var selectedPivotAvailable = selectedPivotItemKey && selectedPivot;
        return {
            selectedPivot: selectedPivotAvailable ? selectedPivot.itemKey : "",
            commands: selectedPivotAvailable ? this._actionsManager.getCommandObservables(props.commands, selectedPivot, resetActions) : [],
            viewActions: selectedPivotAvailable ? this._actionsManager.getViewActionObservables(props.viewActions, selectedPivot, resetActions) : []
        };
    };
    /**
     * Handles the onClick event on the pivot button.
     */
    PivotBar.prototype._onPivotClick = function (itemKey, ev) {
        var updatePivot = true;
        // If ctrl-click is pressed, and there is a URL specified for this item, then
        // don't handle the click here, allowing the browser to perform a navigation
        // (i.e. open in a new tab/window)
        if (ev.ctrlKey) {
            var item = this._getPivotBarItemByItemKey(itemKey);
            if (item && item.url) {
                if (typeof item.url === "string" || item.url.value) {
                    updatePivot = false;
                }
            }
        }
        if (updatePivot) {
            ev.preventDefault();
            this._updateSelectedItem(itemKey, ev);
        }
    };
    /**
     * Handle the onKeyPress event on the pivot button.
     */
    PivotBar.prototype._onKeyPress = function (itemKey, ev) {
        ev.preventDefault();
        if (ev.which === 13 /* enter */) {
            this._updateSelectedItem(itemKey, ev);
        }
    };
    /**
     * Updates the state with the new selected pivot.
     */
    PivotBar.prototype._updateSelectedItem = function (itemKey, ev) {
        var item = this._getPivotBarItemByItemKey(itemKey);
        if (item && (this.props.onBeforePivotChange || item.onBeforePivotChange)) {
            var itemUrl = item && item.url && (typeof item.url === "string" ? item.url : item.url.value);
            if (item.onBeforePivotChange && !item.onBeforePivotChange(ev, itemKey, itemUrl)) {
                return;
            }
            if (this.props.onBeforePivotChange && !this.props.onBeforePivotChange(ev, itemKey, itemUrl)) {
                return;
            }
        }
        var props = this.props;
        this._pivotSelected = true;
        this.setState(this._buildNewPivotState(props, itemKey));
        // Update notifiers
        if (props.onPivotClicked) {
            props.onPivotClicked(ev, itemKey);
        }
    };
    return PivotBar;
}(BaseComponent));
export { PivotBar };
var PivotActionsManager = /** @class */ (function () {
    function PivotActionsManager() {
        this._commands = {};
        this._viewActions = {};
        this._getPivotActionItemId = function (item) {
            return item.key;
        };
    }
    PivotActionsManager.prototype.getCommandObservables = function (pivotBarCommands, selectedPivot, reset) {
        var _this = this;
        if (reset === void 0) { reset = false; }
        var selectedPivotId = selectedPivot.id;
        if (reset === true) {
            delete this._commands[selectedPivotId];
        }
        if (!this._commands[selectedPivotId]) {
            var commands_1 = [];
            this._commands[selectedPivotId] = commands_1;
            // Add hub commands first
            commands_1.push(ObservableLike.isObservable(pivotBarCommands) ? pivotBarCommands : new ObservableArray(pivotBarCommands));
            // Add commands from the selected pivot
            commands_1.push(ObservableLike.isObservable(selectedPivot.commands) ? selectedPivot.commands : new ObservableArray(selectedPivot.commands));
            // Add contributed commands
            if (selectedPivot.commandProviders) {
                selectedPivot.commandProviders.forEach(function (commandProvider) {
                    var contributedCommands = new ContributedItemWithIdArray(commandProvider, function (command) { return command; }, _this._getPivotActionItemId);
                    commands_1.push(contributedCommands);
                });
            }
        }
        return this._commands[selectedPivotId];
    };
    PivotActionsManager.prototype.getViewActionObservables = function (pivotBarViewActions, selectedPivot, reset) {
        if (reset === void 0) { reset = false; }
        var selectedPivotId = selectedPivot.id;
        if (reset === true) {
            delete this._viewActions[selectedPivotId];
        }
        if (!this._viewActions[selectedPivotId]) {
            var viewActions_1 = [];
            this._viewActions[selectedPivotId] = viewActions_1;
            // Add view actions from the selected pivot first
            viewActions_1.push(ObservableLike.isObservable(selectedPivot.viewActions) ? selectedPivot.viewActions : new ObservableArray(selectedPivot.viewActions));
            // Add hub view actions
            viewActions_1.push(ObservableLike.isObservable(pivotBarViewActions) ? pivotBarViewActions : new ObservableArray(pivotBarViewActions));
            // Add contributed view actions
            if (selectedPivot.viewActionProviders) {
                selectedPivot.viewActionProviders.forEach(function (viewActionProvider) {
                    var contributedViewActions = new ContributedItemArray(viewActionProvider, function (viewAction) { return viewAction; });
                    viewActions_1.push(contributedViewActions);
                });
            }
        }
        return this._viewActions[selectedPivotId];
    };
    PivotActionsManager.prototype.dispose = function () {
        this._commands = null;
        this._viewActions = null;
    };
    return PivotActionsManager;
}());
