import * as tslib_1 from "tslib";
import "../../FabricInit";
import * as React from "react";
import { CommandButton } from 'office-ui-fabric-react/lib/Button';
import { ContextualMenu, ContextualMenuItemType } from 'office-ui-fabric-react/lib/ContextualMenu';
import { FocusZone, FocusZoneDirection } from 'office-ui-fabric-react/lib/FocusZone';
import { IconType } from 'office-ui-fabric-react/lib/Icon';
import { Slider } from 'office-ui-fabric-react/lib/Slider';
import { Toggle } from '../../Toggle';
import { TooltipHost } from '../../Tooltip';
import { BaseComponent, css, getRTL, findIndex } from 'office-ui-fabric-react/lib/Utilities';
import { DynamicText } from '../../Dynamic';
import { VssIcon } from '../../VssIcon';
import * as VSSUI_Resources from '../../VSS.Resources.VSSUI';
var commandBarItemClassName = 'vss-PivotBar--commandBar-item';
/**
 * Internal enum determining where to render the visualization representing the action.
 */
var PivotBarActionArea;
(function (PivotBarActionArea) {
    /**
     * Area where all the actions are visible by default.
     */
    PivotBarActionArea[PivotBarActionArea["Bar"] = 0] = "Bar";
    /**
     * Area where the actions are hidden by default and exist inside an overflow menu.
     */
    PivotBarActionArea[PivotBarActionArea["Overflow"] = 1] = "Overflow";
})(PivotBarActionArea || (PivotBarActionArea = {}));
;
/**
 * Converts the specified actions to visual elements (IContextualMenuItem on the CommandBar).
 * @param actions List if actions to convert.
 * @param viewOptions View options to operate on.
 * @param forceUpdate Callback to force a rerender of the visualizations
 */
export function mapActionsToItems(actions, viewOptions, forceUpdate) {
    return mapActionsToVisualizations(actions, PivotBarActionArea.Bar, viewOptions, forceUpdate);
}
/**
 * Converts an IContextualMenuItem to an IPivotBarAction
 * @param menuItem
 */
export function mapItemToActions(menuItem) {
    var menuItemIconType = menuItem.iconProps.iconType;
    var pivotBarActionIconType = menuItemIconType === IconType.default || menuItemIconType === IconType.Default ? 0 /* fabric */ : 3 /* image */;
    var action = {
        key: menuItem.key,
        name: menuItem.name,
        href: menuItem.href,
        title: menuItem.title,
        iconProps: tslib_1.__assign({}, menuItem.iconProps, { iconType: pivotBarActionIconType }),
        onClick: menuItem.onClick,
        disabled: menuItem.disabled,
        important: menuItem.important,
        target: menuItem.target,
        separator: menuItem.itemType === ContextualMenuItemType.Divider,
    };
    if (menuItem.items) {
        action.children = menuItem.items.map(mapItemToActions);
    }
    return action;
}
/**
 * Internal mapping from action to action representation.
 * @param actions List if actions to convert.
 * @param area Container action of the action.
 * @param viewOptions View options to operate on.
 * @param forceUpdate Callback to force a rerender of the visualizations
 */
function mapActionsToVisualizations(actions, area, viewOptions, forceUpdate) {
    var items = [];
    for (var _i = 0, _a = actions || []; _i < _a.length; _i++) {
        var action = _a[_i];
        var actionVisualization = VisualizationRegistration.getInstance(area, action.actionType || 0 /* Command */);
        items = items.concat(actionVisualization.getVisualization(action, viewOptions, forceUpdate));
    }
    // Mark all CommandBar items with our class name
    items.forEach(function (item) {
        item.className = css(item.className, commandBarItemClassName);
    });
    return items;
}
function setViewOption(viewOptions, key, value, event, action) {
    if (viewOptions) {
        viewOptions.setViewOption(key, value);
    }
    else if (typeof action.onClick === 'function') {
        action.onClick(event, action);
    }
}
var VisualizationRegistration;
(function (VisualizationRegistration) {
    var actionVisualizations = {};
    function getKey(area, actionType) {
        return area + ":" + actionType;
    }
    function register(area, actionType, visualizationType) {
        var key = getKey(area, actionType);
        actionVisualizations[key] = visualizationType;
    }
    VisualizationRegistration.register = register;
    function getInstance(area, actionType) {
        var key = getKey(area, actionType);
        var visualizationType = actionVisualizations[key];
        if (!visualizationType) {
            throw new Error("No visualization found for area " + area + " and action type " + actionType);
        }
        // Return the instance of the visualization
        return new visualizationType();
    }
    VisualizationRegistration.getInstance = getInstance;
})(VisualizationRegistration || (VisualizationRegistration = {}));
/**
 * Command visualization which is same for both bar and overflow areas.
 */
var CommandVisualization = /** @class */ (function () {
    function CommandVisualization() {
    }
    CommandVisualization.prototype.getVisualization = function (action, viewOptions) {
        var item = {
            key: action.key,
            name: action.name || '',
            ariaLabel: action.ariaLabel,
            className: action.actionProps && action.actionProps.className,
            title: action.title,
            onClick: action.onClick,
            href: action.href,
            disabled: action.disabled,
            target: action.target,
            itemType: action.separator ? ContextualMenuItemType.Divider : ContextualMenuItemType.Normal,
        };
        if (action.iconProps) {
            var iconProps = tslib_1.__assign({}, action.iconProps);
            iconProps.className = css(iconProps.className, 'vss-PivotBar--commandBar-icon');
            item.iconProps = VssIcon.getIconProps(iconProps);
        }
        // Default view action may have child actions
        if (Array.isArray(action.children)) {
            item.subMenuProps = {
                className: 'vss-PivotBar--commandBar-overflow',
                items: mapActionsToVisualizations(action.children, PivotBarActionArea.Overflow, viewOptions)
            };
        }
        return [item];
    };
    return CommandVisualization;
}());
VisualizationRegistration.register(PivotBarActionArea.Bar, 0 /* Command */, CommandVisualization);
VisualizationRegistration.register(PivotBarActionArea.Overflow, 0 /* Command */, CommandVisualization);
/**
 * Base visualization for OnOff view action.
 */
var OnOffVisualization = /** @class */ (function () {
    function OnOffVisualization() {
    }
    OnOffVisualization.prototype.getVisualization = function (action, viewOptions) {
        return [];
    };
    OnOffVisualization.prototype.getActionProps = function (action) {
        if (!action.actionProps) {
            action.actionProps = { on: false };
        }
        return action.actionProps;
    };
    OnOffVisualization.prototype.isOn = function (key, viewOptions) {
        return viewOptions ? viewOptions.getViewOption(key) === true : false;
    };
    return OnOffVisualization;
}());
/**
 * OnOff visualization for bar area.
 */
var OnOffBarVisualization = /** @class */ (function (_super) {
    tslib_1.__extends(OnOffBarVisualization, _super);
    function OnOffBarVisualization() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    OnOffBarVisualization.prototype.getVisualization = function (action, viewOptions, forceUpdate) {
        var _this = this;
        var actionProps = this.getActionProps(action);
        var isOn = this.isOn(action.key, viewOptions);
        var itemNameValue = isOn ? VSSUI_Resources.PivotBarViewActionOnText : VSSUI_Resources.PivotBarViewActionOffText;
        // Show item name if not explicitly made invisible
        var itemName;
        if (actionProps.showText !== false) {
            itemName = itemNameValue;
        }
        // Try to get aria-label from the props, if not set, get it from item name if exists
        var itemAriaLabel = isOn ? actionProps.onAriaLabel : actionProps.offAriaLabel;
        var itemTitle = action.title;
        // Set aria-label to title/name if not excplicitly specified
        if (itemAriaLabel === undefined) {
            itemAriaLabel = itemTitle !== undefined ? itemTitle : itemNameValue;
        }
        // Set title to aria-label/name if not explicitly specified
        if (itemTitle === undefined) {
            itemTitle = itemAriaLabel !== undefined ? itemAriaLabel : itemNameValue;
        }
        // Try to get the icon from the action props first
        var iconProps = isOn ? actionProps.onIconProps : actionProps.offIconProps;
        // It might be specified to the view action directly
        if (!iconProps) {
            iconProps = action.iconProps;
        }
        var item = {
            key: action.key,
            onRender: function (item, dismissMenu, posInSet, setSize) {
                var button = (React.createElement(CommandButton, { className: css('vss-PivotBar--commandBar-toggle', actionProps.className), text: itemName, "aria-setsize": setSize, "aria-posinset": posInSet, checked: isOn, disabled: action.disabled, iconProps: VssIcon.getIconProps(iconProps), onClick: function (ev) {
                        setViewOption(viewOptions, item.key, !_this.isOn(item.key, viewOptions), ev, action);
                        forceUpdate && forceUpdate();
                    }, ariaLabel: itemAriaLabel }));
                if (itemTitle) {
                    return React.createElement(TooltipHost, { content: itemTitle, hostClassName: "vss-PivotBar--commandBar-toggle-host" }, button);
                }
                else {
                    return button;
                }
            }
        };
        return [item];
    };
    return OnOffBarVisualization;
}(OnOffVisualization));
VisualizationRegistration.register(PivotBarActionArea.Bar, 1 /* OnOff */, OnOffBarVisualization);
/**
 * OnOff visualization for overflow area.
 */
var OnOffOverflowVisualization = /** @class */ (function (_super) {
    tslib_1.__extends(OnOffOverflowVisualization, _super);
    function OnOffOverflowVisualization() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    OnOffOverflowVisualization.prototype.getVisualization = function (action, viewOptions) {
        var _this = this;
        var actionProps = this.getActionProps(action);
        var items = [];
        var onText = actionProps.onText || VSSUI_Resources.PivotBarViewActionOnText;
        var offText = actionProps.offText || VSSUI_Resources.PivotBarViewActionOffText;
        var onAriaLabel = actionProps.onAriaLabel || (action.ariaLabel ? action.ariaLabel + " " + onText : null);
        var offAriaLabel = actionProps.offAriaLabel || (action.ariaLabel ? action.ariaLabel + " " + offText : null);
        // Add an item with a custom rendering which uses a Toggle component
        var toggleItem = {
            key: action.key,
            onRender: function (item) {
                var toggle = (React.createElement(Toggle, { key: action.key + "-toggle", className: css('vss-PivotBar--commandBar-toggle', actionProps.className), checked: _this.isOn(action.key, viewOptions), disabled: action.disabled, onText: onText, offText: offText, onAriaLabel: onAriaLabel, offAriaLabel: offAriaLabel, onChange: function (event, value) {
                        setViewOption(viewOptions, item.key, value, null, action);
                    } }));
                if (action.title) {
                    return (React.createElement(TooltipHost, { content: action.title, key: action.key + "-toggle-tooltip" }, toggle));
                }
                return toggle;
            }
        };
        // Add section with header for the toggle inside a popup
        if (action.name) {
            items.push({
                key: action.key,
                sectionProps: {
                    items: [toggleItem],
                    title: action.name,
                    key: action.key + "-section"
                },
                itemType: ContextualMenuItemType.Section
            });
        }
        else {
            items.push(toggleItem);
        }
        return items;
    };
    return OnOffOverflowVisualization;
}(OnOffVisualization));
VisualizationRegistration.register(PivotBarActionArea.Overflow, 1 /* OnOff */, OnOffOverflowVisualization);
/**
 * Base visualization for ChoiceGroup view action.
 */
var ChoiceGroupVisualization = /** @class */ (function () {
    function ChoiceGroupVisualization() {
    }
    ChoiceGroupVisualization.prototype.getVisualization = function (action, viewOptions) {
        return [];
    };
    ChoiceGroupVisualization.prototype.getActionProps = function (action) {
        if (!action.actionProps) {
            throw new Error("Options must be specified for the choice group " + action.key);
        }
        return action.actionProps;
    };
    ChoiceGroupVisualization.prototype.getSelectedOption = function (key, actionProps, viewOptions) {
        var selectedOptionKey = (viewOptions ? viewOptions.getViewOption(key) : null) || actionProps.defaultSelectedKey;
        return (actionProps.options.filter(function (o) { return o.key === selectedOptionKey; })[0] ||
            actionProps.options[0]);
    };
    return ChoiceGroupVisualization;
}());
/**
 * A customized split button variation, which enables to select an option from a list of options when drop section is clicked.
 * Clicking button section will cycle through all available options.
 */
var ChoiceGroupButton = /** @class */ (function (_super) {
    tslib_1.__extends(ChoiceGroupButton, _super);
    function ChoiceGroupButton(props) {
        var _this = _super.call(this, props) || this;
        _this._onCycleClick = function (ev) {
            _this._changeOption(ev, _this._getNextOptionIndex(_this.props.options));
        };
        _this._onOptionClick = function (ev, item) {
            ev.stopPropagation();
            _this._changeOption(ev, findIndex(_this.props.options, function (o) { return o.key === item.key; }));
        };
        _this._onDropClick = function (ev) {
            _this.setState({
                showDropdown: true,
                dropTarget: ev.currentTarget
            });
        };
        _this._onDismiss = function (event) {
            _this.setState({ showDropdown: false });
        };
        _this.state = {
            showDropdown: false,
            selectedOptionIndex: _this._getSelectedOptionIndex(_this.props.options, _this.props.defaultSelectedKey)
        };
        return _this;
    }
    ChoiceGroupButton.prototype.componentWillReceiveProps = function (nextProps) {
        this.setState({
            showDropdown: false,
            selectedOptionIndex: this._getSelectedOptionIndex(nextProps.options, nextProps.defaultSelectedKey)
        });
    };
    ChoiceGroupButton.prototype.render = function () {
        var _this = this;
        var _a = this.props, dropButtonClassName = _a.dropButtonClassName, ariaLabel = _a.ariaLabel, dropButtonAriaLabel = _a.dropButtonAriaLabel, options = _a.options;
        var _b = this.state, selectedOptionIndex = _b.selectedOptionIndex, showDropdown = _b.showDropdown, dropTarget = _b.dropTarget;
        var selectedOption = options[selectedOptionIndex];
        return (React.createElement("div", { className: 'vss-ChoiceGroupButton' },
            React.createElement(CommandButton, { ariaLabel: ariaLabel || dropButtonAriaLabel || VSSUI_Resources.PivotBarMoreOptionsAriaLabel, role: "listbox", "aria-expanded": showDropdown, "aria-haspopup": true, className: css("vss-ChoiceGroupButton--drop", dropButtonClassName), text: selectedOption.text, menuIconProps: { iconName: "ChevronDown" }, onClick: this._onDropClick }),
            showDropdown ? (React.createElement(ContextualMenu, { shouldFocusOnMount: true, isBeakVisible: false, target: dropTarget, onDismiss: this._onDismiss, directionalHint: getRTL() ? 4 /* bottomLeftEdge */ : 6 /* bottomRightEdge */, items: options.map(function (o) {
                    return {
                        key: o.key,
                        name: o.text,
                        ariaLabel: o.ariaLabel,
                        isChecked: o === selectedOption,
                        canCheck: true,
                        onClick: _this._onOptionClick
                    };
                }) })) : (null)));
    };
    ChoiceGroupButton.prototype._getSelectedOptionIndex = function (options, defaultSelectedKey) {
        if (defaultSelectedKey === void 0) { defaultSelectedKey = null; }
        var selectedOptionIndex = defaultSelectedKey ? findIndex(options, function (o) { return o.key === defaultSelectedKey; }) : -1;
        return Math.max(selectedOptionIndex, 0);
    };
    ChoiceGroupButton.prototype._getNextOptionIndex = function (options) {
        var nextOptionIndex = this.state.selectedOptionIndex + 1;
        if (nextOptionIndex === options.length) {
            nextOptionIndex = 0;
        }
        return nextOptionIndex;
    };
    ChoiceGroupButton.prototype._changeOption = function (ev, index) {
        var _a = this.props, onChange = _a.onChange, options = _a.options;
        this.setState({ selectedOptionIndex: index });
        if (onChange) {
            onChange(ev, options[index]);
        }
    };
    return ChoiceGroupButton;
}(BaseComponent));
/**
 * ChoiceGroup visualization for bar area.
 */
var ChoiceGroupBarVisualization = /** @class */ (function (_super) {
    tslib_1.__extends(ChoiceGroupBarVisualization, _super);
    function ChoiceGroupBarVisualization() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ChoiceGroupBarVisualization.prototype.getVisualization = function (action, viewOptions) {
        var actionProps = this.getActionProps(action);
        var selectedOption = this.getSelectedOption(action.key, actionProps, viewOptions);
        // Add item for the selected option which is always visible
        return [{
                key: action.key,
                className: css('vss-PivotBar--commandBar-barChoiceGroup', actionProps.className),
                onRender: function (item, dismissMenu, posInSet, setSize) {
                    return (React.createElement(ChoiceGroupButton, { ariaLabel: item.ariaLabel, "aria-setsize": setSize, "aria-posinset": posInSet, dropButtonAriaLabel: actionProps.dropButtonAriaLabel, defaultSelectedKey: selectedOption.key, options: actionProps.options, dropButtonClassName: 'vss-PivotBar--commandBar-barChoiceDrop', onChange: function (ev, option) {
                            setViewOption(viewOptions, action.key, option.key, ev, action);
                        } }));
                }
            }];
    };
    return ChoiceGroupBarVisualization;
}(ChoiceGroupVisualization));
VisualizationRegistration.register(PivotBarActionArea.Bar, 2 /* ChoiceGroup */, ChoiceGroupBarVisualization);
/**
 * ChoiceGroup visualization for overflow area.
 */
var ChoiceGroupOverflowVisualization = /** @class */ (function (_super) {
    tslib_1.__extends(ChoiceGroupOverflowVisualization, _super);
    function ChoiceGroupOverflowVisualization() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ChoiceGroupOverflowVisualization.prototype.getVisualization = function (action, viewOptions) {
        var actionProps = this.getActionProps(action);
        var selectedOption = this.getSelectedOption(action.key, actionProps, viewOptions);
        var items = [];
        // Add header for the choice group inside a popup
        if (action.name) {
            items.push({
                key: action.key + "-header",
                name: action.name,
                itemType: ContextualMenuItemType.Header
            });
        }
        // Add each option individually
        for (var _i = 0, _a = actionProps.options; _i < _a.length; _i++) {
            var option = _a[_i];
            items.push({
                key: option.key,
                name: option.text,
                ariaLabel: option.ariaLabel,
                isChecked: option === selectedOption,
                canCheck: true,
                title: option.title,
                onClick: function (ev, item) {
                    setViewOption(viewOptions, action.key, item.key, ev, action);
                    if (viewOptions && ev) {
                        ev.preventDefault();
                    }
                }
            });
        }
        return items;
    };
    return ChoiceGroupOverflowVisualization;
}(ChoiceGroupVisualization));
VisualizationRegistration.register(PivotBarActionArea.Overflow, 2 /* ChoiceGroup */, ChoiceGroupOverflowVisualization);
/**
 * Text visualization for both bar and overflow areas.
 */
var TextVisualization = /** @class */ (function () {
    function TextVisualization() {
    }
    TextVisualization.prototype.getActionProps = function (action) {
        var actionProps = action.actionProps;
        if (!actionProps || !actionProps.text) {
            throw new Error("Text must be specified through actionProps for " + action.key);
        }
        return actionProps;
    };
    TextVisualization.prototype.getVisualization = function (action, viewOptions) {
        var actionProps = this.getActionProps(action);
        return [
            {
                key: action.key,
                className: 'vss-PivotBar--commandBar-text',
                onRender: function (item) {
                    var dynamicText = React.createElement(DynamicText, { className: actionProps.className, key: action.key + "-text", text: actionProps.text });
                    if (action.title) {
                        return (React.createElement(TooltipHost, { content: action.title, key: action.key + "-tooltip" }, dynamicText));
                    }
                    return dynamicText;
                }
            }
        ];
    };
    return TextVisualization;
}());
VisualizationRegistration.register(PivotBarActionArea.Bar, 3 /* Text */, TextVisualization);
VisualizationRegistration.register(PivotBarActionArea.Overflow, 3 /* Text */, TextVisualization);
var PivotBarSlider = /** @class */ (function (_super) {
    tslib_1.__extends(PivotBarSlider, _super);
    function PivotBarSlider() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._onOuterKeyDown = function (event) {
            if (event.which === 13 /* enter */) {
                if (_this._innerFocus) {
                    _this._innerFocus.focus();
                    event.preventDefault();
                    event.stopPropagation();
                }
            }
        };
        _this._onInnerKeyDown = function (event) {
            if (event.which === 27 /* escape */) {
                if (_this._outerFocus) {
                    _this._outerFocus.focus();
                    event.preventDefault();
                    event.stopPropagation();
                }
            }
        };
        return _this;
    }
    PivotBarSlider.prototype.render = function () {
        var _this = this;
        return (React.createElement(TooltipHost, { content: this.props.title },
            React.createElement("div", { ref: function (el) { return _this._outerFocus = el; }, onKeyDown: this._onOuterKeyDown, "data-is-focusable": true, className: 'vss-PivotBarSlider' },
                React.createElement(FocusZone, { componentRef: function (fz) { return _this._innerFocus = fz; }, onKeyDown: this._onInnerKeyDown, direction: FocusZoneDirection.horizontal },
                    React.createElement(Slider, tslib_1.__assign({}, this.props))))));
    };
    return PivotBarSlider;
}(BaseComponent));
/**
 * Base visualization for Slider view action.
 */
var SliderVisualization = /** @class */ (function () {
    function SliderVisualization() {
    }
    SliderVisualization.prototype.getActionProps = function (action) {
        if (!action.actionProps) {
            action.actionProps = {};
        }
        return action.actionProps;
    };
    SliderVisualization.prototype.getValue = function (key, viewOptions) {
        return viewOptions ? viewOptions.getViewOption(key) || 0 : 0;
    };
    Object.defineProperty(SliderVisualization.prototype, "useLabel", {
        get: function () {
            return true;
        },
        enumerable: true,
        configurable: true
    });
    SliderVisualization.prototype.getVisualization = function (action, viewOptions) {
        var _this = this;
        var actionProps = this.getActionProps(action);
        var value = this.getValue(action.key, viewOptions);
        // Add item for the selected option which is always visible
        return [{
                key: action.key,
                className: 'vss-PivotBar--commandBar-barSlider',
                onRender: function (item, dismissMenu, posInSet, setSize) {
                    return (React.createElement(PivotBarSlider, { key: action.key + "-slider", className: css('vss-PivotBar--commandBar-slider', actionProps.className), label: _this.useLabel ? action.name : undefined, value: value, min: actionProps.minValue, max: actionProps.maxValue, step: actionProps.step, ariaLabel: action.ariaLabel, "aria-setsize": setSize, "aria-posinset": posInSet, title: action.title, showValue: false, disabled: action.disabled, onChange: function (value) {
                            setViewOption(viewOptions, action.key, value, null, action);
                        } }));
                }
            }];
    };
    return SliderVisualization;
}());
/**
 * Slider visualization for bar area.
 */
var SliderBarVisualization = /** @class */ (function (_super) {
    tslib_1.__extends(SliderBarVisualization, _super);
    function SliderBarVisualization() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return SliderBarVisualization;
}(SliderVisualization));
VisualizationRegistration.register(PivotBarActionArea.Bar, 4 /* Slider */, SliderBarVisualization);
/**
 * Slider visualization for overflow area.
 */
var SliderOverflowVisualization = /** @class */ (function (_super) {
    tslib_1.__extends(SliderOverflowVisualization, _super);
    function SliderOverflowVisualization() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(SliderOverflowVisualization.prototype, "useLabel", {
        get: function () {
            return false;
        },
        enumerable: true,
        configurable: true
    });
    SliderOverflowVisualization.prototype.getVisualization = function (action, viewOptions) {
        var items = _super.prototype.getVisualization.call(this, action, viewOptions);
        if (action.name) {
            // For overflow area, add label as a header
            items.unshift({
                key: action.key + "-header",
                name: action.name,
                itemType: ContextualMenuItemType.Header
            });
        }
        return items;
    };
    return SliderOverflowVisualization;
}(SliderVisualization));
VisualizationRegistration.register(PivotBarActionArea.Overflow, 4 /* Slider */, SliderOverflowVisualization);
