import * as tslib_1 from "tslib";
import "../../FabricInit";
import * as React from 'react';
import { CommandBar } from 'office-ui-fabric-react/lib/CommandBar';
import { BaseComponent, css } from 'office-ui-fabric-react/lib/Utilities';
import { mapActionsToItems } from '../../Components/PivotBar/PivotBarAction';
import { arrayEquals } from '../../Utilities/Internal';
import * as VSSUI_Resources from '../../VSS.Resources.VSSUI';
var PivotCommandBar = /** @class */ (function (_super) {
    tslib_1.__extends(PivotCommandBar, _super);
    function PivotCommandBar(props) {
        var _this = _super.call(this, props) || this;
        _this._onCommandsChanged = function () {
            _this.setState({ commands: _this._getCommands(_this.props) });
        };
        _this._onViewActionsChanged = function () {
            _this.setState({ viewActions: _this._getViewActions(_this.props) });
        };
        /**
         * Conveniently autobound forceUpdate method.
         */
        _this._forceUpdate = function () {
            _this.forceUpdate();
        };
        _this.state = {
            commands: _this._getCommands(_this.props),
            viewActions: _this._getViewActions(_this.props)
        };
        return _this;
    }
    PivotCommandBar.prototype.componentWillReceiveProps = function (nextProps) {
        var changed = false;
        if (!arrayEquals(this.props.commands, nextProps.commands)) {
            this._unsubscribeItems(this.props.commands, this._onCommandsChanged);
            this._subscribeItems(nextProps.commands, this._onCommandsChanged);
            changed = true;
        }
        if (!arrayEquals(this.props.viewActions, nextProps.viewActions)) {
            this._unsubscribeItems(this.props.viewActions, this._onViewActionsChanged);
            this._subscribeItems(nextProps.viewActions, this._onViewActionsChanged);
            changed = true;
        }
        if (this.props.showFilterAction !== nextProps.showFilterAction ||
            this.props.filterBar !== nextProps.filterBar) {
            changed = true;
        }
        if (changed) {
            this.setState({
                commands: this._getCommands(nextProps),
                viewActions: this._getViewActions(nextProps)
            });
        }
    };
    PivotCommandBar.prototype.componentDidMount = function () {
        this._subscribeItems(this.props.commands, this._onCommandsChanged);
        this._subscribeItems(this.props.viewActions, this._onViewActionsChanged);
    };
    PivotCommandBar.prototype.componentWillUnmount = function () {
        this._unsubscribeItems(this.props.commands, this._onCommandsChanged);
        this._unsubscribeItems(this.props.viewActions, this._onViewActionsChanged);
    };
    PivotCommandBar.prototype._subscribeItems = function (itemSources, observer) {
        (itemSources || []).forEach(function (source) { return source.subscribe(observer); });
    };
    PivotCommandBar.prototype._unsubscribeItems = function (itemSources, observer) {
        (itemSources || []).forEach(function (source) { return source.unsubscribe(observer); });
    };
    PivotCommandBar.prototype.render = function () {
        var _this = this;
        var _a = this.props, className = _a.className, viewOptions = _a.viewOptions, filterBar = _a.filterBar, hasAppliedFilter = _a.hasAppliedFilter, showFilterAction = _a.showFilterAction, onShowFilterBarClicked = _a.onShowFilterBarClicked, showFilterBarViewOptionKey = _a.showFilterBarViewOptionKey, prependSeparator = _a.prependSeparator, role = _a.role, ariaLabel = _a.ariaLabel;
        var _b = this.state, commands = _b.commands, viewActions = _b.viewActions;
        // Set up commands first by keeping important commands visible and the rest under the overflow menu (...)
        var commandItems = [];
        var commandOverflowItems = [];
        if (commands && commands.length) {
            var importantCommands = commands.filter(function (c) { return c.important; });
            var overflowCommands = commands.filter(function (c) { return !c.important; });
            // Important commands are always visible
            commandItems = mapActionsToItems(importantCommands, viewOptions, this._forceUpdate);
            // Non-important commands go to overflow section
            commandOverflowItems = mapActionsToItems(overflowCommands, viewOptions, this._forceUpdate);
        }
        // Set up view actions the same way we do for commands
        // Instead of (...), non important view actions go under 'View Options'
        var farActions = [];
        if (viewActions && viewActions.length) {
            // Add important view actions first which target BEFORE the view options entry.
            farActions.push.apply(farActions, viewActions.filter(function (va) { return va.important && va.viewActionRenderArea === 1 /* beforeViewOptions */; }));
            // Add non-important view actions into a "view options" entry.
            var overflowViewActions = viewActions.filter(function (va) { return !va.important; });
            if (overflowViewActions.length) {
                farActions.push({
                    key: "viewOptions",
                    title: VSSUI_Resources.PivotBarViewOptionsText,
                    ariaLabel: VSSUI_Resources.PivotBarViewOptionsText,
                    actionProps: { className: "vss-PivotBar--viewOptionsMenu" },
                    important: true,
                    actionType: 0 /* Command */,
                    iconProps: { iconName: "Equalizer" },
                    children: overflowViewActions,
                });
            }
            // Add important view actions which target AFTER the view options entry    
            farActions.push.apply(farActions, viewActions.filter(function (va) { return va.important && va.viewActionRenderArea !== 1 /* beforeViewOptions */ && va.viewActionRenderArea !== 2 /* farRight */; }));
        }
        // Add filter view action automatically when a filterbar exists in the children
        if (showFilterAction && !filterBar) {
            farActions.push({
                key: showFilterBarViewOptionKey,
                important: true,
                actionType: 1 /* OnOff */,
                actionProps: {
                    showText: false,
                    onAriaLabel: VSSUI_Resources.PivotBarFilterActionTextHide,
                    offAriaLabel: VSSUI_Resources.PivotBarFilterActionTextShow,
                    onIconProps: { iconName: hasAppliedFilter ? "FilterSolid" : "Filter" },
                    offIconProps: { iconName: hasAppliedFilter ? "FilterSolid" : "Filter" },
                },
                onClick: onShowFilterBarClicked
            });
        }
        // Add far-right important view actions last
        if (viewActions && viewActions.length) {
            farActions.push.apply(farActions, viewActions.filter(function (va) { return va.important && va.viewActionRenderArea === 2 /* farRight */; }));
        }
        var viewActionItems = mapActionsToItems(farActions, viewOptions, this._forceUpdate);
        // If available, show inline filterbar to the left of all view actions
        if (filterBar) {
            viewActionItems.unshift({
                className: "vss-PivotBar--inlineFilterBar",
                key: "filterBar",
                onRender: function () { return filterBar; }
            });
        }
        if (commandItems.length || commandOverflowItems.length || viewActionItems.length) {
            return (React.createElement("div", { className: "vss-PivotBar--right", role: role, "aria-label": ariaLabel },
                !!(prependSeparator && (commandItems.length || commandOverflowItems.length)) && React.createElement("div", { className: "vss-PivotBar--separator" }),
                React.createElement(CommandBar, { componentRef: (function (commandBar) { _this._commandBar = commandBar; }), className: css(className), items: commandItems, overflowItems: commandOverflowItems, farItems: viewActionItems, elipisisAriaLabel: VSSUI_Resources.PivotBarMoreCommandsAriaLabel })));
        }
        return null;
    };
    PivotCommandBar.prototype.focus = function () {
        if (this._commandBar) {
            this._commandBar.focus();
        }
    };
    PivotCommandBar.prototype._getCommands = function (props) {
        var commands = [];
        var pivotCommands = props.commands || [];
        for (var _i = 0, pivotCommands_1 = pivotCommands; _i < pivotCommands_1.length; _i++) {
            var pc = pivotCommands_1[_i];
            commands.push.apply(commands, pc.value);
        }
        return commands;
    };
    PivotCommandBar.prototype._getViewActions = function (props) {
        var viewActions = [];
        var pivotViewActions = props.viewActions || [];
        for (var _i = 0, pivotViewActions_1 = pivotViewActions; _i < pivotViewActions_1.length; _i++) {
            var pv = pivotViewActions_1[_i];
            viewActions.push.apply(viewActions, pv.value);
        }
        return viewActions;
    };
    return PivotCommandBar;
}(BaseComponent));
export { PivotCommandBar };
