import * as React from "react";
import { IReadonlyObservableValue } from '../../Core/Observable';
export interface IRadioButtonGroup {
    /**
     * Sets focus to the RadioButtonGroup.
     */
    focus(): void;
}

export interface IRadioButtonGroupProps {
    /**
     * Optional class name to add to the radio button group element.
     */
    className?: string;

    /**
     * The id of the button that should be focused by default.
     */
    defaultButtonId?: string;

    /**
     * Set to true if you don't want the radio button group's focus managed by a FocusZone.
     */
    excludeFocusZone?: boolean;

    /**
     * Optional id of the checkbox group element.
     */
    id?: string;

    /**
     * Callback called when a new button is selected.
     */
    onSelect?: (buttonId: string) => void;

    /**
     * The id of the currently selected radio button.
     */
    selectedButtonId?: IReadonlyObservableValue<string> | string;

    /**
     * Text to display above the radio group.
     */
    text?: string;
}

export interface IRadioButtonProps {
    /**
     * Id of another element which labels this one for screen reader users.
     */
    ariaLabelledBy?: string;

    /**
     * Optional class name to add to the radio button element.
     */
    className?: string;

    /**
     * If true, the button cannot be interacted with.
     */
    disabled?: boolean;

    /**
     * Set to true if you don't want the radio button's focus managed by a FocusZone.
     */
    excludeFocusZone?: boolean;

    /**
     * Set to true to omit setting the tab index.
     */
    excludeTabStop?: boolean;

    /**
     * id of the radio button.
     */
    id: string;

    /**
     * Optional aria role.
     * @default radio
     */
    role?: string;

    /**
     * Text to display next to the radio button.
     */
    text?: string;
}