import * as tslib_1 from "tslib";
import "../../Core/core.scss";
import "../../Common.scss";
import "./RadioButton.scss";
import * as React from "react";
import { FocusZone, FocusZoneContext } from '../../FocusZone';
import { ObservableLike } from '../../Core/Observable';
import { css, noop } from '../../Util';
export var RadioButtonGroupContext = React.createContext({
    onFocus: noop,
    onSelect: noop
});
var groupId = 1;
var RadioButtonGroup = /** @class */ (function (_super) {
    tslib_1.__extends(RadioButtonGroup, _super);
    function RadioButtonGroup(props) {
        var _this = _super.call(this, props) || this;
        _this.onChange = function (buttonId) {
            if (_this.props.onSelect) {
                _this.props.onSelect(buttonId);
            }
        };
        _this.onFocus = function (buttonId) {
            _this.setState({ focusedButtonId: buttonId });
        };
        _this.onSelectionChange = function (selectedButtonId) {
            console.log("selection changing");
            _this.forceUpdate();
        };
        _this._id = "radio-group-" + groupId++;
        _this.state = {};
        return _this;
    }
    RadioButtonGroup.prototype.focus = function () {
        var id = this.state.focusedButtonId || this.props.defaultButtonId;
        if (!id) {
            var children = React.Children.toArray(this.props.children);
            if (children.length) {
                var firstChild = children[0];
                id = firstChild.props.id;
            }
        }
        if (id) {
            document.getElementById(id).focus();
        }
    };
    RadioButtonGroup.prototype.render = function () {
        var selectedButtonId = this.props.selectedButtonId && ObservableLike.getValue(this.props.selectedButtonId);
        return (React.createElement(RadioButtonGroupContext.Provider, { value: {
                defaultButtonId: this.props.defaultButtonId,
                focusedButtonId: this.state.focusedButtonId,
                onFocus: this.onFocus,
                onSelect: this.onChange,
                selectedButtonId: selectedButtonId
            } },
            React.createElement("div", { className: "flex-column" },
                this.props.text && React.createElement("div", { className: "bolt-radio-button-group-text" }, this.props.text),
                this.props.excludeFocusZone ? (this.props.children) : (React.createElement(FocusZone, { direction: 2 /* Vertical */, isCircularNavigation: true },
                    React.createElement("div", { className: css(this.props.className, "bolt-radio-button-group", "flex-column"), id: this.props.id || this._id, role: "radiogroup" }, this.props.children))))));
    };
    RadioButtonGroup.prototype.componentDidMount = function () {
        ObservableLike.subscribe(this.props.selectedButtonId, this.onSelectionChange);
    };
    RadioButtonGroup.prototype.componentWillUnmount = function () {
        ObservableLike.unsubscribe(this.props.selectedButtonId, this.onSelectionChange);
    };
    return RadioButtonGroup;
}(React.Component));
export { RadioButtonGroup };
var RadioButton = /** @class */ (function (_super) {
    tslib_1.__extends(RadioButton, _super);
    function RadioButton() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    RadioButton.prototype.render = function () {
        var _this = this;
        return (React.createElement(RadioButtonGroupContext.Consumer, null, function (groupContext) { return (React.createElement(FocusZoneContext.Consumer, null, function (zoneContext) { return (React.createElement(RadioButtonInternal, tslib_1.__assign({}, _this.props, { focusedButton: groupContext.focusedButtonId === _this.props.id ||
                (!groupContext.focusedButtonId && groupContext.selectedButtonId === _this.props.id) ||
                (!groupContext.selectedButtonId && groupContext.defaultButtonId === _this.props.id), focuszoneId: zoneContext.focuszoneId, onFocus: groupContext.onFocus, onSelect: groupContext.onSelect, selectedButton: groupContext.selectedButtonId === _this.props.id }))); })); }));
    };
    return RadioButton;
}(React.Component));
export { RadioButton };
var RadioButtonInternal = /** @class */ (function (_super) {
    tslib_1.__extends(RadioButtonInternal, _super);
    function RadioButtonInternal() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onClick = function (event) {
            if (!event.defaultPrevented) {
                if (!_this.props.disabled) {
                    _this.props.onSelect(_this.props.id);
                    event.preventDefault();
                }
            }
        };
        _this.onFocus = function (event) {
            _this.props.onFocus(_this.props.id);
        };
        _this.onKeyDown = function (event) {
            if (!event.defaultPrevented) {
                if (event.which === 32 /* Space */) {
                    event.preventDefault();
                }
            }
        };
        _this.onKeyUp = function (event) {
            if (!event.defaultPrevented) {
                if (!_this.props.disabled) {
                    if (event.which === 13 /* Enter */ || event.which === 32 /* Space */) {
                        _this.props.onSelect(_this.props.id);
                        event.preventDefault();
                    }
                }
            }
        };
        return _this;
    }
    RadioButtonInternal.prototype.render = function () {
        var tabIndex;
        // Determine the type of tabbing this radio button has.
        if (!this.props.excludeTabStop) {
            if (this.props.focusedButton) {
                tabIndex = 0;
            }
            else if (!this.props.disabled) {
                tabIndex = -1;
            }
        }
        var labelId = this.props.text && "rb-label-" + this.props.id;
        var radio = (React.createElement("div", { "aria-checked": this.props.selectedButton, "aria-disabled": this.props.disabled, "aria-labelledby": this.props.ariaLabelledBy || this.props.id || labelId, className: css(this.props.className, "bolt-radio-button", this.props.disabled ? "disabled" : "enabled", this.props.selectedButton && "checked", this.props.text && "labelled", "bolt-focus-treatment"), "data-focuszone": !this.props.disabled && !this.props.excludeFocusZone ? this.props.focuszoneId : undefined, id: this.props.id, key: this.props.id, onClick: this.onClick, onFocus: this.onFocus, onKeyDown: this.onKeyDown, onKeyUp: this.onKeyUp, role: this.props.role || "radio", tabIndex: tabIndex },
            React.createElement("div", { className: "bolt-radio-button-icon" },
                React.createElement("div", { className: "bolt-radio-button-bullet" })),
            this.props.text && (React.createElement("div", { className: "bolt-radio-button-label", id: labelId }, this.props.text)),
            this.props.children));
        return radio;
    };
    return RadioButtonInternal;
}(React.Component));
