import * as tslib_1 from "tslib";
import "../../FabricInit";
import * as React from 'react';
import { css, BaseComponent, getRect } from 'office-ui-fabric-react/lib/Utilities';
var ResizeDelayMs = 100;
var ResponsiveViewport = /** @class */ (function (_super) {
    tslib_1.__extends(ResponsiveViewport, _super);
    function ResponsiveViewport(props) {
        var _this = _super.call(this, props) || this;
        _this._resolveElement = function (element) {
            _this._element = element;
        };
        var initialWidth = props.initialWidth;
        _this.state = {
            activeBreakpoints: initialWidth == null ? [] : _this._getActiveBreakpoints(initialWidth)
        };
        _this._onAsyncResize = _this._async.throttle(_this._onAsyncResize, ResizeDelayMs, {
            leading: false
        });
        return _this;
    }
    ResponsiveViewport.prototype.componentDidMount = function () {
        this._events.on(window, 'resize', this._onAsyncResize);
        // React to initial width
        this._onAsyncResize();
    };
    ResponsiveViewport.prototype.componentWillUnmount = function () {
        this._events.dispose();
    };
    ResponsiveViewport.prototype.shouldComponentUpdate = function (newProps, newState) {
        if (this.props !== newProps) {
            return true;
        }
        var newBreakPoints = newState.activeBreakpoints;
        var oldBreakPoints = this.state.activeBreakpoints;
        return newBreakPoints.length !== oldBreakPoints.length || newBreakPoints.some(function (b, idx) { return b !== oldBreakPoints[idx]; });
    };
    ResponsiveViewport.prototype.render = function () {
        var _a = this.props, _b = _a.tag, tag = _b === void 0 ? 'div' : _b, className = _a.className, children = _a.children, onRenderContent = _a.onRenderContent;
        var activeBreakpoints = this.state.activeBreakpoints;
        var content = onRenderContent && onRenderContent(activeBreakpoints) || children;
        return React.createElement(tag, {
            className: css.apply(void 0, [className].concat((activeBreakpoints && activeBreakpoints.map(function (x) { return x.className; })))),
            ref: this._resolveElement
        }, content);
    };
    ResponsiveViewport.prototype.measure = function () {
        this._resize();
    };
    ResponsiveViewport.prototype._onAsyncResize = function () {
        this._resize();
    };
    ResponsiveViewport.prototype._resize = function () {
        var rect = getRect(this._element);
        var width = rect.width;
        this.setState({
            activeBreakpoints: this._getActiveBreakpoints(width)
        });
    };
    ResponsiveViewport.prototype._getActiveBreakpoints = function (width) {
        var breakPoints = this.props.breakPoints;
        return breakPoints.filter(function (bp) {
            if (bp.minWidth >= 0 && width < bp.minWidth) {
                return false;
            }
            if (bp.maxWidth >= 0 && width >= bp.maxWidth) {
                return false;
            }
            return true;
        });
    };
    return ResponsiveViewport;
}(BaseComponent));
export { ResponsiveViewport };
