import "../../FabricInit";
import * as React from "react";
import { IBaseProps } from "office-ui-fabric-react/lib/Utilities";
import { ISearchBox as IFabricSearchBox } from "office-ui-fabric-react/lib/SearchBox";

export interface ISearchBox extends IFabricSearchBox
{

}

export interface ISearchBoxProps {
    /**
     * Optional callback to access the ISearchBox interface. Use this instead of ref for accessing
     * the public methods and properties of the component.
     */
    componentRef?: (component: ISearchBox | null) => void;

    /**
     * Placeholder for the search box.
     */
    placeholder?: string;

    /**
    * If the control is disabled function for when the typed input for the SearchBox has changed.
    */
   disabled?: boolean;

    /**
    * Callback function for when the typed input for the SearchBox has changed.
    */
    onChange?: (newValue: any) => void;

    /**
     * Callback executed when the user presses enter in the search box.
     */
    onSearch?: (newValue: any) => void;

    /**
     * Callback executed when the user clears the search box by either clicking 'X' or hitting escape.
     */
    onClear?: (ev?: any) => void;

    /**
     * Callback executed when the user presses escape in the search box.
     */
    onEscape?: (ev?: any) => void;
 
    /**
     * The value of the text in the SearchBox.
     */
    value?: string;

    /**
     * The aria label of the SearchBox for the benefit of screen readers.
     * @defaultvalue placeholder
     */
    ariaLabel?: string;

    /**
     * The props for the clear button.
     */
    clearButtonProps?: IButtonProps;

    /**
     * Whether or not the SearchBox is underlined.
     * @default false
     */
    underlined?: boolean;

    /**
     * Whether or not to animate the SearchBox icon on focus.
     * @default false
     */
    disableAnimation?: boolean;
}