import "../../FabricInit";
import { IBaseProps } from "office-ui-fabric-react/lib/Utilities";

export interface IStackPanel {
    /**
     * Pushes a new item onto the internal Stack
     * @param item 
     */
    push(item: IStackPanelItem): void;

    /**
     * Removes and returns the current top item
     */
    pop(): IStackPanelItem | undefined;

    /**
     * Returns the current top item
     */
    peek(): IStackPanelItem | undefined;
}

export interface IStackPanelProps extends IBaseProps<IStackPanel> {
    /**
     * Initial items to treat as the stack; will render the LAST element
     */
    initialStack?: IStackPanelItem[];

    /**
     * Class name to be emitted onto the root element of the StackPanel
     */
    className?: string;
}

export interface IStackPanelItem {
    /**
     * Optional key for your panel element; your component
     * may use it to identify items, but the StackPanel does
     * not use it directly
     */
    key?: string;

    /**
     * Displayed title in the title area of the StackPanel
     */
    title?: string;

    /**
     * Content to render below the Title row
     */
    renderContent?: () => JSX.Element;
}

export interface IStackPanelProviderContext {
    /**
     * Pushes a new item onto the internal Stack
     * @param item 
     */
    push: (item: IStackPanelItem) => void;

    /**
     * Removes and returns the current top item
     */
    pop: () => IStackPanelItem | undefined;

    /**
     * Returns the current top item
     */
    peek: () => IStackPanelItem | undefined;
}