import * as tslib_1 from "tslib";
import "../../FabricInit";
import * as React from "react";
import { css } from "office-ui-fabric-react/lib/Utilities";
import { TooltipHost, TooltipOverflowMode } from "office-ui-fabric-react/lib/Tooltip";
import { getFriendlyDisplayValue } from '../../Utilities/FriendlyNumber';
import { Icon } from "office-ui-fabric-react/lib/Icon";
import { Badge } from '../../Badge';
/**
 * Presentational component that represents a single tab.
 */
var Tab = /** @class */ (function (_super) {
    tslib_1.__extends(Tab, _super);
    function Tab() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onClick = function (event) {
            var url = _this.props.url;
            var updatePivot = true;
            // If ctrl-click is pressed, and there is a URL specified for this item, then
            // don't handle the click here, allowing the browser to perform a navigation
            // (i.e. open in a new tab/window)
            if (event.ctrlKey) {
                if (url) {
                    updatePivot = false;
                }
            }
            if (updatePivot) {
                event.preventDefault();
                _this.updateSelectedItem(event);
            }
        };
        _this.onKeyUp = function (event) {
            if (!event.defaultPrevented) {
                if (event.which === 32 /* space */ || event.which === 13 /* enter */) {
                    event.preventDefault();
                    _this.updateSelectedItem(event);
                }
            }
        };
        return _this;
    }
    Tab.prototype.render = function () {
        var _a = this.props, ariaLabel = _a.ariaLabel, index = _a.index, setSize = _a.setSize, id = _a.id, isSelected = _a.isSelected, url = _a.url;
        var pivotItemContent = this.renderPivotItemContent();
        if (url) {
            return React.createElement("a", { href: url, onClick: this.onClick, onKeyUp: this.onKeyUp, className: css("vss-Tab focus-treatment", isSelected ? "selected" : ""), role: "tab", "data-is-focusable": true, key: id, "aria-label": ariaLabel, "aria-selected": isSelected, "aria-posinset": index + 1, "aria-setsize": setSize }, pivotItemContent);
        }
        else {
            return React.createElement("div", { onClick: this.onClick, onKeyUp: this.onKeyUp, className: css("vss-Tab focus-treatment", isSelected ? "selected" : ""), role: "tab", "data-is-focusable": true, key: id, "aria-label": ariaLabel, "aria-selected": isSelected, "aria-posinset": index + 1, "aria-setsize": setSize }, pivotItemContent);
        }
    };
    Tab.prototype.renderPivotItemContent = function () {
        var _a = this.props, name = _a.name, iconProps = _a.iconProps, badgeCount = _a.badgeCount;
        var badgeDisplayValue = getFriendlyDisplayValue(badgeCount);
        var badgeTooltip = undefined;
        if (badgeCount && badgeDisplayValue !== badgeCount.toString()) {
            badgeTooltip = badgeCount.toString();
        }
        var badgeContent = undefined;
        if (badgeTooltip) {
            badgeContent = (React.createElement(TooltipHost, { content: badgeTooltip }, badgeDisplayValue));
        }
        else if (badgeDisplayValue) {
            badgeContent = badgeDisplayValue;
        }
        var badge = badgeContent && React.createElement(Badge, { className: "tab-badge" }, badgeContent);
        return (React.createElement("span", { className: "innerContainer" },
            iconProps !== undefined && (React.createElement(Icon, tslib_1.__assign({ className: "icon" }, iconProps))),
            name !== undefined &&
                React.createElement("span", { className: "text" },
                    React.createElement(TooltipHost, { content: name, overflowMode: TooltipOverflowMode.Parent }, name)),
            badge));
    };
    /**
     * Updates the state with the new selected pivot.
     */
    Tab.prototype.updateSelectedItem = function (ev) {
        var onClick = this.props.onClick;
        if (this.props.onBeforeTabChange && !this.props.onBeforeTabChange(ev, this.props.id, this.props.url)) {
            ev.preventDefault();
            return;
        }
        // Update notifiers
        if (onClick) {
            onClick(this.props.id);
        }
    };
    return Tab;
}(React.Component));
export { Tab };
