import * as tslib_1 from "tslib";
import * as React from "react";
import { TabProviderContext } from "./TabProviderContext";
import { Tab } from "./Tab";
import { Tabs } from "./Tabs";
import { TabGroupProviderContext } from './TabGroupProviderContext';
var TabList = /** @class */ (function (_super) {
    tslib_1.__extends(TabList, _super);
    function TabList(props) {
        var _this = _super.call(this, props) || this;
        _this.generateGroupMap = function (groups) {
            if (!groups)
                return {};
            var map = {};
            groups.forEach(function (group) { map[group.id] = group; });
            return map;
        };
        _this.processChildren = function (groups) {
            var childGroups = {};
            React.Children.forEach(_this.props.children, function (child) {
                if (child) {
                    var groupId = child.props.groupId || "";
                    if (!childGroups[groupId]) {
                        childGroups[groupId] = {
                            items: [child],
                            groupProps: groups[groupId] || {
                                id: "",
                                name: "",
                                order: -1
                            }
                        };
                    }
                    else {
                        childGroups[groupId].items.push(child);
                    }
                }
            });
            return childGroups;
        };
        _this.processContributions = function (tabs, baseGroups, childGroups) {
            tabs.forEach(function (tab) {
                var mappedTab = _this.createRow(tab);
                var groupId = tab.groupId || "";
                if (!childGroups[groupId]) {
                    childGroups[groupId] = {
                        items: [mappedTab],
                        groupProps: baseGroups[groupId] || {
                            id: "",
                            name: "",
                            order: -1
                        }
                    };
                }
                else {
                    childGroups[groupId].items.push(mappedTab);
                }
            });
            return childGroups;
        };
        _this.processGroups = function (groups) {
            var elements = [];
            // Flattening to allow for sort
            var flatMappedGroups = [];
            for (var groupId in groups) {
                flatMappedGroups.push(groups[groupId]);
            }
            flatMappedGroups
                .sort(function (a, b) { return a.groupProps.order - b.groupProps.order; })
                .forEach(function (group) {
                elements.push(group.groupProps.name !== '' && React.createElement("div", { key: group.groupProps.name, className: "vss-TabList-Heading title-xs flex-noshrink" }, group.groupProps.name));
                elements.push.apply(elements, group.items);
            });
            return elements;
        };
        _this.createRow = function (tab) {
            return React.createElement(Tab, { key: tab.id, id: tab.id, name: tab.name, url: tab.url, onBeforeTabChange: tab.onBeforeTabChange });
        };
        _this.renderTitle = function (header) {
            return React.createElement("div", { className: "vss-TabList-Title title-s flex-noshrink" }, header);
        };
        _this.onSelectedTabIdChanged = function () {
            _this.forceUpdate();
        };
        return _this;
    }
    TabList.prototype.render = function () {
        var _this = this;
        return React.createElement(TabGroupProviderContext.Consumer, null, function (provider) {
            var tabGroups = _this.props.tabGroups;
            var mergedGroups = tabGroups ? tabGroups.concat(provider.groups) : provider.groups;
            var groupMap = _this.generateGroupMap(mergedGroups);
            return React.createElement(TabProviderContext.Consumer, null, function (provider) {
                var selectedTabId = _this.props.selectedTabId || (provider.selectedId);
                var tabGroups = _this.processContributions(provider.tabs, groupMap, _this.processChildren(groupMap));
                var trueChildren = _this.processGroups(tabGroups);
                return React.createElement("div", { className: "vss-TabList flex-column" },
                    _this.props.listTitle && _this.renderTitle(_this.props.listTitle),
                    React.createElement(Tabs, tslib_1.__assign({ selectedTabId: selectedTabId }, _this.props, { onSelectedTabChanged: provider.onSelectedTabChanged, orientation: 1 /* Vertical */ }), trueChildren));
            });
        });
    };
    return TabList;
}(React.Component));
export { TabList };
