import * as tslib_1 from "tslib";
import * as React from "react";
export var TabProviderContext = React.createContext({ selectedId: undefined, tabs: [], commandBarItems: [], renderContent: undefined, renderFilterBar: undefined, onSelectedTabChanged: undefined });
var TabProvider = /** @class */ (function (_super) {
    tslib_1.__extends(TabProvider, _super);
    function TabProvider(props) {
        var _this = _super.call(this, props) || this;
        _this.onSelectedTabChanged = function (newTabId) {
            _this.props.selectedTabId.value = newTabId;
        };
        _this.onSelectedTabIdChanged = function () {
            _this.forceUpdate();
        };
        _this.onTabsChanged = function (event) {
            var _a;
            (_a = _this.tabs).push.apply(_a, event.addedItems);
            _this.tabs = _this.tabs.sort(function (a, b) { return (a.order || 100) - (b.order || 100); });
            _this.forceUpdate();
        };
        _this.tabs = [];
        _this.observableArrays = [];
        props.providers && props.providers.forEach(function (provider) {
            var _a;
            var observableArray = provider.getItems();
            if (observableArray) {
                _this.observableArrays.push(observableArray);
                (_a = _this.tabs).push.apply(_a, observableArray.items);
            }
        });
        _this.tabs = _this.tabs.sort(function (a, b) { return (a.order || 100) - (b.order || 100); });
        return _this;
    }
    TabProvider.prototype.render = function () {
        var selectedTabId = this.props.selectedTabId.value;
        var selectedTab = undefined;
        this.tabs.forEach(function (tab) {
            if (tab.id === selectedTabId) {
                selectedTab = tab;
            }
        });
        if (selectedTab === undefined) {
            selectedTab = {
                id: selectedTabId,
                name: undefined,
                render: undefined
            };
        }
        return React.createElement(TabProviderContext.Provider, { value: { onSelectedTabChanged: this.onSelectedTabChanged, selectedId: selectedTab.id, tabs: this.tabs, commandBarItems: selectedTab.commandBarItems, renderContent: selectedTab.render, renderFilterBar: selectedTab.renderFilterBar } }, this.props.children);
    };
    TabProvider.prototype.componentDidMount = function () {
        var _this = this;
        this.props.selectedTabId && this.props.selectedTabId.subscribe(this.onSelectedTabIdChanged);
        this.observableArrays.forEach(function (observableArray) { return observableArray.subscribe(_this.onTabsChanged); });
    };
    TabProvider.prototype.componentWillUnmount = function () {
        var _this = this;
        this.observableArrays.forEach(function (observableArray) { return observableArray.unsubscribe(_this.onTabsChanged); });
        this.props.selectedTabId && this.props.selectedTabId.unsubscribe(this.onSelectedTabIdChanged);
    };
    return TabProvider;
}(React.Component));
export { TabProvider };
