import * as tslib_1 from "tslib";
import "../../FabricInit";
import * as React from "react";
import { css } from "office-ui-fabric-react/lib/Utilities";
import { FocusZone, FocusZoneDirection } from "office-ui-fabric-react/lib/FocusZone";
import { ObservableLike } from '../../Core/Observable';
/**
 * Renders all children as focusable, selectable elements and indicates whether or not they are
 * currently selected.
 */
var Tabs = /** @class */ (function (_super) {
    tslib_1.__extends(Tabs, _super);
    function Tabs(props) {
        var _this = _super.call(this, props) || this;
        _this.onTabClick = function (newTabId) {
            var onSelectedTabChanged = _this.props.onSelectedTabChanged;
            var _a = _this._tabPropsMap[newTabId], onBeforeTabChange = _a.onBeforeTabChange, url = _a.url;
            if (onBeforeTabChange) {
                onBeforeTabChange(undefined, newTabId, url);
            }
            if (onSelectedTabChanged) {
                onSelectedTabChanged(newTabId);
            }
        };
        _this.onSelectedTabIdChanged = function () {
            _this.forceUpdate();
        };
        _this._tabPropsMap = {};
        if (ObservableLike.isObservable(props.selectedTabId)) {
            props.selectedTabId.subscribe(_this.onSelectedTabIdChanged);
        }
        return _this;
    }
    Tabs.prototype.componentWillUnmount = function () {
        if (ObservableLike.isObservable(this.props.selectedTabId)) {
            this.props.selectedTabId.unsubscribe(this.onSelectedTabIdChanged);
        }
    };
    Tabs.prototype.render = function () {
        var _this = this;
        var _a = this.props, _b = _a.tabSize, tabSize = _b === void 0 ? "tall" /* Tall */ : _b, _c = _a.orientation, orientation = _c === void 0 ? 0 /* Horizontal */ : _c;
        var selectedTabId = ObservableLike.getValue(this.props.selectedTabId);
        var childrenCount = 0;
        var childIndex = 0;
        var idToIndex = {};
        React.Children.map(this.props.children, function (child) {
            if (child) {
                idToIndex[child.props.id] = childIndex++;
                childrenCount++;
            }
        });
        var children = React.Children.map(this.props.children, function (child) {
            if (child) {
                var isSelected = child.props.id === selectedTabId;
                var onClick = _this.onTabClick;
                _this._tabPropsMap[child.props.id] = child.props;
                return React.cloneElement(child, tslib_1.__assign({}, child.props, { isSelected: isSelected,
                    onClick: onClick, index: idToIndex[child.props.id], setSize: childrenCount }));
            }
        });
        var orientationClass = orientation === 1 /* Vertical */ ? "flex-column" : "flex-row";
        var focusZoneDirection = orientation === 1 /* Vertical */ ? FocusZoneDirection.vertical : FocusZoneDirection.horizontal;
        return React.createElement(FocusZone, { className: css("vss-Tabs", orientationClass, this.props.className, tabSize), role: "tablist", direction: focusZoneDirection }, children);
    };
    return Tabs;
}(React.Component));
export { Tabs };
