import * as tslib_1 from "tslib";
import "../../FabricInit";
import * as React from "react";
import { VssIcon } from '../../VssIcon';
import { BaseComponent, inputProperties, css, getNativeProps, memoizeFunction } from 'office-ui-fabric-react/lib/Utilities';
var TextField = /** @class */ (function (_super) {
    tslib_1.__extends(TextField, _super);
    function TextField(props) {
        var _this = _super.call(this, props) || this;
        _this.onInput = function (ev) {
            var value = _this.state.value;
            var _a = _this.props, onChanged = _a.onChanged, onBeforeChange = _a.onBeforeChange, onChange = _a.onChange, validator = _a.validator;
            var newValue = ev.target.value;
            if (onChange) {
                onChange(ev, newValue);
                _this.adjustInputHeight();
            }
            else {
                if (newValue === value) {
                    return;
                }
                onBeforeChange(newValue);
                _this.setState({
                    value: newValue
                }, function () { onChanged(newValue), _this.adjustInputHeight(); });
                if (validator) {
                    _this._async.debounce(_this.setErrorMessage, validator.deferredValidationTime !== undefined ? validator.deferredValidationTime : 200)(newValue);
                }
            }
        };
        _this.onFocus = function () {
            var validator = _this.props.validator;
            if (validator && validator.validateOnFocusIn) {
                _this.setErrorMessage(_this.state.value);
            }
        };
        _this.onBlur = function () {
            var validator = _this.props.validator;
            if (validator && validator.validateOnFocusOut) {
                _this.setErrorMessage(_this.state.value);
            }
        };
        _this.checkPropValue = memoizeFunction(function (value) {
            var validator = _this.props.validator;
            if (validator && value) {
                _this._async.debounce(_this.setErrorMessage, validator.deferredValidationTime !== undefined ? validator.deferredValidationTime : 200)(value);
            }
            return null;
        }, 1);
        var startingValue;
        if (props.value) {
            startingValue = props.value;
        }
        else if (props.defaultValue) {
            startingValue = props.defaultValue;
        }
        else {
            startingValue = "";
        }
        _this.state = {
            value: startingValue,
            errorMessage: undefined
        };
        return _this;
    }
    TextField.prototype.render = function () {
        var _this = this;
        var _a = this.props, multiline = _a.multiline, className = _a.className, description = _a.description, onRenderDescription = _a.onRenderDescription, value = _a.value;
        this.checkPropValue(value);
        return React.createElement("div", { className: css("vss-TextField flex-column", className) },
            multiline ? this.renderTextArea() : this.renderInput(),
            onRenderDescription ?
                React.createElement("div", { id: "description", ref: function (el) { _this.description = el; } }, onRenderDescription(this.props, this._onRenderDescription)) :
                (description ? this._onRenderDescription(this.props) : null),
            this.state.errorMessage && React.createElement("div", { className: "error-message" }, this.state.errorMessage));
    };
    TextField.prototype.componentDidMount = function () {
        var validator = this.props.validator;
        if (validator && validator.validateOnLoad !== false) {
            this.setErrorMessage(this.state.value);
        }
        this.adjustInputStyle();
        this.adjustInputHeight();
        this.forceUpdate();
    };
    TextField.prototype.renderInput = function () {
        var _this = this;
        var _a = this.props, prefixIconProps = _a.prefixIconProps, suffixIconProps = _a.suffixIconProps, textFieldSize = _a.textFieldSize, disabled = _a.disabled, onRenderPrefix = _a.onRenderPrefix, onRenderSuffix = _a.onRenderSuffix, ariaLabel = _a.ariaLabel, value = _a.value;
        var errorMessage = this.state.errorMessage;
        var inputProps = getNativeProps(this.props, inputProperties, ['defaultValue']);
        return React.createElement("div", { className: "flex flex-row" },
            React.createElement("div", { className: "input-container flex-row flex-grow" },
                onRenderPrefix ? React.createElement("span", { ref: function (el) { _this.prefix = el; }, className: css("prefix-suffix-positioning prefix custom", textFieldSize) }, onRenderPrefix()) :
                    prefixIconProps ? React.createElement(VssIcon, tslib_1.__assign({}, prefixIconProps, { className: css("prefix-suffix-positioning prefix icon", textFieldSize) })) : null,
                React.createElement("input", tslib_1.__assign({}, inputProps, { className: css("input flex-grow", prefixIconProps && !onRenderPrefix && "prefix-icon-padding", suffixIconProps && !onRenderSuffix && "suffix-icon-padding", textFieldSize, disabled && "disabled", errorMessage && "error"), style: this.inputStyle, value: value ? value : this.state.value, onChange: this.onInput, onFocus: this.onFocus, onBlur: this.onBlur, "aria-label": ariaLabel, "aria-describedby": this.description ? this.description.id : this.props['aria-describedby'], "aria-invalid": !!this.state.errorMessage })),
                onRenderSuffix ? React.createElement("span", { ref: function (el) { _this.suffix = el; }, className: css("prefix-suffix-positioning suffix custom", textFieldSize) }, onRenderSuffix()) :
                    (suffixIconProps ? React.createElement(VssIcon, tslib_1.__assign({}, suffixIconProps, { className: css("prefix-suffix-positioning suffix icon", textFieldSize) })) : null)));
    };
    TextField.prototype.renderTextArea = function () {
        var _this = this;
        var _a = this.props, resizable = _a.resizable, disabled = _a.disabled, ariaLabel = _a.ariaLabel, autoAdjustHeight = _a.autoAdjustHeight, value = _a.value;
        var errorMessage = this.state.errorMessage;
        var textAreaProps = getNativeProps(this.props, inputProperties, ['defaultValue']);
        return React.createElement("textarea", tslib_1.__assign({}, textAreaProps, { className: css("textarea flex-grow", (!resizable || autoAdjustHeight) && "not-resizable", disabled && "disabled", errorMessage && "error", autoAdjustHeight && "auto-adjust"), value: value ? value : this.state.value, ref: function (el) { _this.textarea = el; }, onChange: this.onInput, onFocus: this.onFocus, onBlur: this.onBlur, "aria-label": ariaLabel, "aria-describedby": this.description ? this.description.id : this.props['aria-describedby'], "aria-invalid": !!this.state.errorMessage }));
    };
    TextField.prototype.adjustInputStyle = function () {
        var rules = {};
        if (this.prefix) {
            rules.paddingLeft = this.prefix.offsetWidth + 6;
        }
        if (this.suffix) {
            rules.paddingRight = this.suffix.offsetWidth + 6;
        }
        this.inputStyle = rules;
    };
    TextField.prototype._onRenderDescription = function (props) {
        var _this = this;
        return React.createElement("div", { id: "description", ref: function (el) { _this.description = el; }, className: css("description") }, props.description);
    };
    TextField.prototype.setErrorMessage = function (value) {
        var validator = this.props.validator;
        var errorMessage = validator.validate(value);
        this.setState({
            errorMessage: errorMessage,
        });
        if (validator.onNotifyValidationResult) {
            validator.onNotifyValidationResult(errorMessage, value);
        }
    };
    TextField.prototype.adjustInputHeight = function () {
        if (this.props.multiline && this.props.autoAdjustHeight && this.textarea) {
            this.textarea.style.height = (this.textarea.scrollHeight + 2) + 'px';
        }
    };
    TextField.defaultProps = {
        multiline: false,
        resizable: false,
        autoAdjustHeight: true,
        onBeforeChange: function () { },
        onChanged: function () { },
        textFieldSize: "normal" /* Normal */,
    };
    return TextField;
}(BaseComponent));
export { TextField };
