import * as tslib_1 from "tslib";
import "../../FabricInit";
import * as React from "react";
import { TreeList } from "../../Components/Tree/TreeList";
import { BaseComponent } from "office-ui-fabric-react/lib/Utilities";
var Tree = /** @class */ (function (_super) {
    tslib_1.__extends(Tree, _super);
    function Tree(props) {
        var _this = _super.call(this, props) || this;
        _this.onItemToggled = function (treeListItem) {
            if (treeListItem.isExpandable) {
                treeListItem.isExpanded = !treeListItem.isExpanded;
                var treeItemProvider = _this.props.treeItemProvider;
                if (treeListItem.isExpanded) {
                    // If the item is now expanded, we need to re-build this part of the sub-tree
                    _this.buildTree(treeItemProvider, treeListItem.treeItem.item);
                    // Call forceUpdate to account for a new isLoading state for one of the nodes.
                    _this.forceUpdate();
                }
                else {
                    // If the the item is now collapse, we need to remove all of the children of
                    // this item from the tree.
                    var oldTreeListItems = _this.state.treeListItems;
                    var newTreeListItems = [];
                    var indentLevel = undefined;
                    // Loop through all of the items currently in the tree and add these items, 
                    // until we hit the item that is being collapsed.
                    // Then, add this item, and do not add another item until we find an item with
                    // the same indent level as the item that is being collapsed.
                    for (var _i = 0, oldTreeListItems_1 = oldTreeListItems; _i < oldTreeListItems_1.length; _i++) {
                        var item = oldTreeListItems_1[_i];
                        if (item === treeListItem) {
                            indentLevel = treeListItem.indentLevel;
                            newTreeListItems.push(item);
                        }
                        else if (indentLevel === undefined || indentLevel >= item.indentLevel) {
                            newTreeListItems.push(item);
                            indentLevel = undefined;
                        }
                    }
                    treeListItem.isLoading = false;
                    _this.setState({
                        treeListItems: newTreeListItems
                    });
                }
            }
        };
        _this.childrenMap = {};
        var columns = _this.props.columns.slice();
        var treeListItemColumns = _this.buildColumns(columns);
        _this.state = {
            treeListItems: [],
            treeListItemColumns: treeListItemColumns
        };
        // build the tree from the root
        _this.buildTree(props.treeItemProvider, undefined);
        return _this;
    }
    Tree.prototype.setSelectedItemsById = function (selectedItemIds) {
        if (this.treeList) {
            this.treeList.setSelectedItemsById(selectedItemIds);
        }
    };
    Tree.prototype.componentWillReceiveProps = function (newProps) {
        if (this.props.columns !== newProps.columns) {
            var columns = newProps.columns.slice();
            var treeListItemColumns = this.buildColumns(columns);
            this.setState({
                treeListItemColumns: treeListItemColumns
            });
        }
        if (this.props.treeItemProvider !== newProps.treeItemProvider) {
            // If object instance is changed, we assume it's new set of items, so we re-build them
            this.childrenMap = {};
            this.buildTree(newProps.treeItemProvider, undefined);
        }
    };
    Tree.prototype.render = function () {
        var _this = this;
        var _a = this.state, treeListItems = _a.treeListItems, treeListItemColumns = _a.treeListItemColumns;
        return React.createElement(TreeList, tslib_1.__assign({}, this.props, { componentRef: function (treeList) { return _this.treeList = treeList; }, treeListItems: treeListItems, columns: treeListItemColumns, onItemToggled: this.onItemToggled }));
    };
    Tree.prototype.buildColumns = function (columns) {
        return columns.map(function (column) {
            return tslib_1.__assign({}, column, { onRender: function (treeListItem) { return column.onRender(treeListItem.treeItem); } });
        });
    };
    Tree.prototype.buildTree = function (treeItemProvider, item) {
        var _this = this;
        var promise = Promise.resolve(treeItemProvider.getChildren(item));
        // this will only be undefined if we are re-building the entire tree
        var treeListItem = item !== undefined && treeItemProvider.getTreeListItem(item);
        if (treeListItem) {
            treeListItem.isLoading = true;
        }
        promise.then(function (children) {
            // Once we have the children, we need to make the recursive call to addTreeItems
            // to aggregate all of the children.
            var promises = [];
            var indentLevel = item !== undefined ? treeListItem.indentLevel + 1 : 0;
            for (var _i = 0, children_1 = children; _i < children_1.length; _i++) {
                var item_1 = children_1[_i];
                promises.push(_this.addTreeItems(treeItemProvider, item_1, indentLevel));
            }
            // By using all, we're in an all or nothing situation, where if a single promise
            // is rejected, we won't show any nodes for the sub-tree.
            Promise.all(promises).then(function (results) {
                var childTreeListItems = [].concat.apply([], results);
                var newTreeListItems = [];
                if (item === undefined) {
                    newTreeListItems = childTreeListItems;
                }
                else {
                    // If this was a sub-tree, copy over the old tree items, inserting the
                    // children of this item in the correct spot.
                    var oldTreeListItems = _this.state.treeListItems;
                    for (var _i = 0, oldTreeListItems_2 = oldTreeListItems; _i < oldTreeListItems_2.length; _i++) {
                        var oldItem = oldTreeListItems_2[_i];
                        newTreeListItems.push(oldItem);
                        if (item === oldItem.treeItem.item) {
                            newTreeListItems = newTreeListItems.concat(childTreeListItems);
                        }
                    }
                }
                // If the item is no longer expanded or no longer loading, then something has
                // cancelled the expand operation, so we should no longer call setState.
                if (!treeListItem || (treeListItem && treeListItem.isExpanded && treeListItem.isLoading)) {
                    if (treeListItem) {
                        treeListItem.isLoading = false;
                    }
                    _this.setState({
                        treeListItems: newTreeListItems
                    });
                }
            });
        });
    };
    Tree.prototype.addTreeItems = function (treeItemProvider, item, indentLevel) {
        var _this = this;
        var treeListItems = [];
        var treeListItem = this.getOrCreateTreeListItem(treeItemProvider, item, indentLevel);
        treeListItems.push(treeListItem);
        if (treeListItem.isExpanded) {
            // If the item is expanded, we have to add its children. We'll first try to get the children
            // from our cache - if we don't have them, we'll call getChildren on the item provider
            var cachedChildren = this.childrenMap[treeListItem.treeItem.id];
            if (cachedChildren !== undefined) {
                treeListItem.isLoading = false;
                return Promise.resolve(cachedChildren);
            }
            return Promise.resolve(treeItemProvider.getChildren(item)).then(function (children) {
                var promises = [];
                for (var _i = 0, children_2 = children; _i < children_2.length; _i++) {
                    var item_2 = children_2[_i];
                    promises.push(_this.addTreeItems(treeItemProvider, item_2, indentLevel + 1));
                }
                return Promise.all(promises).then(function (results) {
                    var allChildren = [].concat.apply(treeListItems, results);
                    _this.childrenMap[treeListItem.treeItem.id] = allChildren;
                    return allChildren;
                });
            });
        }
        else {
            return Promise.resolve(treeListItems);
        }
    };
    Tree.prototype.getOrCreateTreeListItem = function (treeItemProvider, item, indentLevel) {
        var treeListItem = treeItemProvider.getTreeListItem(item);
        if (!treeListItem) {
            var treeItem = treeItemProvider.getTreeItem(item);
            treeListItem = {
                treeItem: treeItem,
                indentLevel: indentLevel,
                isExpandable: treeItem.isExpandable,
                isExpanded: !!treeItem.defaultIsExpanded,
                isLoading: false
            };
            treeItemProvider.setTreeListItem(item, treeListItem);
        }
        return treeListItem;
    };
    return Tree;
}(BaseComponent));
export { Tree };
