import * as tslib_1 from "tslib";
import "../../FabricInit";
import * as React from "react";
import { VssDetailsList } from "../../VssDetailsList";
import { BaseComponent, css } from "office-ui-fabric-react/lib/Utilities";
import { Icon } from "office-ui-fabric-react/lib/Icon";
import { DetailsRow } from "office-ui-fabric-react/lib/DetailsList";
import { Selection, SelectionMode } from "office-ui-fabric-react/lib/Selection";
import { Spinner, SpinnerSize } from "office-ui-fabric-react/lib/Spinner";
import "./Tree.css";
var TreeList = /** @class */ (function (_super) {
    tslib_1.__extends(TreeList, _super);
    function TreeList(props) {
        var _this = _super.call(this, props) || this;
        _this.newlySelectedItem = undefined;
        _this.onRenderRow = function (props, renderer) {
            var treeListItem = props.item;
            if (treeListItem.isExpandable) {
                return React.createElement(ExpandCollapseRow, { expanded: treeListItem.isExpanded, treeListItem: treeListItem, rowProps: props, collapse: _this.onCollapse, expand: _this.onExpand, onKeyDown: _this.props.onRowKeyDown });
            }
            return renderer(props);
        };
        _this.onCollapse = function (treeListItem) {
            if (treeListItem.isExpanded) {
                _this.props.onItemToggled(treeListItem);
                return true;
            }
            return false;
        };
        _this.onExpand = function (treeListItem) {
            if (!treeListItem.isExpanded) {
                _this.props.onItemToggled(treeListItem);
                return true;
            }
            return false;
        };
        _this.onSelectionChanged = function () {
            var selection = _this.selection.getSelection();
            var selectedItems = [];
            if (selection && selection.length > 0) {
                var selectedTreeListItem = selection[0];
                selectedItems = [selectedTreeListItem.treeItem.item];
                _this.newlySelectedItem = selectedTreeListItem;
                _this.itemsToSelect = undefined;
            }
            if (_this.props.onSelectedItemsChanged) {
                _this.props.onSelectedItemsChanged(selectedItems);
            }
        };
        _this.onItemToggled = function (treeListItem, forceCollapse) {
            if (!treeListItem.isExpanded) {
                _this.props.onItemToggled(treeListItem);
            }
            else {
                if (forceCollapse) {
                    _this.props.onItemToggled(treeListItem);
                }
                else if (_this.canCollapse(treeListItem)) {
                    _this.props.onItemToggled(treeListItem);
                }
                else {
                    // The item was just now selected, so we don't want to collapse, but we should next time
                    _this.newlySelectedItem = undefined;
                }
            }
        };
        _this.canCollapse = function (treeListItem) {
            // Do not collapse the item if it was just now selected
            return _this.selection.mode === SelectionMode.none ? true : _this.newlySelectedItem !== treeListItem;
        };
        // Don't support multiple selection mode
        var selectionMode = props.selectionMode === SelectionMode.none ? SelectionMode.none : SelectionMode.single;
        _this.selection = new Selection({
            onSelectionChanged: _this.onSelectionChanged,
            selectionMode: selectionMode,
            getKey: function (item) { return item.treeItem.id; }
        });
        return _this;
    }
    TreeList.prototype.setSelectedItemsById = function (selectedItemIds) {
        this.selection.setAllSelected(false);
        this.itemsToSelect = selectedItemIds;
        this.setSelection();
    };
    TreeList.prototype.render = function () {
        var _this = this;
        var _a = this.props, className = _a.className, treeListItems = _a.treeListItems, primaryTreeColumnKey = _a.primaryTreeColumnKey, actionsColumnKey = _a.actionsColumnKey, onPrimaryCellClick = _a.onPrimaryCellClick, getMenuItemProviders = _a.getMenuItemProviders, getMenuItems = _a.getMenuItems, allocateSpaceForActionsButtonWhileHidden = _a.allocateSpaceForActionsButtonWhileHidden, isHeaderVisible = _a.isHeaderVisible, initialFocusedIndex = _a.initialFocusedIndex, onRowDidMount = _a.onRowDidMount;
        var columns = this.props.columns.slice();
        var primaryColumnIndex = 0;
        if (primaryTreeColumnKey !== undefined) {
            for (var i = 0; i < columns.length; i++) {
                if (columns[i].key === primaryTreeColumnKey) {
                    primaryColumnIndex = i;
                    break;
                }
            }
        }
        var initialColumnRender = columns[primaryColumnIndex].onRender;
        var newColumnRender = function (item, index, col) {
            return React.createElement(TreeListPrimaryCell, { onItemToggled: _this.onItemToggled, treeListItem: item, index: index, column: col, onColumnRender: initialColumnRender, indentPaddingInPixels: 20, onClick: onPrimaryCellClick });
        };
        columns[primaryColumnIndex] = tslib_1.__assign({}, columns[primaryColumnIndex], { onRender: newColumnRender });
        return React.createElement("div", { role: "treegrid" },
            React.createElement(VssDetailsList, { className: css("tree-list", className), isHeaderVisible: isHeaderVisible, items: treeListItems, initialFocusedIndex: initialFocusedIndex, columns: columns, actionsColumnKey: actionsColumnKey, getMenuItems: getMenuItems, allocateSpaceForActionsButtonWhileHidden: allocateSpaceForActionsButtonWhileHidden, getMenuItemProviders: getMenuItemProviders, onRenderRow: this.onRenderRow, usePresentationStyles: true, setKey: "0", selection: this.selection, selectionPreservedOnEmptyClick: true, onRowDidMount: onRowDidMount }));
    };
    TreeList.prototype.componentDidUpdate = function () {
        this.setSelection();
    };
    TreeList.prototype.setSelection = function () {
        if (this.itemsToSelect) {
            var idsToRetry = [];
            for (var _i = 0, _a = this.itemsToSelect; _i < _a.length; _i++) {
                var id = _a[_i];
                this.selection.setKeySelected(id, true, false);
                if (!this.selection.isKeySelected(id)) {
                    idsToRetry.push(id);
                }
            }
            if (idsToRetry.length === 0) {
                idsToRetry = undefined;
            }
            this.itemsToSelect = idsToRetry;
        }
    };
    return TreeList;
}(BaseComponent));
export { TreeList };
var TreeListPrimaryCell = /** @class */ (function (_super) {
    tslib_1.__extends(TreeListPrimaryCell, _super);
    function TreeListPrimaryCell() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onExpandCollapseButtonClicked = function (event) {
            event.stopPropagation();
            // Force collapse of the item if the expand collapse button was explicity clicked.
            _this.props.onItemToggled(_this.props.treeListItem, true);
        };
        _this.onCellClicked = function (event) {
            if (_this.props.onClick) {
                _this.props.onClick(event, _this.props.treeListItem.treeItem.item);
            }
            else {
                _this.props.onItemToggled(_this.props.treeListItem, false);
            }
        };
        return _this;
    }
    TreeListPrimaryCell.prototype.render = function () {
        var treeListItem = this.props.treeListItem;
        var indentPixels = this.props.indentPaddingInPixels * treeListItem.indentLevel;
        var spinner = treeListItem.isLoading && React.createElement(Spinner, { size: SpinnerSize.xSmall, className: "tree-list-spinner" });
        // Even if the item isn't expandable, we will show a hidden icon to keep items aligned.
        var collapseButton = React.createElement(Icon, { className: css("tree-chevron", { "invisible": !treeListItem.isExpandable }), iconName: treeListItem.isExpanded ? "ChevronDownSmall" : "ChevronRightSmall", onClick: this.onExpandCollapseButtonClicked });
        return React.createElement("div", { className: "tree-column-wrapper", onClick: this.onCellClicked, style: {
                "paddingLeft": indentPixels + "px"
            } },
            spinner || collapseButton,
            this.props.onColumnRender(this.props.treeListItem, this.props.index, this.props.column));
    };
    return TreeListPrimaryCell;
}(React.Component));
var ExpandCollapseRow = /** @class */ (function (_super) {
    tslib_1.__extends(ExpandCollapseRow, _super);
    function ExpandCollapseRow() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onKeyDown = function (event) {
            if (event.which === 39 /* KeyCode.RightArrow */) {
                if (_this.props.expand(_this.props.treeListItem)) {
                    event.preventDefault();
                    event.stopPropagation();
                }
            }
            else if (event.which === 37 /* KeyCode.LeftArrow */) {
                if (_this.rowComponent !== null && _this.detailsElement !== null && document.activeElement != _this.detailsElement) {
                    _this.rowComponent.focus();
                }
                else if (_this.props.collapse(_this.props.treeListItem)) {
                    event.preventDefault();
                    event.stopPropagation();
                }
            }
            else if (_this.props.onKeyDown) {
                _this.props.onKeyDown(event, _this.props.rowProps.item.treeItem.item);
            }
        };
        return _this;
    }
    ExpandCollapseRow.prototype.render = function () {
        var _this = this;
        return (React.createElement("div", { onKeyDown: this.onKeyDown, ref: function (rowElement) { _this.rowElement = rowElement; } },
            React.createElement(DetailsRow, tslib_1.__assign({}, this.props.rowProps, { "aria-expanded": this.props.expanded, ref: function (rowComponent) { _this.rowComponent = rowComponent; } }))));
    };
    ExpandCollapseRow.prototype.componentDidMount = function () {
        this.detailsElement = null;
        if (this.rowElement !== null) {
            this.detailsElement = this.rowElement.querySelector("[tabIndex='0']") || this.rowElement.children[0];
        }
    };
    return ExpandCollapseRow;
}(React.Component));
