import * as tslib_1 from "tslib";
import "../../FabricInit";
import * as React from "react";
import { Callout } from "office-ui-fabric-react/lib/Callout";
import { IconButton } from "office-ui-fabric-react/lib/Button";
import { FocusZone, FocusZoneDirection } from "office-ui-fabric-react/lib/FocusZone";
import { Link } from "office-ui-fabric-react/lib/Link";
import { Icon } from "office-ui-fabric-react/lib/Icon";
import { ResizeGroup } from "office-ui-fabric-react/lib/ResizeGroup";
import { TooltipHost, TooltipOverflowMode } from '../../Tooltip';
import { BaseComponent, createRef, css, findIndex, getId, getRTL } from "office-ui-fabric-react/lib/Utilities";
import { VssIcon } from '../../VssIcon';
import { PickList, PICKER_CHANGE_EVENT } from '../../PickList';
import * as VSSUIResources from '../../VSS.Resources.VSSUI';
import { ItemIndicatorSet } from '../../ItemIndicator';
import "./VssBreadcrumb.css";
var DEFAULT_MAXDISPLAY_ITEMS = 4;
var PICKLIST_HOVER_TIMEOUT = 80;
var PROVIDER_LOAD_TIMEOUT = 100;
var PickerDropdown = /** @class */ (function (_super) {
    tslib_1.__extends(PickerDropdown, _super);
    function PickerDropdown(props) {
        var _this = _super.call(this, props) || this;
        _this._onPositioned = function () {
            _this._pickList.focus();
        };
        _this._onDismiss = function () {
            _this.closeDropdown();
        };
        _this._updateDelegate = function () { return _this.forceUpdate(); };
        _this.state = { calloutIsVisible: false };
        return _this;
    }
    PickerDropdown.prototype.render = function () {
        var _this = this;
        var _a = this.props, getPinElement = _a.getPinElement, item = _a.item, width = _a.width, minWidth = _a.minWidth, maxWidth = _a.maxWidth, maxHeight = _a.maxHeight, pickListClassName = _a.pickListClassName;
        if (this.state.calloutIsVisible) {
            var itemPickerProvider_1 = item.itemPicker;
            return React.createElement(Callout, { isBeakVisible: false, target: getPinElement(), onPositioned: this._onPositioned, onDismiss: this._onDismiss },
                React.createElement(PickList, { className: css("picker-dropdown", pickListClassName), componentRef: function (pickList) { return (_this._pickList = pickList); }, getActions: function () { return itemPickerProvider_1.actions; }, getListItem: itemPickerProvider_1.getListItem, groups: itemPickerProvider_1.groups, pivots: itemPickerProvider_1.pivots, indicators: itemPickerProvider_1.dropdownIndicators, initiallySelectedItems: itemPickerProvider_1.selectedItem ? [itemPickerProvider_1.selectedItem] : undefined, isSearchable: itemPickerProvider_1.isSearchable, items: itemPickerProvider_1.getItems(), searchResultsLoadingText: itemPickerProvider_1.searchResultsLoadingText, noItemsText: itemPickerProvider_1.noItemsText, itemAriaDescription: itemPickerProvider_1.itemAriaDescription, searchNoResultsText: itemPickerProvider_1.searchNoResultsText, searchResultsGroupName: itemPickerProvider_1.searchResultsGroupName, searchTextPlaceholder: itemPickerProvider_1.searchTextPlaceholder, shouldVirtualize: function (items) {
                        return (items || []).length > 50;
                    }, width: width, minWidth: minWidth, maxWidth: maxWidth, maxHeight: maxHeight, onFocusLost: this._onDismiss, onSearch: itemPickerProvider_1.onSearch, onSelectionChanged: item.onSelectionChanged, onRenderItemText: itemPickerProvider_1.onRenderItemText, onSelectedPivotChanged: itemPickerProvider_1.onSelectedPivotChanged }));
        }
        return null;
    };
    PickerDropdown.prototype.componentDidMount = function () {
        if (this.props.item.isHeaderPickerItem && this.props.item.itemPicker) {
            this._events.on(this.props.item.itemPicker, PICKER_CHANGE_EVENT, this._updateDelegate);
        }
    };
    PickerDropdown.prototype.componentWillUnmount = function () {
        if (this.props.item.isHeaderPickerItem && this.props.item.itemPicker) {
            this._events.off(this.props.item.itemPicker, PICKER_CHANGE_EVENT, this._updateDelegate);
        }
    };
    PickerDropdown.prototype.componentWillReceiveProps = function (nextProps) {
        if (this.props.item.isHeaderPickerItem && this.props.item.itemPicker && this.props.item.itemPicker !== nextProps.item.itemPicker) {
            this._events.off(this.props.item.itemPicker, PICKER_CHANGE_EVENT, this._updateDelegate);
            this._events.on(nextProps.item.itemPicker, PICKER_CHANGE_EVENT, this._updateDelegate);
        }
    };
    PickerDropdown.prototype.openDropdown = function () {
        if (this.props.onPickListOpened) {
            this.props.onPickListOpened();
        }
        this.setState({ calloutIsVisible: true });
    };
    PickerDropdown.prototype.closeDropdown = function () {
        this.setState({ calloutIsVisible: false });
        if (this.props.onPickListClosed) {
            this.props.onPickListClosed();
        }
    };
    PickerDropdown.prototype.toggleDropdown = function () {
        if (this.state.calloutIsVisible) {
            this.closeDropdown();
        }
        else {
            this.openDropdown();
        }
    };
    return PickerDropdown;
}(BaseComponent));
var VssBreadcrumb = /** @class */ (function (_super) {
    tslib_1.__extends(VssBreadcrumb, _super);
    function VssBreadcrumb(props) {
        var _this = _super.call(this, props) || this;
        _this._pinElements = {};
        _this._linkElements = {};
        _this._linkItems = [];
        _this._focusedLinkIndex = -1;
        _this._contributedItems = [];
        _this._onItemClick = function (item) {
            _this.setState({
                pickerItemKey: item.key
            }, function () {
                if (_this.pickList) {
                    _this.pickList.toggleDropdown();
                }
            });
        };
        _this.onRenderItem = function (item, defaultRenderer) {
            var _a = _this.props, items = _a.items, presentationStyles = _a.presentationStyles, shouldUseFocusZone = _a.shouldUseFocusZone;
            var pickerItemKey = _this.state.pickerItemKey;
            var lastItem = items[items.length - 1];
            // We want the text AND the icon to be clickable, but since the defaultRenderer attaches the onClick
            // to the text only, we make sure that doesn't even render an on-click and that the onclick goes to
            // the element that contains both the text and icon.
            var itemNoOnClick = tslib_1.__assign({}, item);
            delete itemNoOnClick.onClick;
            delete itemNoOnClick.href;
            var icon = _this._getIcon(item.leftIconProps);
            var originalDefaultRendered = defaultRenderer(itemNoOnClick);
            var originalClassName = originalDefaultRendered.props && originalDefaultRendered.props.className;
            var itemClass = presentationStyles & 1 /* smallSize */ ? "vss-Breadcrumb--itemText-small" : "vss-Breadcrumb--itemText vss-HubBreadcrumb--itemText";
            itemClass = css(itemClass, "ellipsis-text", { "vss-Breadcrumb--light": !!(presentationStyles & 2 /* lightColor */) });
            var clonedElementProps = tslib_1.__assign({}, originalDefaultRendered.props, { className: css(originalClassName, itemClass), style: originalDefaultRendered.props.style ? tslib_1.__assign({}, originalDefaultRendered.props.style) : {} });
            if (_this.props.maxItemWidth) {
                clonedElementProps.style.maxWidth = _this.props.maxItemWidth;
            }
            var defaultRendered = React.cloneElement(originalDefaultRendered, clonedElementProps);
            var tabIndex = shouldUseFocusZone ? 0 : -2;
            var ariaDescribedById = getId("vss-breadcrumb-item-described-by");
            if (item.isHeaderPickerItem && item.itemPicker) {
                var iconVisible = !item.nonInteractive && !!(presentationStyles & 8 /* dividerLink */);
                var onItemClick = void 0;
                if (!item.nonInteractive) {
                    onItemClick = _this._onItemClick.bind(_this, item);
                }
                return (React.createElement("div", { ref: function (elem) {
                        if (elem) {
                            _this._pinElements[item.key] = elem;
                        }
                    }, className: css("vss-Breadcrumb--item vss-HubBreadcrumb--item", { "no-click": !onItemClick, "vss-Breadcrumb--item-min-height": !!(presentationStyles & 16 /* minHeight */) }) },
                    React.createElement(Link, { key: "BreadcrumbItem$" + item.key, componentRef: function (elem) {
                            if (elem) {
                                _this._linkElements[item.key] = elem;
                            }
                        }, onClick: onItemClick, href: item.href, "aria-haspopup": true, "aria-expanded": _this.state.pickerIsOpen, "aria-label": item.ariaLabel, "aria-describedby": ariaDescribedById, tabIndex: tabIndex, onFocus: shouldUseFocusZone ? undefined : _this._onLinkFocus.bind(_this, item.key) },
                        React.createElement("div", { className: 'vss-Breadcrumb--itemContainer vss-HubBreadcrumb--itemContainer' },
                            icon,
                            defaultRendered,
                            iconVisible && _this._getBreadcrumbIcon(item)),
                        React.createElement("div", { id: ariaDescribedById, className: "vss-Breadcrumb--hiddenElement" }, item.ariaDescription)),
                    pickerItemKey && pickerItemKey === item.key && _this._renderPickerDropdown(item),
                    _this._getIndicators(item)));
            }
            else if (item.onClick || item.href) {
                return (React.createElement("div", { className: css("vss-Breadcrumb--item vss-HubBreadcrumb--item", { "vss-Breadcrumb--item-min-height": !!(presentationStyles & 16 /* minHeight */) }) },
                    React.createElement(Link, { componentRef: function (elem) {
                            if (elem) {
                                _this._linkElements[item.key] = elem;
                            }
                        }, key: "BreadcrumbItem$" + item.key, onClick: item.onClick, href: item.href, "aria-label": item.ariaLabel, "aria-describedby": ariaDescribedById, tabIndex: tabIndex, onFocus: shouldUseFocusZone ? undefined : _this._onLinkFocus.bind(_this, item.key) },
                        React.createElement("div", { className: "vss-Breadcrumb--itemContainer" },
                            icon,
                            defaultRendered),
                        React.createElement("div", { id: ariaDescribedById, className: "vss-Breadcrumb--hiddenElement" }, item.ariaDescription))));
            }
            else {
                return (React.createElement("div", { className: css("vss-Breadcrumb--item vss-HubBreadcrumb--item", { "vss-Breadcrumb--item-min-height": !!(presentationStyles & 16 /* minHeight */) }), "aria-label": item.ariaLabel },
                    icon,
                    defaultRendered));
            }
        };
        _this._onLinkFocus = function (itemKey) {
            if (_this._focusedLinkIndex < 0) {
                _this._focusedLinkIndex = findIndex(_this._linkItems, function (item) { return item.key === itemKey; });
            }
        };
        _this._getCustomDivider = function (dividerProps) {
            var dividerItem = dividerProps.item;
            var presentationStyles = _this.props.presentationStyles;
            if (presentationStyles !== undefined && !(presentationStyles & 8 /* dividerLink */)) {
                return (React.createElement("div", { key: "BreadcrumbItemDivider$" + dividerItem.key, className: css("vss-Breadcrumb--divider vss-HubBreadcrumb--divider", {
                        "vss-Breadcrumb--divider-extra-padding": !!(presentationStyles & 4 /* extraPadding */),
                        "vss-Breadcrumb--divider-light": !!(presentationStyles & 2 /* lightColor */)
                    }) }, _this._getBreadcrumbIcon(dividerItem)));
            }
            return (React.createElement("div", { key: "BreadcrumbItemDivider$" + dividerItem.key, className: "vss-Breadcrumb--divider-container", ref: function (elem) {
                    if (elem) {
                        _this._pinElements[dividerItem.key] = elem;
                    }
                } }, _this._getDividerButtonOrIcon(dividerItem)));
        };
        _this._onKeyDown = function (event) {
            if (event.which === 37 /* KeyCode.LeftArrow */ && _this._focusedLinkIndex > 0) {
                _this._linkElements[_this._linkItems[--_this._focusedLinkIndex].key].focus();
                event.preventDefault();
            }
            if (event.which === 39 /* KeyCode.RightArrow */ && _this._focusedLinkIndex >= 0 && _this._focusedLinkIndex < _this._linkItems.length - 1) {
                _this._linkElements[_this._linkItems[++_this._focusedLinkIndex].key].focus();
                event.preventDefault();
            }
            if (event.which === 40 /* KeyCode.DownArrow */) {
                _this._onItemClick(_this._linkItems[_this._focusedLinkIndex]);
            }
        };
        _this._onBlur = function () {
            if (!_this.state.pickerIsOpen) {
                _this._focusedLinkIndex = -1;
            }
        };
        _this._itemsChanged = function (items, removedItems) {
            if (_this._isMounted) {
                // Build map of what is currently in the list of breadcrumb items
                var existingItemsMap_1 = {};
                _this._contributedItems.forEach(function (item) {
                    // if an item is being removed, then do not add to the existing map.
                    if (!removedItems || removedItems.indexOf(item.key) === -1) {
                        existingItemsMap_1[item.key] = item;
                    }
                });
                // Merge in updated items.  This may include updates to existing items or new items.  If exisiting items have the same id
                // and different priorities, the higher priority item will be shown.
                items.forEach(function (item) {
                    if (!existingItemsMap_1[item.key] || (existingItemsMap_1[item.key].priority || 0) <= (item.priority || 0)) {
                        existingItemsMap_1[item.key] = item;
                    }
                });
                var newItems_1 = [];
                Object.keys(existingItemsMap_1).forEach(function (key) {
                    newItems_1.push(existingItemsMap_1[key]);
                });
                _this._contributedItems = newItems_1;
            }
        };
        _this.state = {
            pickerIsOpen: false,
            pickerItemKey: null,
            loadingTimeoutId: _this._setLoadingTimeout(),
            displayItems: props.items.slice()
        };
        _this._updateLinkItems();
        return _this;
    }
    VssBreadcrumb.prototype.componentWillReceiveProps = function (newProps) {
        if (newProps.items !== this.props.items) {
            this._updateLinkItems();
            this.setState({ displayItems: this._arrangeItems(newProps.items.concat(this._contributedItems)) });
        }
        if (newProps.providers !== this.props.providers) {
            this._contributedItems = [];
            this.setState({ loadingTimeoutId: this._setLoadingTimeout(), displayItems: newProps.items.slice() });
            this._loadItemsFromProviders();
        }
    };
    VssBreadcrumb.prototype.componentDidMount = function () {
        this._isMounted = true;
        this._loadItemsFromProviders();
    };
    VssBreadcrumb.prototype._updateLinkItems = function () {
        this._linkItems = this.state.displayItems.filter(function (item) {
            return ((item.isHeaderPickerItem && item.itemPicker) || item.href || item.onClick);
        });
    };
    VssBreadcrumb.prototype._setLoadingTimeout = function () {
        var _this = this;
        return this._async.setTimeout(function () { _this.setState({ loadingTimeoutId: undefined }); }, PROVIDER_LOAD_TIMEOUT);
    };
    VssBreadcrumb.prototype._loadItemsFromProviders = function () {
        var _this = this;
        // If the breadcrumb items are being supplied through a callback we will ensure we
        // start resolving them and setup the loading breadcrumb items.
        if (this.props.providers && this.props.providers.length > 0) {
            // create array that will track when providers have indicated they have loaded their intial items
            var uninitializedProviders_1 = this.props.providers.map(function (x, i) { return i; });
            var _loop_1 = function (i) {
                var provider = this_1.props.providers[i];
                // Load the items from the provider
                provider.loadItems(function (items, removedItems) {
                    _this._itemsChanged(items, removedItems);
                }, function () {
                    // If loading timeout is still set, see if all providers now initialized.
                    if (_this.state.loadingTimeoutId) {
                        // Look for the provider in the array of providers still loading
                        var indexOfProvider = uninitializedProviders_1.indexOf(i);
                        if (indexOfProvider > -1) {
                            // If we found it, remove it from the array
                            uninitializedProviders_1.splice(indexOfProvider, 1);
                            // If no providers still loading, then remove the loading timeout
                            if (_this._isMounted && uninitializedProviders_1.length === 0) {
                                _this.props.onItemsLoaded();
                                _this.setState({ loadingTimeoutId: undefined, displayItems: _this._arrangeItems(_this.props.items.concat(_this._contributedItems)) });
                                _this._updateLinkItems();
                            }
                        }
                    }
                    else if (_this._isMounted) {
                        _this.props.onItemsLoaded();
                        _this.setState({ displayItems: _this._arrangeItems(_this.props.items.concat(_this._contributedItems)) });
                        _this._updateLinkItems();
                    }
                });
            };
            var this_1 = this;
            // Invoke each provider
            for (var i = 0; i < this.props.providers.length; i++) {
                _loop_1(i);
            }
        }
        else {
            // No providers so load immediately
            this.setState({ loadingTimeoutId: undefined });
        }
    };
    VssBreadcrumb.prototype.componentWillUnmount = function () {
        this._isMounted = false;
    };
    VssBreadcrumb.prototype._getIcon = function (iconProps) {
        if (iconProps) {
            return React.createElement(VssIcon, tslib_1.__assign({ className: "vss-Breadcrumb--item-leftIcon vss-HubBreadcrumb--item-leftIcon" }, iconProps));
        }
        return null;
    };
    VssBreadcrumb.prototype._renderPickerDropdown = function (item) {
        var _this = this;
        var _a = this.props, pickListClassName = _a.pickListClassName, userWidth = _a.pickListWidth, minWidth = _a.pickListMinWidth, maxWidth = _a.pickListMaxWidth, maxHeight = _a.pickListMaxHeight;
        var calculatedMinWidth = item.pickerWidth || userWidth || minWidth || (this._pinElements[item.key] && this._pinElements[item.key].clientWidth);
        var useFixedWidth = !maxWidth;
        var className = item.pickListClassName || pickListClassName;
        return (React.createElement(PickerDropdown, { ref: function (pickList) { _this.pickList = pickList; }, item: item, getPinElement: function () { return _this._pinElements[item.key]; }, width: useFixedWidth && calculatedMinWidth, minWidth: !useFixedWidth && calculatedMinWidth, maxWidth: maxWidth, maxHeight: maxHeight, pickListClassName: className, onPickListClosed: function () { return _this.setState({ pickerIsOpen: false }); }, onPickListOpened: function () { return _this.setState({ pickerIsOpen: true }); } }));
    };
    VssBreadcrumb.prototype._getBreadcrumbIcon = function (item) {
        var pickerItemKey = this.state.pickerItemKey;
        var dividerType = this.props.dividerType;
        if (dividerType === 1 /* ForwardSlash */) {
            return React.createElement("div", { className: "vss-Breadcrumb--forward-slash" }, "/");
        }
        return (React.createElement(VssIcon, { iconName: item.isHeaderPickerItem || (item.key === pickerItemKey && this.state.pickerIsOpen) ? 'ChevronDown' : 'ChevronRight', iconType: 0 /* fabric */, key: "RightIcon$" + item.key, className: css("vss-Breadcrumb--item-dropdown vss-HubBreadcrumb--item-dropdown", { "vss-Breadcrumb--item-dropdown-with-picker vss-HubBreadcrumb--item-dropdown-with-picker": item.isHeaderPickerItem }) }));
    };
    VssBreadcrumb.prototype._getDividerButtonOrIcon = function (dividerItem) {
        var pickerItemKey = this.state.pickerItemKey;
        var breadcrumbIcon = this._getBreadcrumbIcon(dividerItem);
        return !!dividerItem.itemPicker ?
            React.createElement(Link, { onClick: this._onItemClick.bind(this, dividerItem), "aria-expanded": this.state.pickerIsOpen, className: css("vss-Breadcrumb--divider vss-Breadcrumb--divider-link") },
                breadcrumbIcon,
                pickerItemKey && pickerItemKey === dividerItem.key && dividerItem.itemPicker && this._renderPickerDropdown(dividerItem))
            : breadcrumbIcon;
    };
    VssBreadcrumb.prototype._getIndicators = function (item) {
        var headerItemPicker = item.isHeaderPickerItem && item.itemPicker;
        if (headerItemPicker && headerItemPicker.selectedItem && headerItemPicker.selectedItemIndicators && headerItemPicker.selectedItemIndicators.length) {
            return React.createElement(ItemIndicatorSet, { indicators: headerItemPicker.selectedItemIndicators.map(function (indicator) { return indicator.getItemIndicator(headerItemPicker.selectedItem); }) });
        }
        else {
            return null;
        }
    };
    /**
     * Sort our breadcrumb items
     * @param items
     */
    VssBreadcrumb.prototype._arrangeItems = function (items) {
        var shouldSortMenuItems = items.some(function (x) { return x.rank >= 0; });
        var breadcrumbItems = items.slice();
        if (shouldSortMenuItems) {
            // This is not a stable sort. If we pass items with the same rank, we may not always get them back in the
            // same order. We have a stable sort in vssf webplatform, but we're not taking a dependency there.
            breadcrumbItems.sort(function (a, b) {
                var aRank = a.rank || Number.MAX_VALUE;
                var bRank = b.rank || Number.MAX_VALUE;
                return aRank - bRank;
            });
        }
        return breadcrumbItems;
    };
    VssBreadcrumb.prototype.render = function () {
        var _a = this.props, ariaLabel = _a.ariaLabel, className = _a.className, _b = _a.maxDisplayedItems, maxDisplayedItems = _b === void 0 ? DEFAULT_MAXDISPLAY_ITEMS : _b, onReduceData = _a.onReduceData, overflowIndex = _a.overflowIndex, shouldUseFocusZone = _a.shouldUseFocusZone, dataDidRender = _a.dataDidRender;
        return (React.createElement("div", { className: css("vss-Breadcrumb--container vss-HubBreadcrumb--container", className), onKeyDown: shouldUseFocusZone ? undefined : this._onKeyDown, onBlur: shouldUseFocusZone ? undefined : this._onBlur },
            React.createElement(InternalBreadcrumb, { onReduceData: onReduceData ? onReduceData : function (data) { return undefined; }, dataDidRender: dataDidRender, items: this.state.displayItems, className: css("vss-Breadcrumb vss-HubBreadcrumb"), onRenderItem: this.onRenderItem, ariaLabel: ariaLabel || VSSUIResources.BreadcrumbAriaLabel, maxDisplayedItems: maxDisplayedItems, overflowIndex: overflowIndex, dividerAs: this._getCustomDivider, shouldUseFocusZone: shouldUseFocusZone })));
    };
    VssBreadcrumb.defaultProps = {
        items: [],
        onItemsLoaded: function () { }
    };
    return VssBreadcrumb;
}(BaseComponent));
export { VssBreadcrumb };
var OVERFLOW_KEY = 'overflow';
var nullFunction = function () { return null; };
var InternalBreadcrumb = /** @class */ (function (_super) {
    tslib_1.__extends(InternalBreadcrumb, _super);
    function InternalBreadcrumb(props) {
        var _this = _super.call(this, props) || this;
        _this._focusZone = createRef();
        _this._onReduceData = function (data) {
            var renderedItems = data.renderedItems, renderedOverflowItems = data.renderedOverflowItems;
            var overflowIndex = data.props.overflowIndex;
            var movedItem = renderedItems[overflowIndex];
            renderedItems = renderedItems.slice();
            renderedItems.splice(overflowIndex, 1);
            renderedOverflowItems = renderedOverflowItems.concat([movedItem]);
            if (movedItem !== undefined) {
                return tslib_1.__assign({}, data, { renderedItems: renderedItems, renderedOverflowItems: renderedOverflowItems });
            }
        };
        _this._onRenderBreadcrumb = function (data) {
            var _a = data.props, ariaLabel = _a.ariaLabel, _b = _a.dividerAs, DividerType = _b === void 0 ? Icon : _b, _c = _a.onRenderItem, onRenderItem = _c === void 0 ? _this._onRenderItem : _c, overflowAriaLabel = _a.overflowAriaLabel, overflowIndex = _a.overflowIndex, className = _a.className, shouldUseFocusZone = _a.shouldUseFocusZone;
            var renderedOverflowItems = data.renderedOverflowItems, renderedItems = data.renderedItems;
            var contextualItems = renderedOverflowItems.map(function (item, index) { return ({
                name: item.text,
                key: item.key,
                onClick: item.onClick ? _this._onBreadcrumbClicked.bind(_this, item) : null,
                href: item.href
            }); });
            // Find index of last rendered item so the divider icon
            // knows not to render on that item
            var lastItemIndex = renderedItems.length - 1;
            var hasOverflowItems = renderedOverflowItems && renderedOverflowItems.length !== 0;
            var itemElements = renderedItems.map(function (item, index) { return (React.createElement("li", { className: "internal-breadcrumb--listItem", key: item.key || String(index) },
                onRenderItem(item, _this._onRenderItem),
                (index !== lastItemIndex || (hasOverflowItems && index === (overflowIndex - 1))) && (React.createElement(DividerType, { className: "internal-breadcrumb--chevron", iconName: getRTL() ? 'ChevronLeft' : 'ChevronRight', item: item })))); });
            if (hasOverflowItems) {
                itemElements.splice(overflowIndex, 0, (React.createElement("li", { className: "internal-breadcrumb--overflow", key: OVERFLOW_KEY },
                    React.createElement(IconButton, { className: "internal-breadcrumb--overflowButton", iconProps: { iconName: 'More' }, role: 'button', "aria-haspopup": 'true', ariaLabel: overflowAriaLabel, onRenderMenuIcon: nullFunction, menuProps: {
                            items: contextualItems,
                            directionalHint: 4 /* bottomLeftEdge */
                        } }),
                    overflowIndex !== lastItemIndex + 1 && (React.createElement(DividerType, { className: "internal-breadcrumb--chevron", iconName: getRTL() ? 'ChevronLeft' : 'ChevronRight', item: renderedOverflowItems[renderedOverflowItems.length - 1] })))));
            }
            var listComponent = (React.createElement("ol", { className: "internal-breadcrumb--list" }, itemElements));
            if (shouldUseFocusZone) {
                listComponent = (React.createElement(FocusZone, { componentRef: _this._focusZone, direction: FocusZoneDirection.horizontal }, listComponent));
            }
            return (React.createElement("div", { className: className, role: 'navigation', "aria-label": ariaLabel }, listComponent));
        };
        _this._onRenderItem = function (item) {
            return (React.createElement("span", { className: "internal-breadcrumb--item" },
                React.createElement(TooltipHost, { content: item.text, overflowMode: TooltipOverflowMode.Parent }, item.text)));
        };
        _this._onBreadcrumbClicked = function (item, ev) {
            if (item.onClick) {
                item.onClick(ev, item);
            }
        };
        _this._validateProps(props);
        return _this;
    }
    InternalBreadcrumb.prototype.render = function () {
        var _a = this.props, _b = _a.onReduceData, onReduceData = _b === void 0 ? this._onReduceData : _b, dataDidRender = _a.dataDidRender, overflowIndex = _a.overflowIndex, maxDisplayedItems = _a.maxDisplayedItems, items = _a.items, className = _a.className;
        var renderedItems = items.slice();
        var renderedOverflowItems = renderedItems.splice(overflowIndex, renderedItems.length - maxDisplayedItems);
        var breadCrumbData = {
            props: this.props,
            renderedItems: renderedItems,
            renderedOverflowItems: renderedOverflowItems
        };
        return (React.createElement(ResizeGroup, { onRenderData: this._onRenderBreadcrumb, onReduceData: onReduceData, dataDidRender: dataDidRender, data: breadCrumbData, className: "vss-Breadcrumb--resizeGroup" }));
    };
    InternalBreadcrumb.prototype.componentWillReceiveProps = function (nextProps) {
        this._validateProps(nextProps);
    };
    /**
     * Validate incoming props
     * @param props Props to validate
     */
    InternalBreadcrumb.prototype._validateProps = function (props) {
        var maxDisplayedItems = props.maxDisplayedItems, overflowIndex = props.overflowIndex, items = props.items;
        if (overflowIndex < 0 ||
            maxDisplayedItems > 1 && overflowIndex > maxDisplayedItems - 1 ||
            items.length > 0 && overflowIndex > items.length - 1) {
            throw new Error('Breadcrumb: overflowIndex out of range');
        }
    };
    InternalBreadcrumb.defaultProps = {
        items: [],
        maxDisplayedItems: 999,
        overflowIndex: 0
    };
    return InternalBreadcrumb;
}(BaseComponent));
