import * as tslib_1 from "tslib";
import "../../FabricInit";
import * as React from "react";
import * as ReactDOM from "react-dom";
import { ContextualMenu, ContextualMenuItemType } from "office-ui-fabric-react/lib/ContextualMenu";
import { css, BaseComponent, elementContains } from 'office-ui-fabric-react/lib/Utilities';
import "./VssContextualMenu.css";
/**
 * Creates office fabric contextual menu, also takes contribution ids to get menu contributions from extensions
 * P.S: ContextualMenu generates some warnings look: https://github.com/OfficeDev/office-ui-fabric-react/issues/357
 */
var VssContextualMenu = /** @class */ (function (_super) {
    tslib_1.__extends(VssContextualMenu, _super);
    function VssContextualMenu(props) {
        var _this = _super.call(this, props) || this;
        _this._invalidateItem = function (key) {
            var foundMatch = false;
            // If there is a sub menu, first look for match in the sub menu
            if (_this.subMenuId) {
                var matches = _this._findMenuItems(_this.subMenuId);
                var itemIndex = _this._findItemIndex(matches, key);
                foundMatch = _this._focusItem(matches, itemIndex - 1);
            }
            // If we did not focus anything in sub menu, try this menu
            if (!foundMatch) {
                var matches = _this._findMenuItems(_this.props.id);
                var itemIndex = _this._findItemIndex(matches, key);
                foundMatch = _this._focusItem(matches, itemIndex - 1);
                // check to see if the first match is not the item being removed.  If it is not then focus it.
                if (!foundMatch) {
                    if (itemIndex > 0) {
                        foundMatch = _this._focusItem(matches, 0);
                    }
                    else {
                        _this._setTimeout(function () {
                            var matches = _this._findMenuItems(_this.props.id);
                            _this._focusItem(matches, 0);
                        }, 0);
                    }
                }
            }
            // If still no matches, then try and focus the previously active element
            if (!foundMatch && _this._activeElement && _this.contains(document.activeElement)) {
                _this._activeElement.focus();
            }
        };
        _this._itemsChanged = function (items, removedItems, groupInfo) {
            if (_this._isMounted) {
                // Build map of what is currently in the list of menu items
                var existingItemsMap_1 = {};
                _this._items.forEach(function (item) {
                    // if an item is being removed, then do not add to the existing map.
                    if (!removedItems || removedItems.indexOf(item.key) === -1) {
                        existingItemsMap_1[item.key] = item;
                    }
                });
                // Merge in updated items.  This may include updates to existing items or new items
                items.forEach(function (item) {
                    item.invalidateItem = function () {
                        _this._invalidateItem(item.key);
                    };
                    existingItemsMap_1[item.key] = item;
                });
                var newItems_1 = [];
                Object.keys(existingItemsMap_1).forEach(function (key) {
                    newItems_1.push(existingItemsMap_1[key]);
                });
                _this._items = newItems_1;
                _this.setState({ displayItems: _this._arrangeItems(newItems_1, groupInfo) });
            }
        };
        _this._onDismiss = function (ev, dismissAll) {
            // If focus remains in the menu when it is dismissed we will set focus back to the
            // previously focused element.
            if (_this._activeElement && _this.contains(document.activeElement)) {
                _this._activeElement.focus();
            }
            _this.props.onDismiss(ev, dismissAll);
        };
        _this._onRenderSubMenu = function (props) {
            _this.subMenuId = props.id;
            if (props.items) {
                props.items.forEach(function (item) {
                    item.invalidateItem = function () {
                        _this._invalidateItem(item.key);
                    };
                });
            }
            return React.createElement(VssContextualMenu, tslib_1.__assign({ ref: function (submenuComponent) { _this._submenuComponent = submenuComponent; } }, props));
        };
        _this.timers = [];
        _this._items = [];
        _this._groups = [];
        _this.state = {
            displayItems: [],
        };
        return _this;
    }
    VssContextualMenu.prototype.render = function () {
        if (this.state.loadingTimeout) {
            return null;
        }
        var displayItems = this.state.displayItems;
        var _a = this.props, className = _a.className, directionalHint = _a.directionalHint, _b = _a.gapSpace, gapSpace = _b === void 0 ? 5 : _b;
        var hasIcons = !!(this._items && this._items.some(function (item) { return !!item.icon || !!item.iconProps; }));
        this._items.forEach(function (menuItem) {
            menuItem.menuHasIcons = hasIcons;
            if (hasIcons && !menuItem.iconProps && !menuItem.icon) {
                menuItem.iconProps = { iconName: '' };
            }
        });
        return React.createElement(ContextualMenu, tslib_1.__assign({ gapSpace: gapSpace, directionalHint: directionalHint }, this.props, { items: displayItems, className: css("vss-contextualMenu-list", className), onDismiss: this._onDismiss, onRenderSubMenu: this._onRenderSubMenu }));
    };
    VssContextualMenu.prototype.componentWillMount = function () {
        var _this = this;
        var loadingTimeout = this._setTimeout(function () {
            _this.setState({ loadingTimeout: undefined });
        }, 100);
        this._items = this.props.items || [];
        this._groups = this.props.groups || [];
        this.setState({
            displayItems: this.props.items ? this._arrangeItems(this.props.items) : [],
            loadingTimeout: loadingTimeout,
        });
        // Ensure we only have a single root menu open, ignore global processing for submenus.
        if (!this.props.isSubMenu) {
            if (VssContextualMenu._activeMenu) {
                VssContextualMenu._activeMenu.dismiss(null, true);
            }
            VssContextualMenu._activeMenu = this;
        }
        // Save the active element as we are bringing up the menu.
        this._activeElement = document.activeElement;
    };
    VssContextualMenu.prototype.componentWillUnmount = function () {
        this._isMounted = false;
        _super.prototype.componentWillUnmount.call(this);
        if (this.state.loadingTimeout) {
            this._clearTimeout(this.state.loadingTimeout);
        }
        if (this.timers) {
            for (var _i = 0, _a = this.timers; _i < _a.length; _i++) {
                var timer = _a[_i];
                window.clearTimeout(timer);
            }
            this.timers = [];
        }
        // If we are the active root menu we can clear our state on closure.
        if (VssContextualMenu._activeMenu === this) {
            delete VssContextualMenu._activeMenu;
        }
    };
    VssContextualMenu.prototype.componentDidMount = function () {
        var _this = this;
        this._isMounted = true;
        // Ensure we have resolved the target to an HTMLElement
        if (typeof this.props.target === 'string') {
            this._targetElement = document.querySelector(this.props.target);
        }
        else {
            this._targetElement = this.props.target;
        }
        // If the menu items are being supplied through a callback we will ensure we have
        // start resolving them and setup the loading menu items.
        if (this.props.providers && this.props.providers.length > 0) {
            // create array that will track when providers have indicated they have loaded their intial items
            var uninitializedProviders_1 = this.props.providers.map(function (x, i) { return i; });
            var _loop_1 = function (i) {
                var provider = this_1.props.providers[i];
                // Load the items from the provider
                provider.loadItems(function (items, removedItems, groupInfo) {
                    _this._itemsChanged(items, removedItems, groupInfo);
                }, function () {
                    // If loading timeout is still set, see if all providers now initialized.
                    if (_this.state.loadingTimeout) {
                        // Look for the provider in the array of providers still loading
                        var indexOfProvider = uninitializedProviders_1.indexOf(i);
                        if (indexOfProvider > -1) {
                            // If we found it, remove it from the array
                            uninitializedProviders_1.splice(indexOfProvider, 1);
                            // If no providers still loading, then remove the loading timeout
                            if (_this._isMounted && uninitializedProviders_1.length === 0) {
                                _this.setState({ loadingTimeout: undefined });
                            }
                        }
                    }
                });
            };
            var this_1 = this;
            // Invoke each provider
            for (var i = 0; i < this.props.providers.length; i++) {
                _loop_1(i);
            }
        }
        else {
            // No providers so load immediately
            this.setState({ loadingTimeout: undefined });
        }
        if (this.props.mouseLeaveDelayMs) {
            // Track the mousemove on the document to close if the mouse is outside the menu for a period of time.
            this._events.on(document.body, "mousemove", function (event) {
                if (_this._targetElement.contains(event.target) || _this.contains(event.target)) {
                    if (_this.closeTimeout) {
                        _this._clearTimeout(_this.closeTimeout);
                        delete _this.closeTimeout;
                    }
                }
                else {
                    if (!_this.closeTimeout) {
                        _this.closeTimeout = _this._setTimeout(function () {
                            delete _this.closeTimeout;
                            _this.dismiss(event, true);
                        }, _this.props.mouseLeaveDelayMs || 0);
                    }
                }
            });
        }
        this._events.on(document, "keydown", function (event) {
            // Invert the display of group names for the opened menus when the ` key pressed.
            if (event.key === "`" && !event.defaultPrevented) {
                _this._forceGroupNames = !_this._forceGroupNames;
                _this.setState({ displayItems: _this._arrangeItems(_this._items) });
            }
            // If a keystroke was made and we are not focused we need to close the menu. 
            // We need to look at shouldFocusOnMount, since it may be an empty menu and we dont want it closed.
            // Don't close the menu for control keys (shift, alt, ctrl)
            else if (event.which !== 16 /* KeyCodes.Shift */
                && event.which !== 17 /* KeyCodes.Ctrl */
                && event.which !== 18 /* KeyCodes.Alt */
                && !_this.props.isSubMenu
                && !_this.props.shouldFocusOnMount
                && !_this.contains(document.activeElement)) {
                _this.dismiss(event, true);
            }
        });
    };
    VssContextualMenu.prototype.componentWillReceiveProps = function (nextProps) {
        if (nextProps.isSubMenu) {
            var oldItems = this.props.items || [];
            var newItems_2 = nextProps.items || [];
            var itemsChanged = oldItems.length !== newItems_2.length;
            if (!itemsChanged) {
                // Check if same items exist
                itemsChanged = oldItems.some(function (value) {
                    return !newItems_2.some(function (nextItem) { return nextItem.key === value.key; });
                });
            }
            if (itemsChanged) {
                this.setState({
                    displayItems: newItems_2 ? this._arrangeItems(newItems_2) : []
                });
            }
        }
    };
    VssContextualMenu.prototype.contains = function (element) {
        return this._isMounted && elementContains(ReactDOM.findDOMNode(this), element) || (this._submenuComponent && this._submenuComponent.contains(element));
    };
    VssContextualMenu.prototype.dismiss = function (reason, dismissAll) {
        if (this.props.onDismiss) {
            this.props.onDismiss(reason, true);
        }
    };
    /**
     * Sort and group our menu items
     * @param items
     */
    VssContextualMenu.prototype._arrangeItems = function (items, groupInfo) {
        var shouldSortMenuItems = items.some(function (x) { return x.rank >= 0; });
        var shouldGroupMenuItems = items.some(function (x) { return !!x.groupKey; });
        items = items.slice();
        if (shouldSortMenuItems) {
            this._sortItems(items);
        }
        if (shouldGroupMenuItems) {
            items = this._groupItems(items, groupInfo);
        }
        this._removeConsecutiveSeparators(items);
        return items;
    };
    VssContextualMenu.prototype._clearTimeout = function (timeoutId) {
        if (this.timers) {
            var timerIndex = this.timers.indexOf(timeoutId);
            this.timers.splice(timerIndex, 1);
        }
        window.clearTimeout(timeoutId);
    };
    VssContextualMenu.prototype._findMenuItems = function (menuId) {
        var menuElement = document.querySelector("#" + menuId);
        if (menuElement) {
            return menuElement.querySelectorAll("[role='menuitem']");
        }
    };
    VssContextualMenu.prototype._findItemIndex = function (matches, key) {
        var matchedIndex = -1;
        if (matches && matches.length > 1) {
            // Loop through the tabbable items and try and find item which has same key that was just clicked
            for (var i = 0; i < matches.length; i++) {
                var item = matches.item(i);
                if (item.parentElement && item.parentElement.id === key) {
                    matchedIndex = i;
                    break;
                }
            }
        }
        return matchedIndex;
    };
    VssContextualMenu.prototype._focusItem = function (matches, itemIndex) {
        var itemFocused = false;
        if (matches && matches.length > itemIndex && itemIndex > -1) {
            var item = matches.item(itemIndex);
            item.focus();
            itemFocused = true;
        }
        return itemFocused;
    };
    /**
     * Arrange the items into groups and put separators between them and headings above them as needed.
     * @param items
     */
    VssContextualMenu.prototype._groupItems = function (items, groupInfo) {
        var groupMap = {};
        var maxGroupRank = 0;
        var ungroupedItems = [];
        var forceGroupNames = this._forceGroupNames;
        // gather known groups
        if (this._groups) {
            maxGroupRank = this._groups.reduce(function (max, g) { return g.rank > max ? g.rank : max; }, 0);
            for (var _i = 0, _a = this._groups; _i < _a.length; _i++) {
                var g = _a[_i];
                groupMap[g.key] = {
                    key: g.key,
                    name: g.name,
                    rank: g.rank === undefined ? ++maxGroupRank : g.rank,
                    showGroupName: g.showGroupName,
                    items: [],
                };
            }
        }
        else {
            this._groups = [];
        }
        if (groupInfo) {
            for (var _b = 0, groupInfo_1 = groupInfo; _b < groupInfo_1.length; _b++) {
                var g = groupInfo_1[_b];
                var existingGroup = groupMap[g.key];
                if (!existingGroup) {
                    groupMap[g.key] = {
                        key: g.key,
                        name: g.name,
                        rank: g.rank === undefined ? ++maxGroupRank : g.rank,
                        showGroupName: g.showGroupName,
                        items: [],
                    };
                    this._groups.push(g);
                }
            }
        }
        // put all the items in groups
        for (var _c = 0, items_1 = items; _c < items_1.length; _c++) {
            var i = items_1[_c];
            if (i.groupKey) {
                if (groupMap[i.groupKey]) {
                    groupMap[i.groupKey].items.push(i);
                }
                else {
                    groupMap[i.groupKey] = {
                        key: i.groupKey,
                        name: i.groupKey,
                        rank: ++maxGroupRank,
                        items: [i],
                    };
                }
            }
            else {
                ungroupedItems.push(i);
            }
        }
        // sort the groups
        var groupList = Object.keys(groupMap).map(function (n) { return groupMap[n]; });
        groupList.sort(function (a, b) { return (a.rank || Number.MAX_VALUE) - (b.rank || Number.MAX_VALUE); });
        groupList.push({
            key: "ungrouped",
            name: "Ungrouped",
            rank: ++maxGroupRank,
            items: ungroupedItems,
        });
        // remove dividers from the beginning and end of each group
        groupList.forEach(function (g) {
            var array = g.items;
            while (array.length > 0 && array[0].itemType === ContextualMenuItemType.Divider) {
                array.shift();
            }
            while (array.length > 0 && array[array.length - 1].itemType === ContextualMenuItemType.Divider) {
                array.pop();
            }
        });
        // merge the groups into the final array
        items = [];
        var first = true;
        for (var _d = 0, groupList_1 = groupList; _d < groupList_1.length; _d++) {
            var g = groupList_1[_d];
            if (g.items.length === 0 && !forceGroupNames) {
                continue;
            }
            // add the separator or header for the top of the group
            if ((g.showGroupName && g.name) || forceGroupNames) {
                items.push({
                    key: "group_" + g.name,
                    itemType: ContextualMenuItemType.Header,
                    name: forceGroupNames ? g.name + " (key: " + g.key + ")" : g.name,
                });
            }
            else if (!first) {
                // don't add a separator when there's a header, because header menu items take care
                // of the separator for us
                items.push({
                    key: "divider_" + g.name,
                    itemType: ContextualMenuItemType.Divider,
                });
            }
            if (first) {
                first = false;
            }
            items = items.concat(g.items);
        }
        return items;
    };
    VssContextualMenu.prototype._removeConsecutiveSeparators = function (items) {
        var lastWasSeparator = true;
        for (var i = items.length - 1; i >= 0; i--) {
            var currentIsSeparator = items[i].itemType === ContextualMenuItemType.Divider || items[i].name === "-";
            if (currentIsSeparator && lastWasSeparator) {
                items.splice(i, 1);
            }
            lastWasSeparator = currentIsSeparator;
        }
    };
    VssContextualMenu.prototype._setTimeout = function (callback, milliseconds) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        var timerId = window.setTimeout.apply(window, [callback, milliseconds].concat(args));
        if (!this.timers) {
            this.timers = [];
        }
        this.timers.push(timerId);
        return timerId;
    };
    /**
     * Sort the items according to their rank property.
     * @param items
     */
    VssContextualMenu.prototype._sortItems = function (items) {
        // This is not a stable sort. If we pass items with the same rank, we may not always get them back in the
        // same order. We have a stable sort in vssf webplatform, but we're not taking a dependency there.
        items.sort(function (a, b) {
            var aRank = a.rank || Number.MAX_VALUE;
            var bRank = b.rank || Number.MAX_VALUE;
            return aRank - bRank;
        });
    };
    return VssContextualMenu;
}(BaseComponent));
export { VssContextualMenu };
