import * as tslib_1 from "tslib";
import "../../FabricInit";
import * as React from 'react';
import { CheckboxVisibility, DetailsList, Selection, SelectionMode } from 'office-ui-fabric-react/lib/DetailsList';
import { BaseComponent, css } from 'office-ui-fabric-react/lib/Utilities';
import { MoreActionsButton } from '../../ContextualMenuButton';
import { ContextualMenuButton } from '../../Components/ContextualMenuButton/ContextualMenuButton';
import { VssContextualMenu } from '../../VssContextualMenu';
import { closest } from '../../Utilities/Internal';
import "./VssDetailsList.css";
/**
 * Wraps Office Fabric Details list with state management specific to Vss Components
 */
var VssDetailsList = /** @class */ (function (_super) {
    tslib_1.__extends(VssDetailsList, _super);
    function VssDetailsList(props) {
        var _this = _super.call(this, props) || this;
        _this.standardPresentationStyle = 16 /* colors */ |
            4 /* flexCellLayout */ |
            64 /* headerRowStyles */ |
            1 /* hideActionsButton */ |
            32 /* linkStyles */ |
            8 /* paddingInCells */ |
            2 /* singleLine */;
        _this._onItemContextMenu = function (item, index, ev) {
            var mouseEvent = ev;
            if (mouseEvent) {
                // We don't want to show the custom contextual menu or prevent default
                // if the target element is an anchor tag.
                var target_1 = mouseEvent.target;
                if (target_1 && (target_1.nodeName === 'A' || closest(target_1, 'A'))) {
                    return true;
                }
            }
            if (_this.props.shouldDisplayActions && !_this.props.shouldDisplayActions(item)) {
                return true;
            }
            if (!mouseEvent) {
                return _this.setState({ activatedItem: item, target: undefined });
            }
            var target = null;
            // For chrome/firefox mouseEvent.button === 0 can be used to assume keyboard interaction here, whereas 
            // in IE the mouseEvent.button will be 2, but the clientX or clientY has a negative value.
            // So for IE and Edge, an HTMLElement needs to be passed in for anchoring otherwise the menu will be at (0, 0).
            if (mouseEvent.button === 0 || !(mouseEvent.clientX > 0 && mouseEvent.clientY > 0)) {
                var targetFromEvent = mouseEvent.target;
                var row = closest(targetFromEvent, '[role="row"]');
                target = row ? row.querySelector("." + ContextualMenuButton.ButtonContainerClassName) : null;
            }
            _this.setState({ activatedItem: item, target: target || mouseEvent });
        };
        _this._onDismiss = function () {
            _this.setState({ activatedItem: undefined, target: undefined });
        };
        _this._onEllipsisClick = function (e) {
            // Important: to handle the toggle modifiers for multi-select scenario, we must update the selection here prior to calling getMenuItems, otherwise
            // getMenuItems may be looking an outdated selection object and returning items not in context of the updated selections. Unfortunately, even if the click
            // event is allowed to propagate down to DetailsList for automatic selection handling, getMenuItems for the ActionsButton would have been called already
            // (that's why selection needs to be updated here and also by design the ActionsButton should ignore toggle modifier if row is already selected).
            var index = Number(e.currentTarget.dataset.index);
            var isMultipleSelect = _this.props.selectionMode === SelectionMode.multiple;
            var selection = _this.state.selection;
            selection.setChangeEvents(false);
            var isToggleModifierPressed = e.ctrlKey || e.metaKey;
            if (isMultipleSelect && e.shiftKey) {
                var clearExistingSelection = !isToggleModifierPressed;
                selection.selectToIndex(index, clearExistingSelection);
            }
            else if (isMultipleSelect) {
                var isNewSelection = !selection.isIndexSelected(index);
                if (isNewSelection && !isToggleModifierPressed) {
                    selection.setAllSelected(false);
                }
                selection.setIndexSelected(index, true, true);
            }
            else {
                selection.setAllSelected(false);
                selection.setIndexSelected(index, true, true);
            }
            selection.setChangeEvents(true);
        };
        _this.state = {};
        return _this;
    }
    VssDetailsList.prototype.componentWillMount = function () {
        var classNames = this._buildClassNames(this.props);
        var columns = this._buildColumns(this.props);
        var selection = this._buildSelection(this.props);
        this.overrideSelectedIndex = false;
        this.setState({ classNames: classNames, columns: columns, selection: selection });
    };
    VssDetailsList.prototype.componentDidMount = function () {
        this.overrideSelectedIndex = true;
    };
    VssDetailsList.prototype.componentWillReceiveProps = function (nextProps) {
        var newState = {};
        if (this.props.className !== nextProps.className ||
            this.props.presentationStyles !== nextProps.presentationStyles ||
            this.props.usePresentationStyles !== nextProps.usePresentationStyles ||
            this.props.rowStyle !== nextProps.rowStyle) {
            newState.classNames = this._buildClassNames(nextProps);
        }
        if (this.props.columns !== nextProps.columns || this.props.actionsColumnKey !== nextProps.actionsColumnKey) {
            newState.columns = this._buildColumns(nextProps);
        }
        if (this.props.selection !== nextProps.selection) {
            newState.selection = this._buildSelection(nextProps);
        }
        this.setState(newState);
    };
    VssDetailsList.prototype.render = function () {
        var _a = this.props.checkboxVisibility, checkboxVisibility = _a === void 0 ? CheckboxVisibility.hidden : _a;
        var _b = this.state, classNames = _b.classNames, columns = _b.columns;
        var props = tslib_1.__assign({}, this.props);
        var hasActions = this._hasActions(this.props);
        props.className = classNames;
        props.checkboxVisibility = checkboxVisibility;
        props.checkboxCellClassName = 'vss-DetailsList--cellCheck';
        props.columns = columns;
        if (hasActions) {
            props.onItemContextMenu = this._onItemContextMenu;
        }
        var detailsList = React.createElement(DetailsList, tslib_1.__assign({}, props, { initialFocusedIndex: this.overrideSelectedIndex ? undefined : props.initialFocusedIndex, selection: this.state.selection, componentRef: this._resolveRef("detailsList") }));
        if (hasActions) {
            return React.createElement("div", { className: 'vss-DetailsList--contextmenu-wrapper' },
                detailsList,
                this._getContextMenu());
        }
        return detailsList;
    };
    VssDetailsList.prototype._createActionsColumn = function (column) {
        var _this = this;
        var wrappedColumn = tslib_1.__assign({}, column);
        wrappedColumn.onRender = function (item, index, c) {
            var originalContent = null;
            if (column.onRender) {
                originalContent = column.onRender(item, index, c);
            }
            else if (_this.props.onRenderItemColumn) {
                originalContent = _this.props.onRenderItemColumn(item, index, c);
            }
            else {
                // What should we do if they provided a column that doesn't have a
                // custom onRender method? The same thing as the base row will do?
            }
            var shouldDisplayActions = true;
            if (_this.props.shouldDisplayActions) {
                shouldDisplayActions = _this.props.shouldDisplayActions(item);
            }
            if (shouldDisplayActions) {
                return React.createElement("div", { className: 'actions-column-wrapper' },
                    React.createElement("div", { className: 'original-content' }, originalContent),
                    React.createElement(MoreActionsButton, { allocateSpaceWhileHidden: _this.props.allocateSpaceForActionsButtonWhileHidden, className: 'vss-DetailsList--contextual-menu-button', getItems: _this.props.getMenuItems && (function () { return _this.props.getMenuItems(item); }), getItemProviders: _this.props.getMenuItemProviders && (function () { return _this.props.getMenuItemProviders(item); }), onDismiss: _this._onDismiss, "data-index": index, onClick: _this._onEllipsisClick }));
            }
            else {
                return originalContent;
            }
        };
        return wrappedColumn;
    };
    VssDetailsList.prototype._hasActions = function (props) {
        return !!(props.actionsColumnKey && (props.getMenuItems || props.getMenuItemProviders));
    };
    VssDetailsList.prototype._buildSelection = function (props) {
        return props.selection || new Selection({ getKey: props.getKey });
    };
    VssDetailsList.prototype._buildColumns = function (props) {
        var actionsColumnKey = props.actionsColumnKey, columns = props.columns;
        if (this._hasActions(props) && columns) {
            var modifiedColumns = [];
            for (var _i = 0, columns_1 = columns; _i < columns_1.length; _i++) {
                var column = columns_1[_i];
                if (column.key === actionsColumnKey) {
                    column = this._createActionsColumn(column);
                }
                modifiedColumns.push(column);
            }
            return modifiedColumns;
        }
        return columns;
    };
    VssDetailsList.prototype._buildClassNames = function (props) {
        var usePresentationStyles = props.usePresentationStyles, className = props.className, rowStyle = props.rowStyle, hideGroupExpansion = props.hideGroupExpansion;
        var presentationStyles = props.presentationStyles;
        var classNames = "";
        if (usePresentationStyles || presentationStyles) {
            if (presentationStyles === undefined) {
                presentationStyles = this.standardPresentationStyle;
            }
            var classNamesList = ['vss-DetailsList--layout'];
            if (className) {
                classNamesList.push(className);
            }
            if (presentationStyles & 1 /* hideActionsButton */) {
                classNamesList.push('hideActionsButton');
            }
            if (presentationStyles & 2 /* singleLine */) {
                classNamesList.push('singleLine');
            }
            if (presentationStyles & 4 /* flexCellLayout */) {
                classNamesList.push('flexCellLayout');
            }
            if (presentationStyles & 8 /* paddingInCells */) {
                classNamesList.push('paddingInCells');
            }
            if (presentationStyles & 16 /* colors */) {
                classNamesList.push('colors');
            }
            if (presentationStyles & 32 /* linkStyles */) {
                classNamesList.push('linkStyles');
            }
            if (presentationStyles & 64 /* headerRowStyles */) {
                classNamesList.push('headerRowStyles');
            }
            classNames = classNamesList.join(" ");
        }
        else {
            classNames = css('vss-DetailsList', className, { 'vss-DetailsList--twoLine': rowStyle === 1 /* twoLine */ });
        }
        classNames = css(classNames, {
            'noGroupExpand': hideGroupExpansion
        });
        return classNames;
    };
    VssDetailsList.prototype._getContextMenu = function () {
        var _a = this.state, activatedItem = _a.activatedItem, target = _a.target;
        if (!activatedItem) {
            return undefined;
        }
        var _b = this.props, getMenuItems = _b.getMenuItems, getMenuItemProviders = _b.getMenuItemProviders;
        var items = getMenuItems ? getMenuItems(activatedItem) : undefined;
        var itemProviders = getMenuItemProviders ? getMenuItemProviders(activatedItem) : undefined;
        if (!items && !itemProviders) {
            return undefined;
        }
        return React.createElement(VssContextualMenu, { items: items, providers: itemProviders, onDismiss: this._onDismiss, shouldFocusOnMount: true, target: target, gapSpace: 0 });
    };
    return VssDetailsList;
}(BaseComponent));
export { VssDetailsList };
