import * as tslib_1 from "tslib";
import "../../FabricInit";
import * as React from 'react';
import { Link } from 'office-ui-fabric-react/lib/Link';
import { TooltipHost, TooltipOverflowMode } from '../../Tooltip';
import { BaseComponent, css } from 'office-ui-fabric-react/lib/Utilities';
import { ItemIndicatorSet } from '../../ItemIndicator';
import { VssIcon } from '../../VssIcon';
/**
 * Helper for the styling / functionality of common title cells in VssDetailsList
 */
var VssDetailsListTitleCell = /** @class */ (function (_super) {
    tslib_1.__extends(VssDetailsListTitleCell, _super);
    function VssDetailsListTitleCell() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    VssDetailsListTitleCell.prototype.render = function () {
        var _a = this.props, secondaryText = _a.secondaryText, onRenderSecondaryText = _a.onRenderSecondaryText, ariaLabel = _a.ariaLabel, iconProps = _a.iconProps;
        if (secondaryText || onRenderSecondaryText) {
            return React.createElement("div", { className: css('vss-DetailsList--titleCellTwoLine', !iconProps && 'noIcon'), "aria-label": ariaLabel },
                this._renderIcon(),
                React.createElement("div", { className: 'vss-DetailsList--titleCellTwoLineText' },
                    this._renderPrimaryText(),
                    this._renderSecondaryText()),
                this._renderIndicators());
        }
        else {
            return React.createElement("div", { className: 'vss-DetailsList--titleCellOneLine', "aria-label": ariaLabel },
                this._renderIcon(),
                this._renderPrimaryText(),
                this._renderIndicators());
        }
    };
    VssDetailsListTitleCell.prototype._renderPrimaryText = function () {
        var _a = this.props, primaryText = _a.primaryText, primaryAction = _a.primaryAction, primaryTarget = _a.primaryTarget, disabled = _a.disabled, onRenderPrimaryText = _a.onRenderPrimaryText;
        var primaryActionRole = this.props.primaryActionRole;
        var textElement = onRenderPrimaryText ?
            onRenderPrimaryText() :
            (primaryAction ? (typeof primaryAction === "function") ? React.createElement(Link, { className: css('vss-DetailsList--titleCellText', { 'disabled': disabled }), role: primaryActionRole || 'button', href: "#", onClick: primaryAction }, primaryText) :
                React.createElement(Link, { className: css('vss-DetailsList--titleCellText', { 'disabled': disabled }), role: primaryActionRole || 'link', href: primaryAction, target: primaryTarget }, primaryText) :
                React.createElement("span", { className: css('vss-DetailsList--titleCellText', { 'disabled': disabled }) }, primaryText));
        return React.createElement("div", { className: 'vss-DetailsList--titleCellPrimary' }, textElement);
    };
    VssDetailsListTitleCell.prototype._renderSecondaryText = function () {
        var _a = this.props, secondaryText = _a.secondaryText, disabled = _a.disabled, onRenderSecondaryText = _a.onRenderSecondaryText;
        var textElement = onRenderSecondaryText ?
            onRenderSecondaryText() :
            React.createElement(TooltipHost, { content: secondaryText, overflowMode: TooltipOverflowMode.Parent },
                React.createElement("span", { className: css('vss-DetailsList--titleCellText', { 'disabled': disabled }) }, secondaryText));
        if (textElement) {
            return React.createElement("div", { className: 'vss-DetailsList--titleCellSecondary' }, textElement);
        }
        return null;
    };
    VssDetailsListTitleCell.prototype._renderIcon = function () {
        var _a = this.props, iconProps = _a.iconProps, disabled = _a.disabled;
        if (iconProps) {
            return React.createElement(VssIcon, tslib_1.__assign({}, iconProps, { className: css('vss-DetailsList--titleCellIcon', iconProps.className, { 'disabled': disabled, 'vss-DetailsList--titleCellIcon--image': iconProps.iconType === 3 /* image */ }) }));
        }
        return null;
    };
    VssDetailsListTitleCell.prototype._renderIndicators = function () {
        var indicators = this.props.indicators ? this.props.indicators.map(function (indicator) {
            return indicator.getItemIndicator();
        }) : [];
        return indicators && indicators.length > 0 ?
            React.createElement(ItemIndicatorSet, { indicators: indicators })
            : null;
    };
    return VssDetailsListTitleCell;
}(BaseComponent));
export { VssDetailsListTitleCell };
