import * as tslib_1 from "tslib";
import "../../FabricInit";
import * as React from "react";
import { PrimaryButton } from "office-ui-fabric-react/lib/Button";
import { BaseComponent, css } from "office-ui-fabric-react/lib/Utilities";
import { Link } from "office-ui-fabric-react/lib/Link";
import "./ZeroData.css";
/**
 * Component for displaying helpful information when there is no data to show.
 */
var ZeroData = /** @class */ (function (_super) {
    tslib_1.__extends(ZeroData, _super);
    function ZeroData() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ZeroData.prototype.render = function () {
        return React.createElement(ZeroDataMultiple, { items: [this.props], className: this.props.className });
    };
    return ZeroData;
}(BaseComponent));
export { ZeroData };
/**
 * Component for displaying helpful information when there is no data to show. This one displays
 * multiple (or one) ZeroDataItems.
 *
 * THIS CLASS IS NOT EXPORTED presently because no design for multiple ZeroDataItems has yet been
 * approved.
 */
var ZeroDataMultiple = /** @class */ (function (_super) {
    tslib_1.__extends(ZeroDataMultiple, _super);
    function ZeroDataMultiple() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ZeroDataMultiple.prototype.render = function () {
        var multiple = this.props.items.length > 1;
        return (React.createElement("div", { className: css("vss-ZeroData flex-row justify-center", multiple ? "multiple" : "single", this.props.className) }, this.props.items.map(function (item, index) { return React.createElement(ZeroDataItem, { item: item, key: index, multiple: multiple }); })));
    };
    return ZeroDataMultiple;
}(BaseComponent));
/**
 * Represents a single item for the ZeroData component.
 */
var ZeroDataItem = /** @class */ (function (_super) {
    tslib_1.__extends(ZeroDataItem, _super);
    function ZeroDataItem() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ZeroDataItem.prototype.render = function () {
        var item = this.props.item;
        var secondary;
        if (typeof item.secondaryText === "string") {
            secondary = React.createElement("span", null, item.secondaryText);
        }
        else {
            secondary = item.secondaryText;
        }
        return (React.createElement("div", { className: css("vss-ZeroDataItem flex-column flex-center", this.props.multiple && "flex-grow") },
            React.createElement("img", { className: "vss-ZeroDataItem--image", src: item.imagePath, alt: item.imageAltText }),
            React.createElement("div", { className: css("vss-ZeroDataItem--primary", this.props.multiple ? "fontSizeLL" : "fontSizeLLL") }, item.primaryText),
            secondary && React.createElement("div", { className: "vss-ZeroDataItem--secondary fontSizeM" }, secondary),
            this._renderAction(item)));
    };
    ZeroDataItem.prototype._renderAction = function (item) {
        if (!item.actionText) {
            return null;
        }
        if (item.actionType === 0 /* ctaButton */) {
            var onActionClick = function (ev) {
                if (item.onActionClick) {
                    item.onActionClick.call(null, ev, item);
                }
            };
            var buttonProps = {
                className: "vss-ZeroDataItem--action",
                text: item.actionText,
                onClick: onActionClick,
            };
            return React.createElement(PrimaryButton, tslib_1.__assign({}, buttonProps));
        }
        else {
            return React.createElement(Link, { href: item.actionHref }, item.actionText);
        }
    };
    return ZeroDataItem;
}(BaseComponent));
