﻿// We expect Object.assign to be provided by a polyfill if it is not implemented by the browser (e.g. IE11). If it
// doesn't exist here, our browser detection has failed and we need to force polyfill loading.
if (!Object.assign && document.cookie.indexOf("TFS-FORCEPOLYFILL=true") === -1) {
    document.cookie = "TFS-FORCEPOLYFILL=true;max-age=31536000;path=/" + (window.location.protocol === "https:" ? ";secure" : "");
    document.cookie = "TFS-TRACEPOLYFILL=true;path=/" + (window.location.protocol === "https:" ? ";secure" : "");
    window.location.reload();
}

//
// Lightweight loader (LWL), no support for asynchronous modules, and modules MUST be defined
// in order to be used properly.
//
var LWL = new function() {
    var modules = {};
    var bundledModuleMap = {};
    var internalModuleNames = {};

    var jsIdentifierRegex = /[^A-Z0-9_]/ig;
    var toJsIdentifier = function(str) {
        var newStr = str.replace(jsIdentifierRegex, "");
        var firstCharCode = newStr.charCodeAt(0);
        if (48 <= firstCharCode && firstCharCode <= 57) {
            newStr = "_" + newStr;
        }
        return newStr;
    }

    var getModule = function(moduleName, requiredModule) {
        if (!modules[requiredModule]) {
            modules[requiredModule] = { exports: {}, reload: true };
            console.warn("Module '" + requiredModule + "' is being required in '" + moduleName + "' before being defined!");
        }
        return modules[requiredModule];
    };

    var normalizePath = function(moduleName, requiredModule) {
        if (requiredModule.indexOf("./") === 0 || requiredModule.indexOf("../") === 0) {
            // The required module is a relative path, so resolve it against the
            // module that we're defining.
            var stack = moduleName.split("/");
            var parts = requiredModule.split("/");
            stack.pop(); // remove current file name (or empty string)
            // (omit if "base" is the current folder without trailing slash)
            for (var i = 0; i < parts.length; i++) {
                if (parts[i] === ".") {
                    continue;
                }
                if (parts[i] === "..") {
                    stack.pop();
                } else {
                    stack.push(parts[i]);
                }
            }
            requiredModule = stack.join("/");
        }
        return requiredModule;
    };

    this.def = function(moduleName, moduleRequirements, moduleCallback, internalModules) {
        // Allow omitting moduleRequirements (react v16 does this)
        if (typeof moduleCallback === "undefined" && typeof moduleRequirements === "function") {
            moduleCallback = moduleRequirements;
            moduleRequirements = [];
        }

        // Determine whether or not a placeholder module was created ahead of time.
        var module = modules[moduleName];
        var args = [];

        // Handle modules that were defined early via a getModule call before we were defined.
        // This can happen if the module defines are not placed in dependency order.
        // This can cause execution issues if global module code access its dependencies.
        if (!module) {
            module = { exports: {} };
            modules[moduleName] = module;
        }

        if (moduleCallback) {
            var exportsArg = null;

            // Add each required module to the list of arguments.
            for (var requirementIndex = 0; requirementIndex < moduleRequirements.length; requirementIndex++) {
                var requiredModule = normalizePath(moduleName, moduleRequirements[requirementIndex]);
                if (requiredModule.indexOf("!") === -1) {
                    if (requiredModule === "require") {
                        args.push(function(requiredModule) {
                            return getModule(moduleName, requiredModule);
                        });
                    } else if (requiredModule === "exports") {
                        exportsArg = module.exports;
                        args.push(exportsArg);
                    } else if (requiredModule === "module") {
                        args.push(module);
                    } else if (modules[requiredModule]) {
                        args.push(getModule(moduleName, requiredModule).exports);
                    } else {
                        var ns = toJsIdentifier(requiredModule.substr(internalModuleNames[requiredModule].length));
                        args.push(getModule(moduleName, internalModuleNames[requiredModule]).exports[ns]);
                    }
                }
            }

            // Initialize the module with the set of required modules.
            // Handle modules that return an object literal
            var exports = moduleCallback.apply(module, args);

            // In normal situations, the callback modifies the exports argument directly. In some
            // situations, such as jQuery, the callback returns a value. If there is a return value
            // that will become the exports.
            if (exports && exports !== exportsArg) {
                if (exportsArg !== null && Object.keys(exportsArg).length > 0) {
                    console.error("Already defined exports on " + moduleName + ". Cannot re-assign new exports object.");
                }
                modules[moduleName].exports = exports;
            }

            if (internalModules) {
                for (var i = 0; i < internalModules.length; ++i) {
                    var fullName = moduleName + "/" + internalModules[i];
                    if (internalModuleNames[fullName] === undefined) {
                        internalModuleNames[fullName] = moduleName;
                    } else {
                        console.warn("Already defined " + fullName);
                    }
                }
            }
        }
    };

    this.allDependenciesDefined = function(moduleName, dependencies) {
        var dependenciesResolved = true;
        if (dependencies) {
            for (var i = 0, l = dependencies.length; i < l; i++) {
                var dep = normalizePath(moduleName, dependencies[i]);
                if (
                    !internalModuleNames[dep] &&
                    !modules[dep] &&
                    dep !== "require" &&
                    dep !== "exports" &&
                    dep !== "module" &&
                    dep.indexOf("!") === -1
                ) {
                    dependenciesResolved = false;
                    break;
                }
            }
        }
        return dependenciesResolved;
    };

    this.getDefinedModuleNames = function() {
        return Object.keys(modules).concat(Object.keys(internalModuleNames));
    };

    this.req = function(moduleName) {
        var fullModuleMatch = modules[moduleName];
        if (fullModuleMatch) {
            return fullModuleMatch.exports;
        }

        var bundledModuleName = internalModuleNames[moduleName];
        var bundledModule =  modules[bundledModuleName].exports;
        return bundledModule[toJsIdentifier(moduleName.substr(bundledModuleName.length))];
    };

    this.def.amd = {};
}();

//
// For the page AMD loader, the define and require methods will be built to support
// multiple loaders. This will allow us to continue to use the Lightweight loader
// for all new platform pages, but support other loaders, like require.js used by the
// older version of the platform.
//

var LWLS = new function() {
    this.require = LWL.req;
    this.define = LWL.def;
    this.define.amd = {};

    this.shim = function() {
        var existingDefine = window.define;
        var existingRequire = window.require;

        if (existingDefine && existingRequire) {
            var lwlModules = {};
            var modulesToRequire = [];
            var delayedRequireTimeout = 0;

            var isLWLModule = function(moduleName) {
                // Process LWWP module namespaces from the global window object. Add them to our set of prefixes
                if (window.__moduleNamespaces) {
                    var newNamespaces = window.__moduleNamespaces;
                    delete window.__moduleNamespaces;

                    for (var i = 0, l = newNamespaces.length; i < l; i++) {
                        lwlModules[newNamespaces[i].toLowerCase()] = true;
                    }
                }
                // Go through the set of LWWP module prefixes and determine if this module is contained in one of them
                var moduleNameLower = moduleName.toLowerCase();
                for (var name in lwlModules) {
                    if (
                        name === moduleNameLower ||
                        (moduleNameLower.substr(0, name.length) === name && moduleNameLower[name.length] === "/")
                    ) {
                        return true;
                    }
                }
                return false;
            };

            var processDelayedRequires = function() {
                // Issue a require statement for all queued LWWP module names
                existingRequire(modulesToRequire, function() {});
                modulesToRequire = [];
                delayedRequireTimeout = 0;
            };

            var cloneModuleDefinition = function(moduleName) {
                var definition = LWL.req(moduleName);
                existingDefine.call(this, moduleName, function() {
                    return definition;
                });
            };

            this.require = existingRequire;

            this.define = function(moduleName, moduleRequirements, moduleCallback, internalModules) {
                if (typeof moduleName === "string" && isLWLModule(moduleName)) {
                    // This is a LWWP module. If its dependencies are already resolved, just define it in-place using
                    // the LWWP loader which causes its body to be executed immediately. Otherwise queue this module
                    // into the list of modules to issue a require statement for so that we ensure its body is executed.
                    if (LWL.allDependenciesDefined(moduleName, moduleRequirements)) {
                        // Define the module immediately and give its exports to the existing loader.
                        LWL.def(moduleName, moduleRequirements, moduleCallback, internalModules);
                        if (internalModules && internalModules.length > 0) {
                            for (var i = 0; i < internalModules.length; ++i) {
                                cloneModuleDefinition(moduleName + "/" + internalModules[i]);
                            }
                        } else {
                            cloneModuleDefinition(moduleName);
                        }
                        return;
                    } else {
                        // One or more dependencies are not defined yet. Queue the module to be require'd
                        modulesToRequire.push(moduleName);

                        // Start the processing if not already started
                        if (!delayedRequireTimeout) {
                            window.setTimeout(processDelayedRequires, 0);
                        }
                    }
                }

                // Define all modules (LWWP and other) using the existing loader's define call.
                existingDefine.apply(this, arguments);
            };

            this.define.amd = existingDefine.amd || {};

            // If any modules are already defined in the LWWP loader, define them in the new loader as well.
            var moduleNames = LWL.getDefinedModuleNames();
            for (var i = 0, l = moduleNames.length; i < l; i++) {
                cloneModuleDefinition(moduleNames[i]);
            }
        }

        window.define = this.define;
        window.require = this.require;
    };
}();

LWLS.shim();
