import * as React from "react";
/**
 * Set of KeyCodes that are used in the platform.
 */
export declare const enum KeyCode {
    Tab = 9,
    Enter = 13,
    Shift = 16,
    Ctrl = 17,
    Alt = 18,
    Pause = 19,
    CapsLock = 20,
    Escape = 27,
    Space = 32,
    LeftArrow = 37,
    UpArrow = 38,
    RightArrow = 39,
    DownArrow = 40,
    WindowsKey = 91,
    NumLock = 144,
    ScrollLock = 145
}
/**
 * function that does nothing and accepts any set of arguments.
 */
export declare function noop(): void;
/**
 * Basic function for building a css classlist string from and array of classes, where
 * one of more of the arguments may be null or undefined.
 *
 * @param classes Array of strings the represents the css class list.
 *
 * @example css("base", "active", x === 42 && "optional") will return "base active optional" if x === 42 or "base active" otherwise
 */
export declare function css(...classes: Array<string | undefined | null | false>): string;
/**
 * Returns the set of parent elements with index 0 the root and the last
 * element is either the direct parent or itself based on includeSelf.
 *
 * @param element The element to get the parent element hierarchy from.
 * @param includeSelf Should the element supplied be included in the parent list.
 */
export declare function getParents(element: HTMLElement, includeSelf?: boolean): HTMLElement[];
/**
 * ElementRelationship is used to define how two elements in the same
 * document are related in position to each other.
 */
export declare const enum ElementRelationship {
    Unrelated = 0,
    Before = 1,
    After = 2,
    Child = 3,
    Parent = 4
}
/**
 * getRelationship returns the relationship of the two specified elements.
 *
 * @param element1
 * @param element2
 */
export declare function getRelationship(element1: HTMLElement, element2: HTMLElement): ElementRelationship;
/**
 * preventDefault is used as a standard delegate to prevent the default behavior
 * for a given event.
 *
 * @param event Synthetic event that should have its default action prevented.
 */
export declare function preventDefault(event: React.SyntheticEvent<HTMLElement>): void;
/**
 * The FocusContainer class is used to track whether focus exists within
 * an element and its React children. If there are elements that are not in
 * the same element hierarchy, the caller should use a React portal to
 * ensure the proper bubbling of events.
 *
 * The owner should pass an onBlur method that is called when focus leaves
 * the current logical element hierarchy.
 */
export declare class FocusContainer {
    private onBlurHost;
    private focusTimeout;
    private focusCount;
    constructor(onBlur: (event: React.FocusEvent<HTMLElement>) => void);
    /**
     * close can be called to close the container and stop monitoring focus
     * changes. This should be called if components are in the process of
     * unmounting.
     */
    close(): void;
    /**
     * isInside will tell you at this moment is the focus inside the component.
     * It may be in a transitional period between different children.
     */
    isInside(): boolean;
    /**
     * onBlur method that should be attached to the onBlur handler of the
     * continers root element.
     */
    onBlur: (event: React.FocusEvent<HTMLElement>) => void;
    /**
     * onFocus method that should be attached to the onFocus handler of the
     * continers root element.
     */
    onFocus: (event: React.FocusEvent<HTMLElement>) => void;
}
/**
 * Make sure the focus treatment is enabled and disabled based on
 * the state of mouse and keyboard usage.
 */
export declare function setFocusVisible(visible: boolean): void;
