import * as tslib_1 from "tslib";
import { ViewState, ViewStateFilter, ViewStateOptions, ViewStateObservableValue } from './ViewState';
/**
 * The keys for common view options known by the Hub component
 */
export var HubViewOptionKeys;
(function (HubViewOptionKeys) {
    /**
     * View option controlling whether or not the hub filter bar is being displayed.
     */
    HubViewOptionKeys.showFilterBar = 'showFilters';
    /**
     * View option controlling whether or not the page is in full-screen mode
     */
    HubViewOptionKeys.fullScreen = 'fullScreen';
})(HubViewOptionKeys || (HubViewOptionKeys = {}));
/**
 * Names of observable events triggered by hub view state
 */
export var HubViewStateEventNames;
(function (HubViewStateEventNames) {
    /**
     * Event fired when the pivot is being changed. Consumers can listen for this
     * event and modify other view state values. This event is not fired when
     * the selected pivot value is changed during the processing of another view-state's event.
     */
    HubViewStateEventNames.pivotChanging = 'pivotChanging';
    /**
     * Event fired when the applied filter is being changed. Consumers can listen for this
     * event and modify other view state values. This event is not fired when
     * the filter is changed during the processing of another view-state's event.
     */
    HubViewStateEventNames.filterChanging = 'filterChanging';
    /**
     * Event fired when a view option value is being changed. Consumers can listen for this
     * event and modify other view state values. This event is not fired when
     * a view option value is changed during the processing of another view-state's event.
     */
    HubViewStateEventNames.viewOptionsChanging = 'viewOptionsChanging';
})(HubViewStateEventNames || (HubViewStateEventNames = {}));
/**
 * Store for the current view of a hub (selected pivot, filters, view options, etc.)
 */
var HubViewState = /** @class */ (function (_super) {
    tslib_1.__extends(HubViewState, _super);
    function HubViewState(options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this) || this;
        _this._viewOptions = new ViewStateOptions(_this.notify.bind(_this), HubViewStateEventNames.viewOptionsChanging, options.viewOptions);
        _this._filter = new ViewStateFilter(_this.notify.bind(_this), HubViewStateEventNames.filterChanging, options.filterOptions);
        _this._selectedPivot = new ViewStateObservableValue(_this.notify.bind(_this), HubViewStateEventNames.pivotChanging, options.defaultPivot);
        return _this;
    }
    Object.defineProperty(HubViewState.prototype, "selectedPivot", {
        get: function () {
            return this._selectedPivot;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HubViewState.prototype, "filter", {
        get: function () {
            return this._filter;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HubViewState.prototype, "viewOptions", {
        get: function () {
            return this._viewOptions;
        },
        enumerable: true,
        configurable: true
    });
    HubViewState.prototype.canSwitchPivots = function (targetPivot) {
        return true;
    };
    return HubViewState;
}(ViewState));
export { HubViewState };
