/**
 * Checks two arrays to see they contain equal elements in the same order.
 *
 * @param array1 First array to check.
 * @param array2 Second array to check.
 * @param comparer Optional comparer to check whether array items are equal. If not specified, items are compared using strict equals.
 * @returns {boolean}
 */
export function arrayEquals(array1, array2, comparer) {
    if (comparer === void 0) { comparer = function (item1, item2) { return item1 === item2; }; }
    if (!array1 || !array2) {
        return false;
    }
    if (array1.length !== array2.length) {
        return false;
    }
    for (var i = 0; i < array1.length; i++) {
        if (!comparer(array1[i], array2[i])) {
            return false;
        }
    }
    return true;
}
/**
 * Gets the closest ancestor of the element (or the element itself) using a query selector.
 *
 * @param element Starting element.
 * @param query Query to locate.
 * @returns {Element}
 */
export function closest(element, query) {
    if (element && element.closest) {
        return element.closest(query);
    }
    // for IE11 and below
    while (element && !(element.matches || element.msMatchesSelector).call(element, query)) {
        element = element.parentElement;
    }
    return element;
}
