import { ObservableArray } from '../../Core/Observable';
import { IItemProvider, IItemWithIdProvider } from "./interfaces";
/**
 * Given an IItemProvider, implements an ObservableArray that contains the items provided by the provider, updates existing items if exists using identifier.
 */
export declare class ContributedItemWithIdArray<TContributedItem, TItem> extends ObservableArray<TItem> {
    constructor(itemProvider: IItemWithIdProvider<TContributedItem>, mapDelegate: (item: TContributedItem) => TItem, getId: (item: TItem) => string);
}
/**
 * Given an IItemProvider, implements an ObservableArray that contains the items provided by the provider, replaces existing items from provider.
 */
export declare class ContributedItemArray<TContributedItem, TItem> extends ObservableArray<TItem> {
    constructor(itemProvider: IItemProvider<TContributedItem>, mapDelegate: (item: TContributedItem) => TItem);
}
/**
 * Implements an IItemProvider that provides modified versions of items it gets from the source provider.
 */
export declare class MutatedItemProvider<TInItem, TOutItem = TInItem> implements IItemProvider<TOutItem> {
    private provider;
    private mutator;
    /**
     * @param provider provider that provides the source items
     * @param mutator function that modifies the items before providing them to this object's listener
     */
    constructor(provider: IItemProvider<TInItem>, mutator: (item: TInItem) => TOutItem);
    loadItems(itemsUpdated: (items: TOutItem[]) => void): void;
}
export declare class PromiseWrappingItemProvider<TItem> implements IItemProvider<Promise<TItem[]>> {
    private provider;
    constructor(provider: IItemProvider<TItem>);
    loadItems(itemsUpdated: (items: Promise<TItem[]>[]) => void): void;
    getItems(): Promise<TItem[]>;
}
export declare class ObservableArrayWrappingItemProvider<TItem> implements IItemProvider<ObservableArray<TItem>> {
    private provider;
    constructor(provider: IItemProvider<TItem>);
    loadItems(itemsUpdated: (items: ObservableArray<TItem>[]) => void): void;
    getItems(): ObservableArray<TItem>;
}
