import * as tslib_1 from "tslib";
import { ObservableArray } from '../../Core/Observable';
/**
 * Given an IItemProvider, implements an ObservableArray that contains the items provided by the provider, updates existing items if exists using identifier.
 */
var ContributedItemWithIdArray = /** @class */ (function (_super) {
    tslib_1.__extends(ContributedItemWithIdArray, _super);
    function ContributedItemWithIdArray(itemProvider, mapDelegate, getId) {
        var _this = _super.call(this, []) || this;
        itemProvider.loadItems(function (items, removedItems) {
            // create map of existing items
            var existingItemsMap = {};
            removedItems = removedItems || [];
            _this.value.forEach(function (item) {
                var id = getId(item);
                if (removedItems.indexOf(id) === -1) {
                    existingItemsMap[id] = item;
                }
            });
            // update existing items
            (items || []).map(mapDelegate).forEach(function (item) {
                existingItemsMap[getId(item)] = item;
            });
            // populate all new items
            var newItems = [];
            Object.keys(existingItemsMap).forEach(function (key) {
                newItems.push(existingItemsMap[key]);
            });
            _this.value = newItems;
        });
        return _this;
    }
    return ContributedItemWithIdArray;
}(ObservableArray));
export { ContributedItemWithIdArray };
/**
 * Given an IItemProvider, implements an ObservableArray that contains the items provided by the provider, replaces existing items from provider.
 */
var ContributedItemArray = /** @class */ (function (_super) {
    tslib_1.__extends(ContributedItemArray, _super);
    function ContributedItemArray(itemProvider, mapDelegate) {
        var _this = _super.call(this, []) || this;
        itemProvider.loadItems(function (items) {
            // When contributed items become available, update observable array
            _this.value = (items || []).map(mapDelegate);
        });
        return _this;
    }
    return ContributedItemArray;
}(ObservableArray));
export { ContributedItemArray };
/**
 * Implements an IItemProvider that provides modified versions of items it gets from the source provider.
 */
var MutatedItemProvider = /** @class */ (function () {
    /**
     * @param provider provider that provides the source items
     * @param mutator function that modifies the items before providing them to this object's listener
     */
    function MutatedItemProvider(provider, mutator) {
        this.provider = provider;
        this.mutator = mutator;
    }
    MutatedItemProvider.prototype.loadItems = function (itemsUpdated) {
        var _this = this;
        this.provider.loadItems(function (items) { return itemsUpdated((items || []).map(_this.mutator)); });
    };
    return MutatedItemProvider;
}());
export { MutatedItemProvider };
var PromiseWrappingItemProvider = /** @class */ (function () {
    function PromiseWrappingItemProvider(provider) {
        this.provider = provider;
    }
    PromiseWrappingItemProvider.prototype.loadItems = function (itemsUpdated) {
        if (itemsUpdated)
            itemsUpdated([this.getItems()]);
    };
    PromiseWrappingItemProvider.prototype.getItems = function () {
        var promise;
        this.provider.loadItems(function (items) {
            promise = new Promise(function (resolve) { return resolve(items); });
        });
        return promise;
    };
    return PromiseWrappingItemProvider;
}());
export { PromiseWrappingItemProvider };
var ObservableArrayWrappingItemProvider = /** @class */ (function () {
    function ObservableArrayWrappingItemProvider(provider) {
        this.provider = provider;
    }
    ObservableArrayWrappingItemProvider.prototype.loadItems = function (itemsUpdated) {
        if (itemsUpdated)
            itemsUpdated([this.getItems()]);
    };
    ObservableArrayWrappingItemProvider.prototype.getItems = function () {
        var observableArray;
        this.provider.loadItems(function (items) {
            observableArray = new ObservableArray(items);
        });
        return observableArray;
    };
    return ObservableArrayWrappingItemProvider;
}());
export { ObservableArrayWrappingItemProvider };
