import * as tslib_1 from "tslib";
import { EventGroup } from 'office-ui-fabric-react/lib/Utilities';
import { Observable } from '../../Core/Observable';
export var VIEW_OPTIONS_CHANGE_EVENT = 'view-option-changed';
var ViewOptions = /** @class */ (function () {
    function ViewOptions(props, observable) {
        if (props === void 0) { props = {}; }
        this.observable = observable || new Observable();
        this.currentState = tslib_1.__assign({}, props.initialState);
    }
    ViewOptions.prototype.subscribe = function (observer, action) {
        this.observable.subscribe(observer, action);
    };
    ViewOptions.prototype.unsubscribe = function (observer, action) {
        this.observable.unsubscribe(observer, action);
    };
    /**
     * Gets a dictionary containing the current values of all view options.
     */
    ViewOptions.prototype.getViewOptions = function () {
        return tslib_1.__assign({}, this.currentState);
    };
    /**
     * Sets the current values of all view options.
     * @param state Dictionary of all current view option values.
     * @param supressChangeEvent If true, don't invoke the onViewOptionsChanged callback at this time
     */
    ViewOptions.prototype.setViewOptions = function (state, supressChangeEvent) {
        var prevState = this.currentState;
        this.currentState = tslib_1.__assign({}, state);
        if (!supressChangeEvent) {
            var changedState = tslib_1.__assign({}, state);
            for (var key in prevState) {
                // Add current view options that are cleared by the new state
                // (i.e. these changed from "something" to "nothing")
                if (!state.hasOwnProperty(key)) {
                    changedState[key] = null;
                }
            }
            this._triggerStateChange(changedState);
        }
    };
    /**
     * Clears all view option values
     */
    ViewOptions.prototype.clear = function () {
        this.setViewOptions({});
    };
    /**
     * Gets the value of the view option.
     * @param key View option item key.
     */
    ViewOptions.prototype.getViewOption = function (key) {
        return this.currentState[key];
    };
    /**
     * Sets the current value of the specified option.
     * @param key Key of the view option.
     * @param supressChangeEvent If true, don't invoke the onViewOptionsChanged callback at this time
     */
    ViewOptions.prototype.setViewOption = function (key, value) {
        this.currentState[key] = value;
        var changedState = {};
        changedState[key] = value;
        this._triggerStateChange(changedState);
    };
    ViewOptions.prototype._triggerStateChange = function (changedState) {
        EventGroup.raise(this, VIEW_OPTIONS_CHANGE_EVENT, changedState);
        this.observable.notify(changedState, VIEW_OPTIONS_CHANGE_EVENT);
    };
    return ViewOptions;
}());
export { ViewOptions };
